@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -149,7 +149,7 @@ void SIG_init()
 		    sg->irq = irq;
 		    g_printf("SIG: IRQ%d, enabling PIC-level %ld\n", irq, pic_irq_list[irq]);
 		    { extern int SillyG_do_irq(void);
-		    pic_seti(pic_irq_list[irq], SillyG_do_irq, 0);
+		    pic_seti(pic_irq_list[irq], SillyG_do_irq, 0, NULL);
 		    }
 		    pic_unmaski(pic_irq_list[irq]);
 		    sg++;
@@ -212,7 +212,7 @@ signal_init(void)
    SIGUSR1		10	?	threads
    SIGSEGV		11	N	dosemu_fault
    SIGUSR2		12	NQ	(SIG_ACQUIRE)
-   SIGPIPE		13
+   SIGPIPE		13      S	SIG_IGN
    SIGALRM		14	NQ	(SIG_TIME)sigalrm
    SIGTERM		15	S	leavedos
    SIGSTKFLT		16
@@ -249,6 +249,7 @@ signal_init(void)
   SETSIG(SIGKILL, leavedos);
 #endif
   SETSIG(SIGQUIT, sigquit);
+  SETSIG(SIGPIPE, SIG_IGN);
 
 #ifdef X_SUPPORT
   if(config.X) {
@@ -323,7 +324,7 @@ sti(void)
 void handle_signals(void) {
   if ( SIGNAL_head != SIGNAL_tail ) {
 #ifdef X86_EMULATOR
-    if ((config.cpuemu>1) && (d.emu>3))
+    if ((config.cpuemu>1) && (debug_level('e')>3))
       {e_printf("EMU86: SIGNAL at %d\n",SIGNAL_head);}
 #endif
     signal_pending = 0;
@@ -334,9 +335,6 @@ void handle_signals(void) {
  * by the kernel ASAP.
  */
       if (SIGNAL_head != SIGNAL_tail) {
-#ifdef X86_EMULATOR
-	if (config.cpuemu>1) CEmuStat|=CeS_SIGPEND;
-#endif
 	signal_pending = 1;
 	if (in_dpmi)
 	  dpmi_eflags |= VIP;
@@ -410,7 +408,8 @@ void SIGALRM_call(void)
 #ifdef USING_NET
   /* check for available packets on the packet driver interface */
   /* (timeout=0, so it immediately returns when none are available) */
-  pkt_check_receive(0);
+  if (config.pktdrv)
+    pic_request(PIC_NET);
 #endif
 #endif
 
@@ -566,9 +565,6 @@ void SIGALRM_call(void)
 inline void SIGNAL_save( void (*signal_call)() ) {
   signal_queue[SIGNAL_tail].signal_handler=signal_call;
   SIGNAL_tail = (SIGNAL_tail + 1) % MAX_SIG_QUEUE_SIZE;
-#ifdef X86_EMULATOR
-  if (config.cpuemu>1) CEmuStat|=CeS_SIGPEND;
-#endif
   signal_pending = 1;
   if (in_dpmi)
     dpmi_eflags |= VIP;
@@ -630,18 +626,14 @@ void
 sigquit(int sig)
 {
   in_vm86 = 0;
-  in_sighandler = 1;
 
   error("sigquit called\n");
   show_ints(0, 0x33);
   show_regs(__FILE__, __LINE__);
 
-  ignore_segv++;
   *(unsigned char *)BIOS_KEYBOARD_FLAGS = 0x80;	/* ctrl-break flag */
-  ignore_segv--;
 
   do_soft_int(0x1b);
-  in_sighandler = 0;
 }
 
 #if 0
