@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -130,7 +130,7 @@ static unsigned int linmode = 0;
 static unsigned int codeorg = 0;
 
 static unsigned int dpmimode=1, saved_dpmimode=1;
-#define IN_DPMI  (in_dpmi && dpmimode)
+#define IN_DPMI  (in_dpmi && !in_dpmi_dos_int && dpmimode)
 
 static char lastd[32];
 static char lastu[32];
@@ -542,6 +542,7 @@ static void mhp_go(int argc, char * argv[])
       WRITE_FLAGS(READ_FLAGS() & ~TF);
       vm86s.vm86plus.vm86dbg_TFpendig=0;
       mhpdbgc.stopped = 0;
+      pic_sti();
    }
 }
 
@@ -568,10 +569,10 @@ static void mhp_trace(int argc, char * argv[])
       mhp_printf("must be in stopped state\n");
    } else {
       mhpdbgc.stopped = 0;
-      if (IN_DPMI) {
-        if (!dpmi_mhp_setTF(1)) return;
+      if (in_dpmi) {
+        dpmi_mhp_setTF(1);
       }
-      else WRITE_FLAGS(READ_FLAGS() | TF);
+      WRITE_FLAGS(READ_FLAGS() | TF);
       mhpdbgc.trapcmd = 1;
    }
 }
@@ -593,12 +594,14 @@ static void mhp_trace_force(int argc, char * argv[])
       mhp_printf("must be in stopped state\n");
    } else {
       mhpdbgc.stopped = 0;
-      if (IN_DPMI) {
-        if (!dpmi_mhp_setTF(1)) return;
+      if (in_dpmi) {
+        dpmi_mhp_setTF(1);
       }
-      else WRITE_FLAGS(READ_FLAGS() | TF);
+      WRITE_FLAGS(READ_FLAGS() | TF);
       vm86s.vm86plus.vm86dbg_TFpendig=1;
       mhpdbgc.trapcmd = 1;
+      /* disable PIC: we want to trace the program, not the HW int handlers */
+      pic_cli();
    }
 }
 
