@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -8,10 +8,14 @@
   event handling: read parameters and put event in driver queue
  ***********************************************************************/
 
-#include"events.h"
-#include"io.h"
-#include"stats.h"
-#include"emulation.h"
+#include "events.h"
+#include "midid.h"
+#include "io.h"
+#include "stats.h"
+#include "emulation.h"
+#include "device.h"
+#include <stdio.h>
+#include <linux/soundcard.h>
 
 #define EOX 0xf7
 
@@ -26,7 +30,7 @@ void do_noteoff(int chn)
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "note_off(chn=%i,note=%i,vel=%i)\n", chn, note, vel);
-	dev->noteoff(chn, note, vel);
+	dev_noteoff(chn, note, vel);
 }
 
 void do_noteon(int chn)
@@ -40,12 +44,7 @@ void do_noteon(int chn)
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "note_on(chn=%i,note=%i,vel=%i)\n", chn, note, vel);
-	if (vel) {
-		dev->noteon(chn, note, vel);
-	} else {
-		/* No velocity */
-		dev->noteoff(chn, note, vel);
-	}
+	dev_noteon(chn, note, vel);
 }
 
 void do_notepressure(int chn)
@@ -60,7 +59,7 @@ void do_notepressure(int chn)
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "note_pressure(chn=%i,note=%i,vel=%i)\n", chn, note, vel);
-	dev->notepressure(chn, note, vel);
+	dev_notepressure(chn, note, vel);
 }
 
 void do_program(int chn)
@@ -75,7 +74,7 @@ void do_program(int chn)
 	if (debug)
 		fprintf(stderr, "change_program(chn=%i,pgm=%i)\n", chn, pgm);
 	/* Use mapping for MT32 <-> GM emulation (imap[]) */
-	dev->program(chn, imap[pgm]);
+	dev_program(chn, imap[pgm]);
 }
 
 void do_channelpressure(int chn)
@@ -87,7 +86,7 @@ void do_channelpressure(int chn)
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "control_change(chn=%i,vel=%i)\n", chn, vel);
-	dev->channelpressure(chn, vel);
+	dev_channelpressure(chn, vel);
 }
 
 void do_bender(int chn)
@@ -102,76 +101,32 @@ void do_bender(int chn)
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "bender(chn=%i,pitch=%i)\n", chn, pitch);
-	dev->bender(chn, pitch);
+	dev_bender(chn, pitch);
 }
 
 void do_sysex(void)
 {
-	int ch; /* Last read char */
-	ch = getbyte_data();
-	if (ch<0) return;
-	getbyte_next();
+#define SYSEX_BUF_LEN 1024
+char buf[SYSEX_BUF_LEN];
+int ch, len = 0;
+	buf[len++] = MIDI_SYSTEM_PREFIX;
+	ch = getbyte();
 	if (debug)
-		fprintf(stderr,"Sysex for vendor ID#%i\n",ch);
+		fprintf(stderr,"Sysex for vendor ID#%x\n",ch);
 	while(1) {
-		ch = getbyte_data();
-		if (ch<0) break;
+		buf[len++] = ch;
 		getbyte_next();
-	}
-	if (getbyte()==EOX) getbyte_next();
-	else
+		if (ch == EOX) {
+			if (debug)
+				fprintf(stderr,"End of SysEx, len=%i\n", len);
+			if (imap == imap_default)
+				dev_sysex(buf, len);
+			return;
+		}
+		ch = getbyte();
+		if (ch<0 || len >= SYSEX_BUF_LEN) return;
 		if (debug)
-			fprintf(stderr,"Sysex event wasn't terminated by EOX\n");
-}
-
-void do_allnotesoff(void)
-{
-	/* NYI */
-	if (warning)
-		fprintf(stderr,"do_allnotesoff NYI\n");
-}
-
-void do_modemessage(int chn,int control)
-{
-	int value;
-	if (debug)
-		fprintf(stderr,"mode message (chn=%i,control=%i)\n",
-                        chn,control);
-	switch(control) {
-	case 122:
-		value=getbyte_data();
-		if (value<0) return;
-		getbyte_next();
-		if (ignored)
-			fprintf(stderr,
-                                "Ignored: Local control (chn=%i,val=%i\n",
-                                chn,value);
-		break;
-	case 123:
-		/* all notes off */
-		do_allnotesoff();
-		break;
-	case 124:
-		if (ignored) fprintf(stderr,"Ignored: omni mode off\n");
-		do_allnotesoff();
-		break;
-	case 125:
-		if (ignored) fprintf(stderr,"Ignored: omni mode on\n");
-		do_allnotesoff();
-		break;
-	case 126:
-		value = getbyte_data();
-		if (value<0) return;
-		getbyte_next();
-		if (ignored) fprintf(stderr,"Ignored: monophonic mode, %i channels\n",value);
-		do_allnotesoff();
-		break;
-	case 127:
-		if (ignored) fprintf(stderr,"Ignored: polyphonic mode\n");
-		do_allnotesoff();
-		break;
-	default:
-		assert(FALSE);
+			fprintf(stderr,"Sysex: %x\n",ch);
 	}
 }
 
@@ -182,14 +137,12 @@ void do_controlchange(int chn)
 	control = getbyte_data();
 	if (control<0) return;
 	getbyte_next();
-	if ((control>=122)&&(control<=127)) do_modemessage(chn,control);
-	/* Unhandled controller; let the driver decide */
 	value = getbyte();
 	if (value<0) return;
 	getbyte_next();
 	if (debug)
 		fprintf(stderr, "control_change(chn=%i,control=%i,val=%i)\n",
                         chn, control, value);
-	dev->control(chn, control, value);
+	dev_control(chn, control, value);
 }
 
