@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,6 +27,7 @@
 #include "disks.h"
 #include "bios.h"
 #include "iodev.h"
+#include "lpt.h"
 #include "bitops.h"
 #include "xms.h"
 #include "int.h"
@@ -36,10 +37,16 @@
 #include "priv.h"
 #include "doshelpers.h"
 #include "utilities.h"
+#include "redirect.h"
+
+#include "joystick.h"
 
 #ifdef USING_NET
 #include "ipx.h"
 #endif
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 
 #include "dpmi.h"
 
@@ -59,7 +66,9 @@ extern void pci_bios(void);
    This flag will be set when doing video routines so that special
    access can be given
 */
+#if 0
 static u_char         in_video = 0;
+#endif
 static u_char         save_hi_ints[128];
 
 static int            card_init = 0;
@@ -70,29 +79,20 @@ static struct timeval scr_tv;        /* For translating UNIX <-> DOS times */
 /* set if some directories are mounted during startup */
 int redir_state = 0;
 
-#ifdef USE_MRP_JOYSTICK
-#error "FIXME: joystick support is broken. Sorry"
-#include "Linux/joystick.h"
-#include <fcntl.h>
-#include <string.h>
-static void mrp_read_joystick(void);
-#endif
-
 void kill_time(long usecs) {
-   hitimer_t t_start;
-   long t_dif;
+   hitimer_t t_start, t_dif;
    scr_tv.tv_sec = 0L;
    scr_tv.tv_usec = usecs;
 
    t_start = GETusTIME(0);
    while ((int) select(STDIN_FILENO, NULL, NULL, NULL, &scr_tv) < (int) 1)
      {
-	t_dif = (long)(GETusTIME(0)-t_start);
+       t_dif = GETusTIME(0)-t_start;
 
         if ((t_dif >= usecs) || (errno != EINTR))
           return ;
         scr_tv.tv_sec = 0L;
-        scr_tv.tv_usec = usecs - t_dif;
+        scr_tv.tv_usec = usecs - (long)t_dif;
      }
 }
 
@@ -110,11 +110,7 @@ static void default_interrupt(u_char i) {
   di_printf("int 0x%02x, ax=0x%04x\n", i, LWORD(eax));
 
   if (!IS_REDIRECTED(i) || (!IVEC(i)) ||
-#ifndef USE_NEW_INT
-      (LWORD(cs) == BIOSSEG && LWORD(eip) == (i * 16 + 2))) {
-#else /* USE_NEW_INT */
       ((SEGOFF2LINEAR(BIOSSEG, INT_OFF(i)) +2) == SEGOFF2LINEAR(_CS, _IP))) {
-#endif /* USE_NEW_INT */
     g_printf("DEFIVEC: int 0x%02x @ 0x%04x:0x%04x\n", i, ISEG(i), IOFF(i));
 
     /* This is here for old SIGILL's that modify IP */
@@ -197,11 +193,6 @@ static void process_master_boot_record(void)
 /* returns 1 if dos_helper() handles it, 0 otherwise */
 static int dos_helper(void)
 {
-#ifdef X86_EMULATOR
-  extern void enter_cpu_emu(void);
-  extern void leave_cpu_emu(void);
-#endif
-
   switch (LO(ax)) {
   case DOS_HELPER_DOSEMU_CHECK:			/* Linux dosemu installation test */
     LWORD(eax) = 0xaa55;
@@ -256,11 +247,7 @@ static int dos_helper(void)
     if (LO(bx) == 0) {
       if (set_ioperm(0x3b0, 0x3db - 0x3b0, 0))
 	warn("couldn't shut off ioperms\n");
-#ifndef USE_NEW_INT
-      SETIVEC(0x10, BIOSSEG, 0x10 * 0x10);	/* restore our old vector */
-#else /* USE_NEW_INT */
       SETIVEC(0x10, BIOSSEG, INT_OFF(0x10));	/* restore our old vector */
-#endif /* USE_NEW_INT */
       config.vga = 0;
     } else {
       unsigned char *ssp;
@@ -292,11 +279,12 @@ static int dos_helper(void)
   case DOS_HELPER_SHOW_BANNER:		/* show banner */
     p_dos_str("\n\nLinux DOS emulator " VERSTR " $Date: " VERDATE " $\n");
     p_dos_str("Last configured at %s on %s\n", CONFIG_TIME, CONFIG_HOST);
-#if 0
+#if 1 
     p_dos_str("This is work in progress.\n");
     p_dos_str("Please test against a recent version before reporting bugs and problems.\n");
     /* p_dos_str("Formerly maintained by Robert Sanders, gt8134b@prism.gatech.edu\n\n"); */
-    p_dos_str("Bugs, Patches & New Code to linux-msdos@vger.kernel.org\n\n");
+    p_dos_str("Submit Bug Reports, Patches & New Code to linux-msdos@vger.kernel.org or via\n");
+    p_dos_str("the SourceForge tracking system at http://www.sourceforge.net/projects/dosemu\n\n");
 #endif
     if (config.dpmi)
       p_dos_str("DPMI-Server Version 0.9 installed\n\n");
@@ -322,6 +310,7 @@ static int dos_helper(void)
      * provided they were not previously enabled by SPKR_NATIVE - AV
      * DANG_END_REMARK
      */
+#if 0
     if (config.allowvideoportaccess) {
       if (config.speaker != SPKR_NATIVE) {
 	v_printf("Giving temporary access to PIT#2\n");
@@ -330,6 +319,7 @@ static int dos_helper(void)
       }
       in_video = 1;
     }
+#endif
     /* DANG_BEGIN_REMARK
      * Many video BIOSes use hi interrupt vector locations as
      * scratchpad area - this is because they come before DOS and feel
@@ -345,6 +335,7 @@ static int dos_helper(void)
 
   case DOS_HELPER_VIDEO_INIT_DONE:
     v_printf("Finished with Video initialization\n");
+#if 0
     if (config.allowvideoportaccess) {
       if (config.speaker != SPKR_NATIVE) {
         v_printf("Removing temporary access to PIT#2\n");
@@ -353,6 +344,7 @@ static int dos_helper(void)
       }
       in_video = 0;
     }
+#endif
     v_printf("Restore hi vector area\n");
     MEMCPY_2DOS(0x380,save_hi_ints,128);
     config.emuretrace <<= 1;
@@ -440,6 +432,10 @@ static int dos_helper(void)
     mouse_helper();
     break;
 
+  case DOS_HELPER_PAUSE_KEY:
+    pic_set_callback(Pause_SEG, Pause_OFF);
+    break;
+
   case DOS_HELPER_CDROM_HELPER:{
       E_printf("CDROM: in 0x40 handler! ax=0x%04x, bx=0x%04x, dx=0x%04x, "
 	       "cx=0x%04x\n", LWORD(eax), LWORD(ebx), LWORD(edx), LWORD(ecx));
@@ -524,17 +520,23 @@ static int dos_helper(void)
     case DOS_HELPER_GETCWD:
         LWORD(eax) = (short)((int)getcwd(SEG_ADR((char *), es, dx), (size_t)LWORD(ecx)));
         break;
+
+    case DOS_HELPER_GETPID:
+	LWORD(eax) = getpid();
+	LWORD(ebx) = getppid();
+	break;
+
   case DOS_HELPER_CHDIR:
         LWORD(eax) = chdir(SEG_ADR((char *), es, dx));
         break;
 #ifdef X86_EMULATOR
   case DOS_HELPER_CPUEMUON:
 #ifdef TRACE_DPMI
-	if (d.dpmit==0)
+	if (debug_level('t')==0)
 #endif
 	{
 #ifdef DONT_DEBUG_BOOT
-	    memcpy(&d,&d_save,sizeof(struct debug_flags));
+	    memcpy(&debug,&debug_save,sizeof(debug));
 #endif
 	    /* we could also enter from inside dpmi, provided we already
 	     * mirrored the LDT into the emu's own one */
@@ -544,7 +546,7 @@ static int dos_helper(void)
   case DOS_HELPER_CPUEMUOFF:
 	if ((config.cpuemu>1) 
 #ifdef TRACE_DPMI
-	&& (d.dpmit==0) 
+	&& (debug_level('t')==0) 
 #endif
 	&& !in_dpmi)
 	    leave_cpu_emu();
@@ -592,16 +594,6 @@ static int dos_helper(void)
   return 1;
 }
 
-#ifndef USE_NEW_INT
-static void int08(u_char i)
-{
-  real_run_int(0x1c);
-  /* REG(eflags) |= VIF; */
-  WRITE_FLAGSE(READ_FLAGSE() | VIF);
-  return;
-}
-#endif /* not USE_NEW_INT */
-
 static void int15(u_char i)
 {
   int num;
@@ -616,10 +608,16 @@ static void int15(u_char i)
   case 0x10:			/* TopView/DESQview */
     switch (LO(ax))
     {
-    case 0x00:			/* giveup timeslice */
-      if (config.hogthreshold)
-        usleep(INT15_IDLE_USECS);
-      return;
+      case 0x00: {		/* giveup timeslice */
+        static int trigger = 0;
+        if (config.hogthreshold) {
+          if (trigger++ >= (config.hogthreshold - 1) * 100) {
+            usleep(INT15_IDLE_USECS);
+	    trigger = 0;
+	  }
+        }
+        return;
+      }
     }
     CARRY;
     break;
@@ -650,14 +648,8 @@ static void int15(u_char i)
     CARRY;
     return;			/* no event wait */
   case 0x84:
-#ifdef USE_MRP_JOYSTICK
-    mrp_read_joystick ();
-    h_printf ("int 15h joystick int: %d %d\n", (int)(LWORD(eax)), (int)(LWORD(ebx)));
+  	joy_bios_read ();
     return;
-#else
-    CARRY;
-    return;			/* no joystick */
-#endif
   case 0x85:
     num = LWORD(eax) & 0xFF;	/* default bios handler for sysreq key */
     if (num == 0 || num == 1) {
@@ -833,6 +825,27 @@ static void int15(u_char i)
 	break;
 
   case 0xe8:
+#if 0
+--------b-15E801-----------------------------
+INT 15 - Phoenix BIOS v4.0 - GET MEMORY SIZE FOR >64M CONFIGURATIONS
+	AX = E801h
+Return: CF clear if successful
+	    AX = extended memory between 1M and 16M, in K (max 3C00h = 15MB)
+	    BX = extended memory above 16M, in 64K blocks
+	    CX = configured memory 1M to 16M, in K
+	    DX = configured memory above 16M, in 64K blocks
+	CF set on error
+Notes:	supported by the A03 level (6/14/94) and later XPS P90 BIOSes, as well
+	  as the Compaq Contura, 3/8/93 DESKPRO/i, and 7/26/93 LTE Lite 386 ROM
+	  BIOS
+	supported by AMI BIOSes dated 8/23/94 or later
+	on some systems, the BIOS returns AX=BX=0000h; in this case, use CX
+	  and DX instead of AX and BX
+	this interface is used by Windows NT 3.1, OS/2 v2.11/2.20, and is
+	  used as a fall-back by newer versions if AX=E820h is not supported
+SeeAlso: AH=8Ah"Phoenix",AX=E802h,AX=E820h,AX=E881h"Phoenix"
+---------------------------------------------
+#endif
     if (LO(ax) == 1) {
 	Bit32u mem = ((config.xms_size > config.ems_size) ?
 			 config.xms_size : config.ems_size);
@@ -855,11 +868,9 @@ void set_ticks(unsigned long new)
   volatile unsigned long *ticks = BIOS_TICK_ADDR;
   volatile unsigned char *overflow = TICK_OVERFLOW_ADDR;
 
-  ignore_segv++;
   *ticks = new;
   /* A timer read should reset the overflow flag */
   *overflow = 0;
-  ignore_segv--;
   h_printf("TICKS: update ticks to %ld\n", new);
 }
 
@@ -870,6 +881,10 @@ static void int1a(u_char i)
   struct timezone;
   struct tm *tm;
 
+#ifdef X86_EMULATOR
+  int tmp = E_MUNPROT_STACK(0);		/* no faults in BIOS area! */
+#endif
+
   switch (HI(ax)) {
 
 /*
@@ -944,7 +959,7 @@ Notes:	there are approximately 18.2 clock ticks per second, 1800B0h per 24 hrs
     LWORD(edx) = last_ticks & 0xffff;
 
 #ifdef DEBUG_INT1A
-    if (d.general) {
+    if (debug_level('g')) {
       long k = last_ticks/18.2065;	/* sorry */
       time(&time_val);
       g_printf("INT1A: read timer = %ld (%ld:%ld:%ld) %s\n", last_ticks,
@@ -1133,8 +1148,12 @@ Return: nothing
   default:
     g_printf("WARNING: unsupported INT0x1a call 0x%02x\n", HI(ax));
     CARRY;
-    return;
   }
+
+#ifdef X86_EMULATOR
+  if (tmp) E_MPROT_STACK(0);
+#endif
+
 }
 
 /* ========================================================================= */
@@ -1165,6 +1184,7 @@ static int ms_dos(int nr)
 {
   switch (nr) {
   case 0x3d:       /* DOS handle open */
+  case 0x6c:
 #ifdef INTERNAL_EMS
     if (config.ems_size && !strncmp(ptr, "EMMXXXX0", 8)) {
       E_printf("EMS: opened EMM file!\n");
@@ -1212,17 +1232,28 @@ static int ms_dos(int nr)
   return 0;
 #endif
 
-  case 0x2C:                    /* get time & date */
-    if (config.hogthreshold)    /* allow linux to idle */
-      usleep(INT2F_IDLE_USECS);
+  case 0x2C: {                   /* get time & date */
+      static int trigger = 0;
+      if (config.hogthreshold) {
+        if (trigger++ >= (config.hogthreshold - 1) * 100) {
+          usleep(INT2F_IDLE_USECS);
+	  trigger = 0;
+	}
+      }
+      return 0;
+    }
+
+  case 0x4B:			/* program load */
+    if(LO(ax) != 0x0)
+      return 0;			/* else as for 0x4c */
+  case 0x4C:                    /* program exit */
+    if(pic_icount) {
+      pic_resched();
+      pic_icount=0;
+    }
     return 0;
 
   default:
-#ifndef USE_NEW_INT
-    if (!in_dpmi)
-      g_printf("INT21 (0x%02x):  we shouldn't be here! ax=0x%04x, bx=0x%04x\n",
-	     nr, LWORD(eax), LWORD(ebx));
-#endif
     return 0;
   }
   return 1;
@@ -1232,34 +1263,29 @@ static int ms_dos(int nr)
 
 void real_run_int(int i)
 {
+#ifdef X86_EMULATOR
+  int tmp;
+  unsigned char *tmp_ssp;
+#endif
   unsigned char *ssp;
   unsigned long sp;
 
-#ifndef USE_NEW_INT
-  /* ssp = (unsigned char *)(REG(ss)<<4); */
-  ssp = (unsigned char *)(READ_SEG_REG(ss)<<4);
-  sp = (unsigned long) LWORD(esp);
-
-  pushw(ssp, sp, vflags);
-  /* pushw(ssp, sp, LWORD(cs)); */
-  pushw(ssp, sp, READ_SEG_REG(cs));
-  pushw(ssp, sp, LWORD(eip));
-  LWORD(esp) -= 6;
-  /* LWORD(cs) = ((us *) 0)[(i << 1) + 1]; */
-  WRITE_SEG_REG(cs, ((us *) 0)[(i << 1) + 1]);
-  LWORD(eip) = ((us *) 0)[i << 1];
-
-#else /* USE_NEW_INT */
   ssp = (unsigned char *)(_SS<<4);
   sp = (unsigned long) _SP;
 
+#ifdef X86_EMULATOR
+  tmp_ssp = ssp+sp;
+  tmp = E_MUNPROT_STACK(tmp_ssp);
+#endif
   pushw(ssp, sp, read_FLAGS());
   pushw(ssp, sp, _CS);
   pushw(ssp, sp, _IP);
+#ifdef X86_EMULATOR
+  if (tmp) E_MPROT_STACK(tmp_ssp);
+#endif
   _SP -= 6;
   _CS = ISEG(i);
   _IP = IOFF(i);
-#endif /* USE_NEW_INT */
 
   /* clear TF (trap flag, singlestep), VIF/IF (interrupt flag), and
    * NT (nested task) bits of EFLAGS
@@ -1267,16 +1293,11 @@ void real_run_int(int i)
    *       some of our own software (...we all are human beings)
    *       For vm86() 'VIF' is the candidate to reset in order to do CLI !
    */
-#ifndef USE_NEW_INT
-  WRITE_FLAGSE(READ_FLAGSE() & ~(VIF | TF | IF | NT));
-#else /* USE_NEW_INT */
   clear_TF();
   clear_NT();
   clear_IF();
-#endif /* USE_NEW_INT */
 }
 
-#ifdef USE_NEW_INT
 /* DANG_BEGIN_FUNCTION run_caller_func(i, from_int)
  *
  * This function runs the specified caller function in response to an
@@ -1330,7 +1351,7 @@ void real_run_int(int i)
 static void run_caller_func(int i, Boolean from_int)
 {
 	void (*caller_function)(int i);
-	ifprintf((d.dos > 1), "DO_INT0x%02x: Using caller_function()\n", i);
+	g_printf("Do INT0x%02x: Using caller_function()\n", i);
 
 #if 0	 /* This causes problems with default_interrupt disable this for now
           * --EB 13 March 1997 
@@ -1365,7 +1386,6 @@ static void run_caller_func(int i, Boolean from_int)
 		/* set_FLAGS(popw(ssp, sp)); */
 	}
 }
-#endif /* USE_NEW_INT */
 
 int can_revector(int i)
 {
@@ -1378,14 +1398,6 @@ int can_revector(int i)
  */
 
   switch (i) {
-  case 0: case 4: case 5: case 7:
-#if 1 /* temp fix for bug in kernel vm86plus code
-       * (fixing kernel patch is in 2.1.27 and will appear in 2.0.30, as Linus promised)
-       * we will remove this, when time goes by --Hans 970225
-       */
-    if (running_kversion > 2001026 || (running_kversion > 2000029 && running_kversion < 2000000) )
-      return NO_REVECT;
-#endif
   case 0x21:			/* we want it first...then we'll pass it on */
 #ifdef DJGPP_HACK
   case 0x23:			/* TMP FIX for ^C under DPMI */
@@ -1440,10 +1452,12 @@ static int can_revector_int21(int i)
   case 0x3e:          /* dos handle close */
   case 0x44:          /* dos ioctl */
 #endif
+  case 0x4b:          /* program load */
+  case 0x4c:          /* program exit */
     return REVECT;
 
   case 0x3d:          /* dos handle open */
-    if (config.emusys || config.emubat)
+    if (config.emusys)
       return REVECT;
     else
       return NO_REVECT;
@@ -1453,34 +1467,34 @@ static int can_revector_int21(int i)
   }
 }
 
-static void int05(u_char i) {
+static void do_print_screen() {
+int x_pos, y_pos;
+ushort *base=SCREEN_ADR(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE));
+    g_printf("PrintScreen: base=%p, lines=%i columns=%i\n", base, li, co);
+    printer_open(1);
+    for (y_pos=0; y_pos < li; y_pos++) {
+	for (x_pos=0; x_pos < co; x_pos++) 
+	    printer_write(1, READ_BYTE(base + y_pos*co + x_pos));
+	printer_write(1, 0x0d);
+	printer_write(1, 0x0a);
+    }
+    printer_flush(1);
+    printer_close(1);
+}
+
+static void int05(u_char i) 
+{
+     /* FIXME does this test actually catch an unhandled bound exception */
     if( *SEG_ADR((Bit8u *), cs, ip) == 0x62 ) {	/* is this BOUND ? */
-	if(!IS_REDIRECTED(5)) {	/* avoid deadlock: eip is not advanced! */
+	    /* avoid deadlock: eip is not advanced! */
 	    error("Unhandled BOUND exception!\n");
 	    leavedos(54);
-	}
-	g_printf("BOUND exception\n");
     }
-    default_interrupt(i);
+    g_printf("INT 5: PrintScreen\n");
+    do_print_screen();
     return;
 }
 
-#ifndef USE_NEW_INT
-void int_a_b_c_d_e_f(u_char i) {
-    g_printf("IRQ->interrupt %x\n", i);
-    show_regs(__FILE__, __LINE__);
-    default_interrupt(i);
-    return;
-}
-
-/* IRQ1, keyb data ready */
-static void int09(u_char i) {
-    fprintf(stderr, "IRQ->interrupt %x\n", i);
-    real_run_int(0x09);
-    return;
-}
-#endif /* not USE_NEW_INT */
-
 /* CONFIGURATION */
 static void int11(u_char i) {
     LWORD(eax) = configuration;
@@ -1493,14 +1507,6 @@ static void int12(u_char i) {
     return;
 }
 
-#ifndef USE_NEW_INT
-/* KEYBOARD */
-static void int16(u_char i) {
-    real_run_int(0x16);
-    return;
-}
-#endif /* not USE_NEW_INT */
-
 /* BASIC */
 static void int18(u_char i) {
   k_printf("BASIC interrupt being attempted.\n");
@@ -1517,8 +1523,6 @@ static void int19(u_char i) {
  */
 static void redirect_devices()
 {
-  extern int RedirectDisk(int, char *, int);
-
   static char s[256] = "\\\\LINUX\\FS", *t = s + 10;
   int i, j;
 
@@ -1634,7 +1638,9 @@ static int redir_it()
 
 
 /* MS-DOS */
-static void int21(u_char i) {
+/* see config.c: int21 is redirected here only when debug_level('D)>0 !! */
+static void int21(u_char i)
+{
 #ifdef X86_EMULATOR
   static char buf[80];
 #endif
@@ -1660,7 +1666,7 @@ static void int21(u_char i) {
 	char *dp = (char *)((LWORD(ds)<<4)+LWORD(edx));
 	unsigned int nb = LWORD(ecx);
 	if (nb>78) nb=78; memcpy(buf,dp,nb); buf[nb]=0;
-	ds_printf("WRITE: [%s]\n",buf);
+	ds_printf("WRITE(40): [%s]\n",buf);
   }
   else if (HI(ax)==9) {
 	char *dp = (char *)((LWORD(ds)<<4)+LWORD(edx));
@@ -1668,10 +1674,10 @@ static void int21(u_char i) {
 	int nb;
 	for (nb=0; (nb<78)&&(*dp!='$'); nb++) *q++ = *dp++;
 	buf[nb]=0;
-	ds_printf("WRITE: [%s]\n",buf);
+	ds_printf("WRITE(09): [%s]\n",buf);
   }
   else if ((HI(ax)==6) && (LO(ax)!=0xff)) {
-	ds_printf("WRITE: [%c]\n",isprint(LO(ax)? LO(ax):'.'));
+	ds_printf("WRITE(06): [%c]\n",isprint(LO(ax)? LO(ax):'.'));
   }
 #endif
   if (!ms_dos(HI(ax)))
@@ -1704,16 +1710,10 @@ static void mouse_post_boot(void)
 	/* This code is dupped for now in base/mouse/mouse.c - JES 96/10/20 */
 	#define Mouse_INT       (0x33 * 16)
 	SETIVEC(0x33, Mouse_SEG, Mouse_INT);
-#ifndef USE_NEW_INT
-      	#define Mouse_INT74     (0x74 * 16)
-	SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
-#else /* USE_NEW_INT */
 #if 0
 	SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
 #endif
 
-#endif /* USE_NEW_INT */
-      
 	/* grab int10 back from video card for mouse */
         ptr = (us*)((BIOSSEG << 4) +
 		    ((long)bios_f000_int10_old - (long)bios_f000));
@@ -1721,10 +1721,8 @@ static void mouse_post_boot(void)
         ptr[0] = IOFF(0x10);
         ptr[1] = ISEG(0x10);
         m_printf("after store, ptr[0] = %x, ptr[1] = %x\n",ptr[0],ptr[1]);
-#if defined(USE_NEW_INT) || (INT10_WATCHER_SEG != BIOSSEG)
 	 /* Otherwise this isn't safe */
 	SETIVEC(0x10, INT10_WATCHER_SEG, INT10_WATCHER_OFF);
-#endif
 }
 
 static void dos_post_boot(void)
@@ -1744,7 +1742,7 @@ static void int28(u_char i) {
   if (config.hogthreshold) {
     /* the hogthreshold value just got redefined to be the 'garrot' value */
     static int time_count = 0;
-    if (++time_count >= config.hogthreshold) {
+    if (++time_count >= config.hogthreshold * 50) {
       usleep(INT28_IDLE_USECS);
       time_count = 0;
     }
@@ -1767,11 +1765,17 @@ static void int2f(u_char i)
        LWORD(eax), LWORD(ebx), LWORD(ecx), LWORD(edx), LWORD(ds), LWORD(es));
 #endif
   switch (LWORD(eax)) {
-  case INT2F_IDLE_MAGIC:   /* magic "give up time slice" value */
-    if (config.hogthreshold)
-      usleep(INT2F_IDLE_USECS);
-    LWORD(eax) = 0;
-    return;
+  case INT2F_IDLE_MAGIC: {  /* magic "give up time slice" value */
+      static int trigger = 0;
+      if (config.hogthreshold) {
+        if (trigger++ >= config.hogthreshold * 100) {
+          usleep(INT2F_IDLE_USECS);
+	  trigger = 0;
+	}
+      }
+      LWORD(eax) = 0;
+      return;
+    }
 
 #ifdef IPX
   case INT2F_DETECT_IPX:  /* TRB - detect IPX in int2f() */
@@ -1786,10 +1790,9 @@ static void int2f(u_char i)
     if (LO(ax) == 0x23) subst_file_ext(SEG_ADR((char *), ds, si));
     if (mfs_redirector()) return;
     break;
-  
+
   case 0x16:		/* misc PM/Win functions */
     if (!config.dpmi) {
-/*  d.emu=4; */
       break;		/* fall into default_interrupt() */
     }
     switch (LO(ax)) {
@@ -1854,7 +1857,7 @@ static void int2f(u_char i)
 
       case 0x86:            /* Are we in protected mode? */
         D_printf("DPMI CPU mode check in real mode.\n");
-        if (in_dpmi) /* set AX to zero only if program executes in protected mode */
+        if (in_dpmi && !in_dpmi_dos_int) /* set AX to zero only if program executes in protected mode */
 	    LWORD(eax) = 0;	/* say ok */
 		 /* else let AX untouched (non-zero) */
       return;
@@ -1896,7 +1899,7 @@ static void int33(u_char i) {
  * garrot control when the dos app is polling the mouse and the mouse is 
  * taking a break. */
   
-  static unsigned short int oldx=0, oldy=0, trigger=0;
+  static unsigned short int oldx=0, oldy=0, trigger=0, trigger1=0;
    
 /* Firstly do the actual mouse function. */   
 /* N.B. This code only works with the intdrv since default_interrupt() does not
@@ -1905,13 +1908,11 @@ static void int33(u_char i) {
  * reschedules dosemu it will then start executing the real mode mouse handler). :-( 
  * Do we need/have we got post_interrupt (IRET) handlers? 
  */
-#ifdef USE_NEW_INT
 /* We have post_interrupt handlers in dpmi --EB 28 Oct 1997 */
-#endif /* USE_NEW_INT */
-  if (mice->intdrv) 
-    mouse_int();
-  else 
+
+  if (!mouse_int())
     default_interrupt(i);
+  
 /* It seems that the only mouse sub-function that could be plausibly used to 
  * poll the mouse is AX=3 - get mouse buttons and position. 
  * The mouse driver should have left AX=3 unaltered during its call.
@@ -1926,9 +1927,9 @@ static void int33(u_char i) {
  */
    if (LWORD(eax) ==0x0003)  {
      if (LWORD(ebx) == 0 && oldx == LWORD(ecx) && oldy == LWORD(edx) ) 
-        trigger++;
+        trigger1++;
       else  { 
-        trigger=0;
+        trigger1=0;
         oldx = LWORD(ecx);
         oldy = LWORD(edx);
       } 
@@ -1936,10 +1937,13 @@ static void int33(u_char i) {
 m_printf("Called/ing the mouse with AX=%x \n",LWORD(eax));
 /* Ok now we test to see if the mouse has been taking a break and we can let the 
  * system get on with some real work. :-) */
-   if (config.hogthreshold && trigger >= config.hogthreshold)  {
-     m_printf("Ignoring the quiet mouse.\n");
-     usleep(INT2F_IDLE_USECS);
-     trigger=0;
+   if (trigger1 >= config.hogthreshold*200) {
+     if (config.hogthreshold && ++trigger >= config.hogthreshold)  {
+       m_printf("Ignoring the quiet mouse.\n");
+       usleep(INT2F_IDLE_USECS);
+       trigger=0;
+     }
+     trigger1--;
    }
 }
 
@@ -1976,52 +1980,28 @@ static void inte7(u_char i) {
 
 void do_int(int i)
 {
-#ifndef USE_NEW_INT
-  void (* caller_function)();
-
-  if ((LWORD(cs) != BIOSSEG) && IS_REDIRECTED(i) && can_revector(i)){
-    real_run_int(i);
-    return;
-  }
-
-  caller_function = interrupt_function[i];
-  caller_function(i);
-  /* This is a kludge to avoid immediate respawning of dosemu
-   * if we have cs:ip pointing to the BIOS stubs' IRET
-   * We do the IRET ourselves, so we directly let the DOSapp
-   * control.
-   * This perhaps avoids chained timer ints to accumulate
-   * without giving the DOSapp a chance to do something.
-   * (not sure if it works)
-   * - Hans Lermen
-   */ 
-  if (LWORD(cs) == BIOSSEG) {
-    unsigned char *csp;
-    csp = SEG_ADR((unsigned char *),cs,ip);
-    if ((*csp == 0xcf)) {
-      unsigned char *ssp = (unsigned char *)(LWORD(ss)<<4);
-      unsigned long sp = (unsigned long) LWORD(esp);
-      LWORD(esp) +=6;
-      LWORD(eip) = popw(ssp, sp);
-      REG(cs) = popw(ssp, sp);
-      WRITE_FLAGS(popw(ssp, sp));
-      if (READ_FLAGS() & IF) WRITE_FLAGSE(READ_FLAGSE() | VIF);
-      else  WRITE_FLAGSE((READ_FLAGSE() | IF) & ~VIF);
-    }
-  }
-#else /* USE_NEW_INT */
  	unsigned long magic_address;
+
+        /* we must clear the AC flag here since real mode INT instructions
+           do that too. An IRET (not IRETD) instruction then does not set AC
+           because the AC flag is in the high part of the eflags.
+           DOS applications usually set AC to try to detect the presence of
+           a 486. They hopefully protect this test using cli and sti, or
+           hardware INTs will mess things up.
+        */
+	if(REG(eflags & AC)) g_printf("Clearing AC flag\n");
+        clear_AC();
 	if (in_dpmi) {
 		if (dpmi_eflags & IF) {
 			set_IF();
 		} else {
 			clear_IF();
 		}
-/* D_printf("DPMI: do_int: dpmi_eflags=%08x\n",dpmi_eflags); */
+/**/ D_printf("DPMI: do_int %x: dpmi_eflags=%08x\n",i,dpmi_eflags);
 	}
 	
 #ifndef TRACE_DPMI
- 	if ((d.defint > 2) && (((i != 0x28) && (i != 0x2f)) || in_dpmi)) {
+ 	if ((debug_level('#') > 2) && (((i != 0x28) && (i != 0x2f)) || in_dpmi)) {
  		di_printf("Do INT0x%02x eax=0x%08x ebx=0x%08x ss=0x%04x esp=0x%08x\n"
  			  "           ecx=0x%08x edx=0x%08x ds=0x%04x  cs=0x%04x ip=0x%04x\n"
  			  "           esi=0x%08x edi=0x%08x es=0x%04x flg=0x%08x\n",
@@ -2034,7 +2014,7 @@ void do_int(int i)
 #if 1  /* This test really ought to be in the main loop before
  	*  instruction execution not here. --EB 10 March 1997 
  	*/
-	
+
  	/* try to catch jumps to 0:0 (e.g. uninitialized user interrupt vectors),
  	   which sometimes can crash the whole system, not only dosemu... */
  	if (SEGOFF2LINEAR(_CS, _IP) < 1024) {
@@ -2053,6 +2033,14 @@ void do_int(int i)
  	magic_address = SEGOFF2LINEAR(BIOSSEG, INT_OFF(i));
  	if (magic_address == (SEGOFF2LINEAR(_CS, _IP) -2)) {
  		run_caller_func(i, FALSE);
+        	if ((debug_level('#') > 2) && (((i != 0x28) && (i != 0x2f)) || in_dpmi)) {
+        		di_printf("RET INT0x%02x eax=0x%08x ebx=0x%08x ss=0x%04x esp=0x%08x\n"
+ 			  "           ecx=0x%08x edx=0x%08x ds=0x%04x  cs=0x%04x ip=0x%04x\n"
+ 			  "           esi=0x%08x edi=0x%08x es=0x%04x flg=0x%08x\n",
+ 			  i, _EAX, _EBX, _SS, _ESP,
+ 			  _ECX, _EDX, _DS, _CS, _IP,
+ 			  _ESI, _EDI, _ES, (int) read_EFLAGS());
+ 	}
  	}
  	else if ((magic_address == IVEC(i)) ||
  		 (can_revector(i) == REVECT)) {
@@ -2061,9 +2049,62 @@ void do_int(int i)
  	else {
  		real_run_int(i);
  	}
-#endif /* USE_NEW_INT */
 }
 
+void fake_int(int cs, int ip)
+{
+  unsigned char *ssp;
+  unsigned long sp;
+
+  g_printf("fake_int: CS:IP %04x:%04x\n", cs, ip);
+  ssp = (unsigned char *)(LWORD(ss)<<4);
+  sp = (unsigned long) LWORD(esp);
+
+  pushw(ssp, sp, vflags);
+  pushw(ssp, sp, cs);
+  pushw(ssp, sp, ip);
+  LWORD(esp) -= 6;
+
+  clear_TF();
+  clear_NT();
+  clear_IF();
+}
+
+void fake_call(int cs, int ip)
+{
+  unsigned char *ssp;
+  unsigned long sp;
+
+  ssp = (unsigned char *)(LWORD(ss)<<4);
+  sp = (unsigned long) LWORD(esp);
+
+  g_printf("fake_call() CS:IP %04x:%04x\n", cs, ip);
+  pushw(ssp, sp, cs);
+  pushw(ssp, sp, ip);
+  LWORD(esp) -= 4;
+}
+
+void fake_pusha(void)
+{
+  unsigned char *ssp;
+  unsigned long sp;
+
+  ssp = (unsigned char *)(LWORD(ss)<<4);
+  sp = (unsigned long) LWORD(esp);
+
+  pushw(ssp, sp, LWORD(eax));
+  pushw(ssp, sp, LWORD(ecx));
+  pushw(ssp, sp, LWORD(edx));
+  pushw(ssp, sp, LWORD(ebx));
+  pushw(ssp, sp, LWORD(esp));
+  pushw(ssp, sp, LWORD(ebp));
+  pushw(ssp, sp, LWORD(esi));
+  pushw(ssp, sp, LWORD(edi));
+  LWORD(esp) -= 16;
+  pushw(ssp, sp, REG(ds));
+  pushw(ssp, sp, REG(es));
+  LWORD(esp) -= 4;
+}
 
 
 /*
@@ -2089,30 +2130,18 @@ void setup_interrupts(void) {
     /* don't overwrite; these have been set during video init */
     if(i == 0x1f || i == 0x43) continue;
 
-    if ((i & 0xf8) == 0x60 || (i >= 0x78 && i<= 0xfd &&
+    /* interrupts >= 0xc0 are scratch (BIOS stack), 
+       unless defined by DOSEMU */	
+    if ((i & 0xf8) == 0x60 || (i >= 0x78 && i < 0xc0 &&
       can_revector(i) == NO_REVECT)) { /* user interrupts */
 	/* show also EMS (int0x67) as disabled */
 	SETIVEC(i, 0, 0);
-    } else {
-#ifndef USE_NEW_INT
-	SETIVEC(i, BIOSSEG, 16 * i);
-#else /* USE_NEW_INT */
+    } else if (i < 0xc0 || can_revector(i) == REVECT) {
 	SETIVEC(i, BIOSSEG, INT_OFF(i));
-#endif /* USE_NEW_INT */
     }
   }
   
   interrupt_function[5] = int05;
-#ifndef USE_NEW_INT
-  interrupt_function[8] = int08;
-  interrupt_function[9] = int09;
-  interrupt_function[0xa] = int_a_b_c_d_e_f;
-  interrupt_function[0xb] = int_a_b_c_d_e_f;
-  interrupt_function[0xc] = int_a_b_c_d_e_f;
-  interrupt_function[0xd] = int_a_b_c_d_e_f;
-  interrupt_function[0xe] = int_a_b_c_d_e_f;
-  interrupt_function[0xf] = int_a_b_c_d_e_f;
-#endif /* not USE_NEW_INT */
   /* This is called only when revectoring int10 */
   interrupt_function[0x10] = int10;
   interrupt_function[0x11] = int11;
@@ -2120,9 +2149,7 @@ void setup_interrupts(void) {
   interrupt_function[0x13] = int13;
   interrupt_function[0x14] = int14;
   interrupt_function[0x15] = int15;
-#ifndef USE_NEW_INT
   interrupt_function[0x16] = int16;
-#endif /* not USE_NEW_INT */
   interrupt_function[0x17] = int17;
   interrupt_function[0x18] = int18;
   interrupt_function[0x19] = int19;
@@ -2136,19 +2163,16 @@ void setup_interrupts(void) {
   interrupt_function[0x2f] = int2f;
   interrupt_function[0x33] = int33;
 #ifdef USING_NET
-  interrupt_function[0x60] = int_pktdrvr;
+  if (config.pktdrv)
+    interrupt_function[0x60] = int_pktdrvr;
 #endif
   interrupt_function[0xe6] = inte6;
   interrupt_function[0xe7] = inte7;
 
   /* Let kernel handle this, no need to return to DOSEMU */
-#ifndef USE_NEW_INT
-  SETIVEC(0x1c, 0xf010, 0xc0);
-#else /* USE_NEW_INT */
  #if 0
   SETIVEC(0x1c, BIOSSEG + 0x10, INT_OFF(0x1c) +2 - 0x100);
  #endif
-#endif /* USE_NEW_INT */
 
   /* show EMS as disabled */
   SETIVEC(0x67, 0, 0);
@@ -2160,19 +2184,11 @@ void setup_interrupts(void) {
     seg = (u_short *) (Mouse_ADD + 10);
     /* tell the mouse driver where we are...exec add, seg, offset */
     mouse_sethandler(ptr, seg, off);
-#ifndef USE_NEW_INT
-    SETIVEC(0x74, Mouse_SEG, Mouse_ROUTINE_OFF);
-    SETIVEC(0x33, Mouse_SEG, Mouse_ROUTINE_OFF);
-  }
-  else
-    *(unsigned char *) (BIOSSEG * 16 + 16 * 0x33) = 0xcf;	/* IRET */
-#else /* USE_NEW_INT */
   }
   else {
     /* point to the retf#2 immediately after the int 33 entry point. */
     SETIVEC(0x33, BIOSSEG, INT_OFF(0x33)+2);
   }
-#endif /* USE_NEW_INT */
 
   SETIVEC(0x16, INT16_SEG, INT16_OFF);
   SETIVEC(0x09, INT09_SEG, INT09_OFF);
@@ -2190,13 +2206,16 @@ void setup_interrupts(void) {
   SETIVEC(0xe7, INTE7_SEG, INTE7_OFF);
   /* End of int 0xe7 for FCB opens */
 
+#ifdef IPX
+  /* IPX. Dummy but should not crash */
+  if (config.ipxsup)
+    SETIVEC(0x7a, BIOSSEG, 0x7a * 16);
+#endif
+
   /* set up relocated video handler (interrupt 0x42) */
   if (config.dualmon == 2) {
     interrupt_function[0x42] = interrupt_function[0x10];
   }
-#ifndef USE_NEW_INT
-  else *(u_char *) 0xff065 = 0xcf;	/* IRET */
-#endif /* not USE_NEW_INT */
 }
 
 
@@ -2249,39 +2268,3 @@ void int_vector_setup(void)
 #endif
 
 }
-
-#ifdef USE_MRP_JOYSTICK
-void mrp_read_joystick(void)
-{
-  /* This only reads the first joystick and doesn't handle the buttons. */
-  /* You will need to install the joystick module for this to work. */
-  static int fd =0;
-  static const char * const fname = "/dev/js0" ;
-  int status;
-  struct JS_DATA_TYPE js;
-
-  if (fd == 0) {
-    fd = open (fname, O_RDONLY);
-    if (fd < 0)
-      perror ("mrp_read_joystick (open)");
-  }
-  if (fd < 0) {
-    CARRY; /* fail */
-    return;
-  }
-
-  status = read (fd, &js, JS_RETURN);
-  if (status != JS_RETURN) {
-    perror ("mrp_read_joystick (read)");
-    fd = -1;
-    CARRY; /* fail */
-    return;
-  }
-  LWORD(eax) = js.x;
-  LWORD(ebx) = js.y;
-  LWORD(ecx) = 0;		/* second joystick x */
-  LWORD(edx) = 0;		/* second joystick y */
-  NOCARRY;
-  return;
-}
-#endif
