@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -430,7 +430,8 @@ interpretCfgSpace(unsigned long *pciheader,unsigned long *pcibuses,int busidx,
 	} else if (PCI_BRIDGE_HOST_CLASS(pciTmp->class)) {
 	    Z_printf("PCI-HOST bridge:\n");
 	    if (++hostbridges > 1) {/* HOST-PCI bridges*/
-		pcibuses[++numbus] = numbus;
+		numbus++;
+		pcibuses[numbus] = numbus;
 	    }
 	}
     }
