@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -378,7 +378,7 @@ void tty_char_out(unsigned char ch, int s, int attr)
 #else
     putchar(0x07);
 #endif
-    break;
+    return;
 
   default:          /* Printable character */
 #if X_GRAPHICS
@@ -550,7 +550,7 @@ static boolean X_set_video_mode(int mode) {
    * ( Now we can do INT 10, AX=1A00 ).
    * Some older DOS programs do it by modifying EQUIP-flags
    * and then let the BIOS say, if it can ?!?!)
-   * If we have config.dualmon, this happens legaly.
+   * If we have config.dualmon, this happens legally.
    */
   if(config.dualmon)
     Video->setmode(7, co, li);
@@ -760,6 +760,8 @@ void return_state(Bit8u *statebuf) {
 	memcpy(statebuf + 0x04, (char *)0x449, 0x466 - 0x449 + 1);
 	/* store bios 0:484-0:486 at ofs 0x22 */
 	memcpy(statebuf + 0x22, (char *)0x484, 0x486 - 0x484 + 1);
+	/* correct number of rows-1 to number of rows at offset 0x22 */
+	statebuf[0x22]++;
 	get_dcc(&active_dcc, &alternate_dcc);
 	statebuf[0x25] = active_dcc;
 	statebuf[0x26] = alternate_dcc;
@@ -812,9 +814,9 @@ void int10_old()
   }
 #endif
 
-  if (d.video >= 3)
+  if (debug_level('v') >= 3)
     {
-      if (d.video >= 4)
+      if (debug_level('v') >= 4)
 /*	dbug_printf("int10 near %04x:%08lx\n", LWORD(cs), REG(eip));*/
 	dbug_printf("int10 near %04x:%08lx\n", READ_SEG_REG(cs), REG(eip));
       if ( (LO(ax) >= ' ') && (LO(ax) < 0x7f) )
@@ -1385,9 +1387,9 @@ void int10_new()
   us *sm;
 
   if(HI(ax) > 0x0f)	// EVIL hack! Remove it later!!!
-  if (d.video >= 3)
+  if (debug_level('v') >= 3)
     {
-      if (d.video >= 4)
+      if (debug_level('v') >= 4)
 	i10_msg("near %04x:%08lx\n", READ_SEG_REG(cs), REG(eip));
       if ( (LO(ax) >= ' ') && (LO(ax) < 0x7f) )
 	i10_msg("AH=%02x AL=%02x '%c'\n",
