@@ -1,16 +1,17 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
 
-#define EXTERN 
-#define INIT(x...)		=  ## x
+#define EXTERN
+#define INIT(x...)		= x
 
 #include <termios.h>
 #include <sys/types.h>
 #include "config.h"
 #include "emu.h"
+#include "mapping.h"
 #include "xms.h"
 #include "disks.h"
 #include "timers.h"
