@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -36,11 +36,7 @@
 
 #include <features.h>
 #include <sys/time.h>
-#if GLIBC_VERSION_CODE >= 2000
 #include <sys/kd.h>
-#else
-#include <linux/kd.h>
-#endif  
 #include "config.h"
 #include "emu.h"
 #include "port.h"
@@ -53,10 +49,9 @@
 #include "keyb_server.h"
 
 #undef  DEBUG_PIT
+#undef  ONE_MINUTE_TEST
 
-#ifdef MONOTON_MICRO_TIMING
 extern hitimer_t pic_itime[33];
-#endif /* MONOTON_MICRO_TIMING */
 
 extern hitimer_t t_vretrace;	/* see env/video/miscemu.c */
 
@@ -129,6 +124,9 @@ void initialize_timers(void)
  */
 void timer_tick(void)
 {
+#ifdef ONE_MINUTE_TEST
+  static int dbug_count = 0;
+#endif
   hitimer_u tp;
   u_long time_curr;
   static u_long time_old = 0;       /* Preserve value for next call */
@@ -138,16 +136,19 @@ void timer_tick(void)
     return;
   }
 
-  /* Get system time in usec */
-  tp.td = GETusTIME(0);
+  /* Get system time in ticks */
+  tp.td = GETtickTIME(0);
+#ifdef ONE_MINUTE_TEST
+  if (++dbug_count > 6000) leavedos(0);
+#endif
 
-  /* compute the number of 100usecs since we started */
+  /* compute the number of 100ticks since we started */
   time_curr  = (tp.td - pit[0].time.td) / 100;
   
   /* Reset old timer value to 0 if time_curr wrapped around back to 0 */
   if (time_curr < time_old) time_old = 0;
   
-  /* Compute number of 100usec ticks since the last time this function ran */
+  /* Compute number of 100ticks ticks since the last time this function ran */
   ticks_accum += (time_curr - time_old);
   
   /* Save old value of the timer */
@@ -191,8 +192,10 @@ void do_sound(Bit16u period)
 	 * If it becomes a problem possibly tuning sound_duration is
 	 * the answer.  I suggest 200ms but as I don't have that
 	 * problem now I'm not worring about it.
+	 *
+	 * But if you set it too low, then sounds can be cut off - clarence
 	 */
-	static const unsigned sound_duration = 990;  /* in miliseconds */
+	static const unsigned sound_duration = 30000;  /* in miliseconds */
 	switch (port61 & 3) {
 	case 3:		/* speaker on & speaker control through timer channel 2 */
 		if ((pit[2].mode == 2) || (pit[2].mode == 3)) {		/* is this test needed? */
@@ -269,26 +272,6 @@ static void pit_latch(int latch)
 	p->outpin = (p->mode&4? 0x00: 0x80);
       }
   }
-#ifndef MONOTON_MICRO_TIMING
-  /* mode 2 -- rate generator */
-  /* mode 6 -- ??? */
-  else if ((p->mode & 3)==2) {
-      ticks = p->cntr - (pic_dos_time % p->cntr);
-      p->read_latch = p->cntr - ticks % p->cntr;
-      p->outpin = (p->read_latch? 0x80: 0x00);
-  }
-  /* mode 3 -- square-wave generator */
-  /* mode 7 -- ??? */
-  else {
-      /* no more fancy calculations to avoid overflow */
-      ticks = (cur_time.td - p->time.td) % p->cntr;
-      /* ticks is now a value which increases from 0 to cntr, and
-         is greater than cntr/2 in the second half of the cycle */
-      ticks = 2 * (ticks % p->cntr);
-      p->outpin = (ticks >= p->cntr? 0x80: 0x00);
-      p->read_latch = p->cntr - ticks % p->cntr;
-  }
-#else /* MONOTON_MICRO_TIMING */
   else {
     /* mode 2 -- rate generator */
     /* mode 6 -- ??? */
@@ -330,7 +313,6 @@ static void pit_latch(int latch)
         p->outpin = (p->read_latch? 0x80: 0x00);
       }
   }
-#endif /* MONOTON_MICRO_TIMING */
   if (p->mode & 0x40)
     p->mode = (p->mode & 7) | 0x80;
 
@@ -502,14 +484,12 @@ void pit_control_outp(ioport_t port, Bit8u val)
 	pit[latch].read_state  = (val >> 4) & 0x03;
 	pit[latch].write_state = (val >> 4) & 0x03;
 	pit[latch].mode        = (val >> 1) & 0x07;
-#ifdef MONOTON_MICRO_TIMING
         if ((val & 4)==0) {      /* modes 0,1,4,5 */
           /* set the time base for the counter - safety code for programs
            * which use a non-periodical mode without reloading the counter
            */
           pit[latch].time.td = GETtickTIME(0);
         }
-#endif
       }
 #ifdef DEBUG_PIT
       i_printf("PORT: writing outp(0x43, 0x%x)\n", val);
@@ -553,50 +533,8 @@ void pit_control_outp(ioport_t port, Bit8u val)
  *
  * DANG_END_FUNCTION
  */
-#if 0  /* =0, if old engine */
 void timer_int_engine(void)
 {
-  static continuous = 0;
-
-  if (ticks_accum < timer_div)
-    continuous = 0;                     /* Reset overflow guard */
-  else {
-
-    /* Overflow guard so that there is no more than 50 irqs in a chain */
-    continuous++;
-    
-    if (continuous > 50) 
-      continuous = 0;                   /* Reset overflow guard */
-    else {
-        pic_request(PIC_IRQ0);            /* Start a timer irq */
-    }
-
-    /* Decrement the ticks accumulator */
-    ticks_accum -= timer_div;
-    
-    /* Prevent more than 250 ms of extra timer ticks from building up.
-     * This ensures that if timer_int_engine get called less frequently
-     * than every 250ms, the code won't choke on an overload of timer 
-     * interrupts.
-     */
-    if (ticks_accum > 250) ticks_accum -= 250;
-  }
-}
-#else /* new engine */
-void timer_int_engine(void)
-{
-/* Try to terminate previous int8 routine before scheduling
- * another one. Without this one dope crashes (DPMI_rm_procedure_
- * running increases up to a stack fault)
- *
- * This is a gross disgusting cludge, that I am going to make even grosser
- * and more disgusting in an attempt to get it to work properly.  Someday
- * I am going to find and fix the root problems that are making this 
- * necessary.  -KenC
- */
- if (in_dpmi) {
-   while (in_dpmi_timer_int) run_dpmi();
- }
  pic_sched(PIC_IRQ0,pit[0].cntr);
  do_irq();
 }
@@ -752,4 +690,3 @@ void pit_reset(void)
   port61 = 0x0c;
   do_ioctl(console_fd,KIOCSOUND,0);    /* sound off */
 }
-#endif /* new engine */
