@@ -1,5 +1,5 @@
-/* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+/*
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -25,6 +25,10 @@
 #include "bios.h"
 #include "lpt.h"
 #include "int.h"
+#include "dosemu_config.h"
+#include "init.h"
+#include "disks.h"
+#include "lpt.h"
 
 #include "dos2linux.h"
 #include "priv.h"
@@ -48,8 +52,8 @@ extern void mapping_close(void);
 
 #if 0  /* initialized in data.c (via emu.h) */
 struct debug_flags d =
-{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-/* d  R  W  D  C  v  X  k  i  T  s  m  #  p  g  c  w  h  I  E  x  M  n  P  r  S */
+{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+/* d  R  W  D  C  v  X  k  i  T  s  m  #  p  g  c  w  h  I  E  x  M  n  P  r  S  j */
 #endif
 
 int kernel_version_code = 0;
@@ -59,8 +63,8 @@ int dosemu_argc;
 char **dosemu_argv;
 
 static void     check_for_env_autoexec_or_config(void);
-int     parse_debugflags(const char *s, unsigned char flag);
-static void     usage(void);
+extern int     parse_debugflags(const char *s, unsigned char flag);
+static void     usage(char *basename);
 void memcheck_type_init(void);
 
 /*
@@ -106,7 +110,11 @@ config_defaults(void)
 {
     char *cpuflags;
     int k = 386;
+    struct disk *dptr;
+    struct printer *pptr;
+    extern struct printer lpt[NUM_PRINTERS];
 
+    parse_debugflags("+cw", 1);
     vm86s.cpu_type = CPU_386;
     /* defaults - used when /proc is missing, cpu!=x86 etc. */
     config.realcpu = CPU_386;
@@ -114,22 +122,25 @@ config_defaults(void)
     config.rdtsc = 0;
     config.mathco = 0;
     config.smp = 0;
+    config.cpummx = 0;
     config.CPUSpeedInMhz = 150;	/* instruction cycles per us, entry level */
 
     open_proc_scan("/proc/cpuinfo");
     switch (get_proc_intvalue_by_key(
           kernel_version_code > 0x20100+74 ? "cpu family" : "cpu" )) {
-      case 3: case 386: config.realcpu = CPU_386;	/* redundant */
-      	break;
       case 5: case 586:
       case 6: case 686:
       case 15:
         config.realcpu = CPU_586;
-        config.pci = 1;	/* fair guess */
         cpuflags = get_proc_string_by_key("features");
         if (!cpuflags) {
           cpuflags = get_proc_string_by_key("flags");
         }
+#ifdef X86_EMULATOR
+        if (cpuflags && strstr(cpuflags, "mmx")) {
+	  config.cpummx = 1;
+        }
+#endif
         if (cpuflags && strstr(cpuflags, "tsc")) {
           /* bogospeed currently returns 0; should it deny
            * pentium features, fall back into 486 case */
@@ -139,7 +150,9 @@ config_defaults(void)
 	    /* last known proc/cpuinfo format is xxx.xxxxxx, with 3
 	     * int and 6 decimal digits - but what if there are less
 	     * or more digits??? */
-	    if (sscanf(cpuflags,"%d.%d",&di,&df)==2) {
+	    /* some broken kernel/cpu combinations apparently imply
+	     * the existence of 0 Mhz CPUs */
+	    if (sscanf(cpuflags,"%d.%d",&di,&df)==2 && (di || df)) {
 		char cdd[8]; int i;
 		long long chz = 0;
 		char *p = cpuflags;
@@ -156,7 +169,9 @@ config_defaults(void)
 		/* speed division factor to get 838ns from CPU clock */
 		config.cpu_tick_spd = (LLF_TICKS*1000000)/chz;
 
-		fprintf (stderr,"kernel CPU speed is %Ld Hz\n",chz);
+		fprintf (stderr,"Linux kernel %d.%d.%d; CPU speed is %Ld Hz\n",
+		   kernel_version_code >> 16, (kernel_version_code >> 8) & 255,
+		   kernel_version_code & 255,chz);
 /*		fprintf (stderr,"CPU speed factors %ld,%ld\n",
 			config.cpu_spd, config.cpu_tick_spd); */
 		config.CPUSpeedInMhz = di + (df>500000);
@@ -180,7 +195,8 @@ config_defaults(void)
       case 4: case 486: config.realcpu = CPU_486;
       	break;
       default:
-      	exit(1);	/* no 186,286,786.. */
+        error("Unknown CPU type!\n");
+	/* config.realcpu is set to CPU_386 at this point */
     }
     config.mathco = strcmp(get_proc_string_by_key("fpu"), "yes") == 0;
     reset_proc_bufferptr();
@@ -193,7 +209,7 @@ config_defaults(void)
       config.smp = 1;		/* for checking overrides, later */
     }
     close_proc_scan();
-    fprintf(stderr,"Running on CPU=%d86, FPU=%d\n",
+    fprintf(stderr,"Dosemu-" VERSTR " Running on CPU=%d86, FPU=%d\n",
       config.realcpu, config.mathco);
 
     config.hdiskboot = 1;	/* default hard disk boot */
@@ -202,25 +218,51 @@ config_defaults(void)
     config.cpuemu = 0;
 #endif
     config.mem_size = 640;
-    config.ems_size = 0;
-    config.ems_frame = 0xd000;
-    config.xms_size = 0;
+    config.ems_size = 2048;
+    config.ems_frame = 0xe000;
+    config.xms_size = 8192;
     config.max_umb = 0;
-    config.dpmi = 0;
-    config.secure = 1;  /* need to have it 'on', else user may trick it out
-                           via -F option */
+    config.dpmi = (!under_root_login && can_do_root_stuff) ? 0 : 16384;
+    config.secure = 0;
     config.mouse_flag = 0;
     config.mapped_bios = 0;
     config.vbios_file = NULL;
     config.vbios_copy = 0;
-    config.vbios_seg = 0xc000;
-    config.vbios_size = 0x10000;
+    config.vbios_seg = 0;
+    config.vbios_size = 0;
     config.console = 0;
     config.console_keyb = 0;
     config.console_video = 0;
     config.kbd_tty = 0;
-    config.fdisks = 0;
-    config.hdisks = 0;
+    config.fdisks = 1;
+    dptr = &disktab[0];
+    dptr->type    = FLOPPY;
+    dptr->default_cmos = THREE_INCH_FLOPPY;
+    dptr->sectors = 0;
+    dptr->heads   = 0;
+    dptr->tracks  = 0;
+    dptr->timeout = 0;
+    dptr->dev_name = "/dev/fd0";              /* default-values */
+    dptr->boot_name = NULL;
+    dptr->wantrdonly = 0;
+    dptr->header = 0;
+    dptr->dexeflags = 0;
+
+    config.hdisks = 1;
+    dptr = &hdisktab[0];
+    dptr->type    = DIR_TYPE;
+    dptr->sectors = -1;
+    dptr->heads   = -1;
+    dptr->tracks  = -1;
+    dptr->timeout = 0;
+    dptr->dev_name = malloc(strlen(DOSEMU_HDIMAGE_DIR) + 8 + 1);   /* default-values */
+    strcpy(dptr->dev_name, DOSEMU_HDIMAGE_DIR);
+    strcat(dptr->dev_name, "/freedos"); 
+    dptr->boot_name = NULL;
+    dptr->wantrdonly = 0;
+    dptr->header = 0;
+    dptr->dexeflags = 0;
+
     config.bootdisk = 0;
     config.exitearly = 0;
     config.term_esc_char = 30;	       /* Ctrl-^ */
@@ -231,37 +273,37 @@ config_defaults(void)
     config.term_charset = CHARSET_LATIN;
     /* config.term_corner = 1; */
     config.X_updatelines = 25;
-    config.X_updatefreq = 8;
+    config.X_updatefreq = 5;
     config.X_display = NULL;	/* NULL means use DISPLAY variable */
-    config.X_title = "dosemu";
-    config.X_icon_name = "dosemu";
-    config.X_blinkrate = 8;
+    config.X_title = "DOS in a BOX";
+    config.X_icon_name = "xdos";
+    config.X_blinkrate = 12;
     config.X_sharecmap = 0;     /* Don't share colourmap in graphics modes */
-    config.X_mitshm = 0;
+    config.X_mitshm = 1;
     config.X_fixed_aspect = 1;
-    config.X_aspect_43 = 0;
+    config.X_aspect_43 = 1;
     config.X_lin_filt = 0;
     config.X_bilin_filt = 0;
     config.X_mode13fact = 2;
     config.X_winsize_x = 0;
     config.X_winsize_y = 0;
     config.X_gamma = 100;
-    config.vgaemu_memsize = 0;
+    config.vgaemu_memsize = 1024;
     config.vesamode_list = NULL;
     config.X_lfb = 1;
     config.X_pm_interface = 1;
-    config.X_keycode = 0;
+    config.X_keycode = 2;
     config.X_font = "vga";
     config.usesX = 0;
     config.X = 0;
-    config.X_mgrab_key = "";	/* off , NULL = "Home" */
-    config.hogthreshold = 10;	/* bad estimate of a good garrot value */
+    config.X_mgrab_key = NULL;	/* on , NULL = "Home" */
+    config.hogthreshold = 1;	/* bad estimate of a good garrot value */
     config.chipset = PLAINVGA;
     config.cardtype = CARD_VGA;
     config.pci_video = 0;
     config.fullrestore = 0;
     config.graphics = 0;
-    config.gfxmemsize = 256;
+    config.gfxmemsize = 1024;
     config.vga = 0;		/* this flags BIOS graphics */
     config.dualmon = 0;
     config.force_vt_switch = 0;
@@ -280,7 +322,6 @@ config_defaults(void)
     config.realdelta = 9154;
 
     config.timers = 1;		/* deliver timer ints */
-    config.keybint = 1;		/* keyboard interrupts, now ALWAYS 1 */
  
     /* Lock file stuff */
     config.tty_lockdir = PATH_LOCKD;    /* The Lock directory  */
@@ -288,17 +329,17 @@ config_defaults(void)
     config.tty_lockbinary = FALSE;      /* Binary lock files ? */
 
     config.num_ser = 0;
-    config.num_lpt = 0;
     config.fastfloppy = 1;
 
     config.emusys = (char *) NULL;
-    config.emubat = (char *) NULL;
     config.emuini = (char *) NULL;
     config.dosbanner = 1;
-    config.allowvideoportaccess = 0;
+    config.allowvideoportaccess = 1;
     config.emuretrace = 0;
 
-    config.keytable = &keytable_list[KEYB_USER]; /* What's the current keyboard  */
+    keyb_layout(-1); /* What's the current keyboard  */
+		config.altkeytable = NULL;
+		config.toggle_mask = 0;
 
     config.detach = 0;		/* Don't detach from current tty and open
 				 * new VT. */
@@ -322,14 +363,32 @@ config_defaults(void)
     mice->lastButtons = 0;
     mice->chordMiddle = 0;
 
+    config.num_lpt = 1;
+    pptr = &lpt[0];
+    pptr->prtcmd = "lpr";
+    pptr->prtopt = "%s";
+    pptr->dev = NULL;
+    pptr->file = NULL;
+    pptr->remaining = -1;
+    pptr->delay = 20;
+
+    config.sound = 1;
     config.sb_base = 0x220;
     config.sb_dma = 1;
     config.sb_irq = 5;
     config.sb_dsp = "/dev/dsp";
-    config.sb_mixer = "/dev/mixer";
+    config.sb_mixer = "";
     config.mpu401_base = 0x330;
 
-    config.vnet = 1;
+    config.joy_device[0] = "/dev/js0";
+    config.joy_device[1] = "/dev/js1";
+    config.joy_dos_min = 1;
+    config.joy_dos_max = 150;
+    config.joy_granularity = 1;
+    config.joy_latency = 0;
+
+    config.netdev = "eth0";
+    config.vnet = 0;
 
     memset(config.features, 0, sizeof(config.features));
 }
@@ -468,14 +527,14 @@ void dump_config_status(void *printfunc)
         config.dualmon, config.force_vt_switch, s);
     (*print)("update %d\nfreq %d\nwantdelta %d\nrealdelta %d\n",
         config.update, config.freq, config.wantdelta, config.realdelta);
-    (*print)("timers %d\nkeybint %d\n",
-        config.timers, config.keybint);
+    (*print)("timers %d\n",
+        config.timers);
     (*print)("tty_lockdir \"%s\"\ntty_lockfile \"%s\"\nconfig.tty_lockbinary %d\n",
         config.tty_lockdir, config.tty_lockfile, config.tty_lockbinary);
     (*print)("num_ser %d\nnum_lpt %d\nnum_mice %d\nfastfloppy %d\n",
         config.num_ser, config.num_lpt, config.num_mice, config.fastfloppy);
-    (*print)("emusys \"%s\"\nemubat \"%s\"\nemuini \"%s\"\n",
-        (config.emusys ? config.emusys : ""), (config.emubat ? config.emubat : ""), (config.emuini ? config.emuini : ""));
+    (*print)("emusys \"%s\"\nemuini \"%s\"\n",
+        (config.emusys ? config.emusys : ""), (config.emuini ? config.emuini : ""));
     (*print)("dosbanner %d\nallowvideoportaccess %d\ndetach %d\n",
         config.dosbanner, config.allowvideoportaccess, config.detach);
     (*print)("debugout \"%s\"\n",
@@ -544,6 +603,9 @@ void dump_config_status(void *printfunc)
 
     (*print)("\nSOUND:\nsb_base 0x%x\nsb_dma %d\nsb_irq %d\nmpu401_base 0x%x\nsb_dsp \"%s\"\nsb_mixer \"%s\"\n",
         config.sb_base, config.sb_dma, config.sb_irq, config.mpu401_base, config.sb_dsp, config.sb_mixer);
+    (*print)("\nJOYSTICK:\njoy_device0 \"%s\"\njoy_device1 \"%s\"\njoy_dos_min %i\njoy_dos_max %i\njoy_granularity %i\njoy_latency %i\n",
+        config.joy_device[0], config.joy_device[1], config.joy_dos_min, config.joy_dos_max, config.joy_granularity, config.joy_latency);
+
     if (!printfunc) {
       (*print)("\n--------------end of runtime configuration dump -------------\n");
       (*print)(  "-------------------------------------------------------------\n\n");
@@ -666,13 +728,16 @@ void secure_option_preparse(int *argc, char **argv)
   char * get_option(char *key, int with_arg)
   {
     char *p;
+    char *basename;
     int o = find_option(key, *argc, argv);
     if (!o) return 0;
     o = option_delete(o, argc, argv);
     if (!with_arg) return "";
     if (!with_arg || o >= *argc) return "";
     if (argv[o][0] == '-') {
-      usage();
+      basename = strrchr(argv[0], '/');   /* parse the program name */
+      basename = basename ? basename + 1 : argv[0];
+      usage(basename);
       exit(0);
     }
     p = strdup(argv[o]);
@@ -702,7 +767,6 @@ void secure_option_preparse(int *argc, char **argv)
       exit(0);
     }
     DOSEMU_LIB_DIR = opt;
-    DOSEMU_USERS_FILE = "none";
   }
 
   opt = get_option("--Fimagedir", 1);
@@ -717,6 +781,140 @@ void secure_option_preparse(int *argc, char **argv)
   leave_priv_setting();
 }
 
+static void config_post_process(void)
+{
+    /* console scrub */
+    if (config.X) {
+#ifdef HAVE_KEYBOARD_V1
+	if (!config.X_keycode) {
+	    extern void keyb_layout(int layout);
+	    keyb_layout(-1);
+	    c_printf("CONF: Forcing neutral Keyboard-layout, X-server will translate\n");
+	}
+#endif
+	config.console_video = config.vga = config.graphics = 0;
+	config.emuretrace = 0;	/* already emulated */
+    }
+    else {
+	if (!can_do_root_stuff && config.console_video) {
+	    /* force use of Slang-terminal on console too */
+	    config.console = config.console_video = config.vga = config.graphics = 0;
+	    config.cardtype = 0;
+	    config.vbios_seg = 0;
+	    config.mapped_bios = 0;
+	    fprintf(stderr, "no console on low feature (non-suid root) DOSEMU\n");
+	}
+    }
+    if ((config.vga || config.X) && config.mem_size > 640) {
+	error("$_dosmem = (%d) not allowed for X and VGA console graphics, "
+              "restricting to 640K\n", config.mem_size);
+        config.mem_size = 640;
+    }
+
+    /* UID scrub */
+    if (under_root_login)  c_printf("CONF: running exclusively as ROOT:");
+    else {
+#ifdef RUN_AS_ROOT
+      c_printf("CONF: mostly running as ROOT:");
+#else
+      c_printf("CONF: mostly running as USER:");
+#endif
+    }
+    c_printf(" uid=%d (cached %d) gid=%d (cached %d)\n",
+        geteuid(), get_cur_euid(), getegid(), get_cur_egid());
+
+    /* Speaker scrub */
+#ifdef X86_EMULATOR
+    if (config.cpuemu && config.speaker==SPKR_NATIVE) {
+	c_printf("SPEAKER: can`t use native mode with cpu-emu\n");
+	config.speaker=SPKR_EMULATED;
+    }
+#endif
+
+    check_for_env_autoexec_or_config();
+
+    if (config.pci && !can_do_root_stuff) {
+        c_printf("CONF: Warning: PCI requires root, disabled\n");
+        config.pci = 0;
+    }
+
+    if (config.pktdrv && !can_do_root_stuff) {
+	c_printf("CONF: Warning: root permissions "
+	"required for Packet Driver, disabling!\n");
+	config.pktdrv = 0;
+    }
+}
+
+static config_scrub_t config_scrub_func[100];
+
+/*
+ * DANG_BEGIN_FUNCTION register_config_scrub
+ * 
+ * description: 
+ * register a function Enforce consistency upon the `config` structure after
+ * all values have been set to remove silly option combinations
+ * 
+ * DANG_END_FUNCTION
+ * 
+ */
+int register_config_scrub(config_scrub_t new_config_scrub)
+{
+	int i;
+	int result = -1;
+	for(i = 0; i < sizeof(config_scrub_func)/sizeof(config_scrub_func[0]); i++) {
+		if (!config_scrub_func[i]) {
+			config_scrub_func[i] = new_config_scrub;
+			result = 0;
+			break;
+		}
+	}
+	if (result < 0) {
+		c_printf("register_config_scrub failed > %d config_scrub functions\n",
+			sizeof(config_scrub_func)/sizeof(config_scrub_func[0]));
+	}
+	return result;
+}
+
+/*
+ * DANG_BEGIN_FUNCTION unregister_config_scrub
+ * 
+ * description: 
+ * Complement of register_config_scrub 
+ * This removes a scrub function.
+ * 
+ * DANG_END_FUNCTION
+ * 
+ */
+void unregister_config_scrub( config_scrub_t old_config_scrub)
+{
+	int i;
+	for(i = 0; i < sizeof(config_scrub_func)/sizeof(config_scrub_func[0]); i++) {
+		if (config_scrub_func[i] == old_config_scrub) {
+			config_scrub_func[i] = 0;
+		}
+	}
+}
+
+/*
+ * DANG_BEGIN_FUNCTION config_scrub
+ * 
+ * description: 
+ * Enforce consistency upon the `config` structure after
+ * all values have been set to remove silly option combinations
+ * 
+ * DANG_END_FUNCTION
+ * 
+ */
+static void config_scrub(void)
+{
+	int i;
+	for(i = 0; i < sizeof(config_scrub_func)/sizeof(config_scrub_func[0]); i++) {
+		config_scrub_t func = config_scrub_func[i];
+		if (func) {
+			func();
+		}
+	}
+}
 
 /*
  * DANG_BEGIN_FUNCTION config_init
@@ -741,8 +939,11 @@ config_init(int argc, char **argv)
     char           *dexe_name = 0;
     char usedoptions[256];
 
+    basename = strrchr(argv[0], '/');   /* parse the program name */
+    basename = basename ? basename + 1 : argv[0];
+
     if (argv[1] && !strcmp("--version",argv[1])) {
-      usage();
+      usage(basename);
       exit(0);
     }
 
@@ -753,8 +954,6 @@ config_init(int argc, char **argv)
     memcheck_type_init();
     our_envs_init(0);
     config_defaults();
-    basename = strrchr(argv[0], '/');	/* parse the program name */
-    basename = basename ? basename + 1 : argv[0];
 
 #ifdef X_SUPPORT
     /*
@@ -770,7 +969,8 @@ config_init(int argc, char **argv)
 #endif
 
     opterr = 0;
-    confname = CONFIG_SCRIPT;
+    if (strcmp(config_script_name, DEFAULT_CONFIG_SCRIPT))
+      confname = config_script_path;
     while ((c = getopt(argc, argv, "ABCcF:f:I:kM:D:P:VNtsgh:H:x:KL:m23456e:E:dXY:Z:o:Ou:U:")) != EOF) {
 	usedoptions[(unsigned char)c] = c;
 	switch (c) {
@@ -835,13 +1035,6 @@ config_init(int argc, char **argv)
 	    break;
 	case 'o':
 	    config.debugout = strdup(optarg);
-	    enter_priv_off();
-	    dbg_fd = fopen(config.debugout, "w");
-	    leave_priv_setting();
-	    if (!dbg_fd) {
-		fprintf(stderr, "can't open \"%s\" for writing\n", config.debugout);
-		exit(1);
-	    }
 	    break;
 	case '2': case '3': case '4': case '5': case '6':
 #if 1
@@ -873,13 +1066,34 @@ config_init(int argc, char **argv)
 	}
     }
 
-    if (config_check_only) d.config = 1;
+    if (dbg_fd == 0) {
+        if (config.debugout == NULL) {
+            char *home = getenv("HOME");
+            if (home) {
+                const static char *debugout = "/.dosemu/boot.log";
+                config.debugout = malloc(strlen(home) + strlen(debugout) + 1);
+                strcpy(config.debugout, home);
+                strcat(config.debugout, debugout);
+            }
+        }
+        if (config.debugout != NULL) {
+            enter_priv_off();
+            dbg_fd = fopen(config.debugout, "w");
+            leave_priv_setting();
+            if (!dbg_fd) {
+                fprintf(stderr, "can't open \"%s\" for writing\n", config.debugout);
+                leavedos(1);
+            }
+        }
+    }
+
+    if (config_check_only) set_debug_level('c',1);
 
     if (dexe_name || !strcmp(basename,"dosexec")) {
 	extern void prepare_dexe_load(char *name);
 	if (!dexe_name) dexe_name = argv[optind];
 	if (!dexe_name) {
-	  usage();
+	  usage(basename);
 	  exit(1);
 	}
 	prepare_dexe_load(dexe_name);
@@ -891,7 +1105,7 @@ config_init(int argc, char **argv)
     restore_usedoptions(usedoptions);
 
     if (config.exitearly && !config_check_only)
-	leavedos(0);
+	exit(0);
 
     if (vm86s.cpu_type > config.realcpu) {
     	vm86s.cpu_type = config.realcpu;
@@ -974,7 +1188,6 @@ config_init(int argc, char **argv)
 	case 'K':
 #if 0 /* now dummy, leave it for compatibility */
 	    warn("Keyboard interrupt enabled...this is still buggy!\n");
-	    config.keybint = 1;
 #endif
 	    break;
 	case 'M':{
@@ -986,7 +1199,6 @@ config_init(int argc, char **argv)
 	    }
 	case 'D':
 	    parse_debugflags(optarg, 1);
-	    if (config_check_only) d.config = 1;
 	    break;
 	case 'P':
 	    if (terminal_fd == -1) {
@@ -1047,52 +1259,17 @@ config_init(int argc, char **argv)
 	case '?':
 	default:
 	    fprintf(stderr, "unrecognized option: -%c\n\r", c);
-	    usage();
+	    usage(basename);
 	    fflush(stdout);
 	    fflush(stderr);
 	    _exit(1);
 	}
     }
-    if (config.X) {
-	if (!config.X_keycode) {
-	    extern void keyb_layout(int layout);
-	    keyb_layout(-1);
-	    c_printf("CONF: Forceing neutral Keyboard-layout, X-server will translate\n");
-	}
-	config.console_video = config.vga = config.graphics = 0;
-	config.emuretrace = 0;	/* already emulated */
-    }
-    else {
-	if (!can_do_root_stuff && config.console) {
-	    /* force use of Slang-terminal on console too */
-	    config.console = config.console_video = config.vga = config.graphics = 0;
-	    config.cardtype = 0;
-	    config.vbios_seg = 0;
-	    config.mapped_bios = 0;
-	    fprintf(stderr, "no console on low feature (non-suid root) DOSEMU\n");
-	}
-    }
-    check_for_env_autoexec_or_config();
-    if (under_root_login)  c_printf("CONF: running exclusively as ROOT:");
-    else {
-#ifdef RUN_AS_ROOT
-      c_printf("CONF: mostly running as ROOT:");
-#else
-      c_printf("CONF: mostly running as USER:");
-#endif
-    }
-    c_printf(" uid=%d (cached %d) gid=%d (cached %d)\n",
-        geteuid(), get_cur_euid(), getegid(), get_cur_egid());
-
-#ifdef X86_EMULATOR
-    if (config.cpuemu && config.speaker==SPKR_NATIVE) {
-	c_printf("SPEAKER: can`t use native mode with cpu-emu\n");
-	config.speaker=SPKR_EMULATED;
-    }
-#endif
+    config_post_process();
+    config_scrub();
     if (config_check_only) {
 	dump_config_status(0);
-	usage();
+	usage(basename);
 	leavedos(0);
     }
 }
@@ -1102,9 +1279,6 @@ static void
 check_for_env_autoexec_or_config(void)
 {
     char           *cp;
-    cp = getenv("AUTOEXEC");
-    if (cp)
-	config.emubat = cp;
     cp = getenv("CONFIG");
     if (cp)
 	config.emusys = cp;
@@ -1116,207 +1290,21 @@ check_for_env_autoexec_or_config(void)
       */
 
 #if 0
-    if (config.emubat)
-	fprintf(stderr, "autoexec extension = %s\n", config.emubat);
     if (config.emusys)
 	fprintf(stderr, "config extension = %s\n", config.emusys);
 #endif
 }
 
-/*
- * DANG_BEGIN_FUNCTION parse_debugflags
- * 
- * arguments: 
- * s - string of options.
- * 
- * description: 
- * This part is fairly flexible...you specify the debugging
- * flags you wish with -D string.  The string consists of the following
- * characters: +   turns the following options on (initial state) -
- * turns the following options off a   turns all the options on/off,
- * depending on whether +/- is set 0-9 sets debug levels (0 is off, 9 is
- * most verbose) #   where # is a letter from the valid option list (see
- * docs), turns that option off/on depending on the +/- state.
- * 
- * Any option letter can occur in any place.  Even meaningless combinations,
- * such as "01-a-1+0vk" will be parsed without error, so be careful. Some
- * options are set by default, some are clear. This is subject to my whim.
- * You can ensure which are set by explicitly specifying.
- * 
- * DANG_END_FUNCTION
- */
-int parse_debugflags(const char *s, unsigned char flag)
-{
-    char            c;
-    int ret = 0;
-#ifdef X_SUPPORT
-    const char      allopts[] = "dARWDCvXkiTtsm#pQgcwhIExMnPrSeZ";
-#else
-    const char      allopts[] = "dARWDCvkiTtsm#pQgcwhIExMnPrSeZ";
-#endif
-/*    abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ  */
-/*      *** *** * ** * *** ***  * ***   *   *  *****  ** *  */
-
-    /*
-     * if you add new classes of debug messages, make sure to add the
-     * letter to the allopts string above so that "1" and "a" can work
-     * correctly.
-     */
-
-    dbug_printf("debug flags: %s\n", s);
-    while ((c = *(s++)))
-	switch (c) {
-	case '+':		/* begin options to turn on */
-	    if (!flag)
-		flag = 1;
-	    break;
-	case '-':		/* begin options to turn off */
-	    flag = 0;
-	    break;
-
-	case 'd':		/* disk */
-	    d.disk = flag;
-	    break;
-	case 'R':		/* disk READ */
-	    d.read = flag;
-	    break;
-	case 'W':		/* disk WRITE */
-	    d.write = flag;
-	    break;
-	case 'D':		/* DOS int 21h */
-	    d.dos = flag;
-	    { static int first = 1;
-	      if(first) { set_int21_revectored(d.dos ? 1 : 0); first = 0; }
-	    }
-	    break;
-        case 'C':               /* CDROM */
-	    d.cdrom = flag;
-            break;
-	case 'v':		/* video */
-	    d.video = flag;
-	    break;
-#ifdef X_SUPPORT
-	case 'X':
-	    d.X = flag;
-	    break;
-#endif
-	case 'k':		/* keyboard */
-	    d.keyb = flag;
-	    break;
-	case 'i':		/* i/o instructions (in/out) */
-	    d.io = flag;
-	    break;
-	case 'T':		/* i/o port tracing */
-	{   extern void init_port_traceing(void);
-	    d.io_trace = flag;
-	    if (d.io_trace) init_port_traceing();
-	    break;
-	}
-	case 's':		/* serial */
-	    d.serial = flag;
-	    break;
-	case 'm':		/* mouse */
-	    d.mouse = flag;
-	    break;
-	case '#':		/* default int */
-	    d.defint =flag;
-	    break;
-	case 'p':		/* printer */
-	    d.printer = flag;
-	    break;
-	case 'g':		/* general messages */
-	    d.general = flag;
-	    break;
-	case 'c':		/* configuration */
-	    d.config = flag;
-	    break;
-	case 'w':		/* warnings */
-	    d.warning = flag;
-	    break;
-	case 'h':		/* hardware */
-	    d.hardware = flag;
-	    break;
-	case 'I':		/* IPC */
-	    d.IPC = flag;
-	    break;
-	case 'E':		/* EMS */
-	    d.EMS = flag;
-	    break;
-	case 'x':		/* XMS */
-	    d.xms = flag;
-	    break;
-	case 'M':		/* DPMI */
-	    d.dpmi = flag;
-	    break;
-	case 'n':		/* IPX network */
-	    d.network = flag;
-	    break;
-	case 'P':		/* Packet driver */
-	    d.pd = flag;
-	    break;
-	case 'Q':		/* Mapping driver */
-	    d.mapping = flag;
-	    break;
-	case 'r':		/* PIC */
-	    d.request = flag;
-	    break;
-	case 'S':		/* SOUND */
-	    d.sound = flag;
-	    break;
-	case 'A':		/* ASPI */
-	    d.aspi = flag;
-	    break;
-	case 'Z':
-	    d.pci = flag;       /* PCI */
-	    break;
-#ifdef X86_EMULATOR
-	case 'e':		/* cpu-emu */
-	    d.emu = flag;
-	    break;
-#endif
-#ifdef TRACE_DPMI
-	case 't':		/* dpmi */
-	    d.dpmit = flag;
-	    break;
-#endif
-	case 'a':{		/* turn all on/off depending on flag */
-		char           *newopts = (char *) malloc(strlen(allopts) + 2);
-
-		newopts[0] = flag ? '+' : '-';
-		newopts[1] = 0;
-		strcat(newopts, allopts);
-#if 1 /* we need to _always_ remove these flags here !!! #ifdef X86_EMULATOR */
-		/* hack-do not set 'e,t' flags if not explicitly specified */
-		{char *p=newopts;
-		 while (*p) {if ((*p=='e')||(*p=='t')) *p='g'; p++;}}
-#endif
-		parse_debugflags(newopts, flag);
-		free(newopts);
-	    }
-	    break;
-	case '0'...'9':	/* set debug level, 0 is off, 9 is most
-				 * verbose */
-	    flag = c - '0';
-	    break;
-	default:
-	    fprintf(stderr, "Unknown debug-msg mask: %c\n\r", c);
-	    dbug_printf("Unknown debug-msg mask: %c\n", c);
-	    ret = 1;
-	}
-  if (config_check_only) d.config = 1;
-  return ret;
-}
-
 static void
-usage(void)
+usage(char *basename)
 {
     fprintf(stderr,
 	"dosemu-" VERSTR "\n\n"
 	"USAGE:\n"
-	"  dos  [-ABCckbVNtsgxKm23456ez] [-h{0|1|2}] [-H dflags \\\n"
+	"  %s  [-ABCckbVNtsgxKm23456ez] [-h{0|1|2}] [-H dflags \\\n"
 	"       [-D flags] [-M SIZE] [-P FILE] [ {-F|-L} File ] \\\n"
 	"       [-u confvar] [-f dosrcFile] [-o dbgfile] 2> vital_logs\n"
-	"  dos --version\n\n"
+	"  %s --version\n\n"
 	"    -2,3,4,5,6 choose 286, 386, 486 or 586 or 686 CPU\n"
 	"    -A boot from first defined floppy disk (A)\n"
 	"    -B boot from second defined floppy disk (B) (#)\n"
@@ -1326,18 +1314,14 @@ usage(void)
 	"    -d detach (?)\n"
 #ifdef X_SUPPORT
 	"    -X run in X Window (#)\n"
+#endif
 /* seems no longer valid bo 18.7.95
 	"    -Y NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n"
 	"    -Z NAME use FIFO NAME for mouse (only with x2dos!)\n"
 */
-	"    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWXcdghikmnprsvwx}\n"
-#else				/* X_SUPPORT */
-	"    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWcdghikmnprsvwx}\n"
-#endif				/* X_SUPPORT */
-	"       #=defint  C=cdrom    D=dos    E=ems       I=ipc     M=dpmi\n"
-	"       P=packet  R=diskread S=sound  W=diskwrite c=config  d=disk\n"
-	"       g=general h=hardware i=i/o    k=keyb      m=mouse   n=ipxnet\n"
-	"       p=printer r=pic      s=serial v=video     w=warning x=xms\n"
+    ,basename, basename);
+    print_debug_usage(stderr);
+    fprintf(stderr,
 	"    -E STRING pass DOS command on command line\n"
 	"    -e SIZE enable SIZE K EMS RAM\n"
 	"    -F use File as global config-file\n"
@@ -1353,8 +1337,8 @@ usage(void)
 	"    -M set memory size to SIZE kilobytes (!)\n"
 	"    -m enable mouse support (!#)\n"
 	"    -N No boot of DOS\n"
-	"    -O write debugmessages to stderr\n"
-	"    -o FILE put debugmessages in file\n"
+	"    -O write debug messages to stderr\n"
+	"    -o FILE put debug messages in file\n"
 	"    -P copy debugging output to FILE\n"
 	"    -t try new timer code (#)\n"
 	"    -u set user configuration variable 'confvar' prefixed by 'u_'.\n"
@@ -1363,9 +1347,9 @@ usage(void)
 	"    -x SIZE enable SIZE K XMS RAM\n"
 	"    --version, print version of dosemu\n"
 	"    (!) BE CAREFUL! READ THE DOCS FIRST!\n"
-	"    (%%) require dos be run as root (i.e. suid)\n"
+	"    (%%) require DOSEMU be run as root (i.e. suid)\n"
 	"    (#) options do not fully work yet\n\n"
-	"xdos [options]           == dos [options] -X\n"
-	"dosexec [options] <file> == dos [options] -L <file>\n"
-    );
+	"xdos [options]           == %s [options] -X\n"
+	"dosexec [options] <file> == %s [options] -L <file>\n"
+    ,basename, basename);
 }
