@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -30,6 +30,7 @@
 #include "vc.h"
 #include "mouse.h"
 #include "port.h"
+#include "joystick.h"
 #ifdef USING_NET
 #include "pktdrvr.h"
 #include "ipx.h"
@@ -50,12 +51,6 @@
 #include "mapping.h"
 
 
-#ifdef USING_NET
-extern void pkt_check_receive_quick(void);
-/* flag to activate use of pic by packet driver */
-#define PICPKT 1
-#endif
-
 #if 0
 static inline void dbug_dumpivec(void)
 {
@@ -191,26 +186,26 @@ void hardware_setup(void)
   extern void  do_irq1(void);
   extern void rtc_int8(void);
   /* PIC init */
-  pic_seti(PIC_IRQ0, timer_int_engine, 0);  /* do_irq0 in pic.c */
+  pic_seti(PIC_IRQ0, timer_int_engine, 0, NULL);  /* do_irq0 in pic.c */
   pic_unmaski(PIC_IRQ0);
   pic_request(PIC_IRQ0);  /* start timer */
-  pic_seti(PIC_IRQ1, do_irq1, 0); /* do_irq1 in dosio.c   */
+  pic_seti(PIC_IRQ1, do_irq1, 0, NULL); /* do_irq1 in dosio.c   */
   pic_unmaski(PIC_IRQ1);
-  pic_seti(PIC_IRQ8, rtc_int8, 0);
+  pic_seti(PIC_IRQ8, rtc_int8, 0, NULL);
   pic_unmaski(PIC_IRQ8);
   if (mice->intdrv || mice->type == MOUSE_PS2 || mice->type == MOUSE_IMPS2) {
-    pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0);
+    pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0, do_mouse_irq);
     pic_unmaski(PIC_IMOUSE);
   }
 #ifdef USING_NET
 #ifdef IPX
-  pic_seti(PIC_IPX, IPXCallRel, 0);
+  pic_seti(PIC_IPX, do_irq, 0, IPXCallRel);
   pic_unmaski(PIC_IPX);
 #endif
 #ifdef PICPKT
-  pic_seti(PIC_NET, pkt_check_receive_quick, 0x61);
+  pic_seti(PIC_NET, pkt_check_receive_quick, 0x61, pkt_receiver_callback);
 #else
-  pic_seti(PIC_NET, pkt_check_receive_quick, 0);
+  pic_seti(PIC_NET, pkt_check_receive_quick, 0, NULL);
 #endif
   pic_unmaski(PIC_NET);
 #endif
@@ -333,7 +328,7 @@ static inline void bios_mem_setup(void)
 {
   int b;
 
-  /* show 0 serial ports and 3 parallel ports, maybe a mouse, and the
+  /* show 0 serial ports and 3 parallel ports, maybe a mouse, game card and the
    * configured number of floppy disks
    */
   CONF_NFLOP(configuration, config.fdisks);
@@ -342,6 +337,12 @@ static inline void bios_mem_setup(void)
   if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_IMPS2) || (mice->intdrv))
     configuration |= CONF_MOUSE;
 
+  /* 
+   * real BIOS' don't seem to set this bit but setting it can't do any harm
+   * and improves joystick detection for the game, Alley Cat, anyway :)
+   */
+  if (joy_exist ()) configuration |= CONF_GAME;
+
   if (config.mathco)
     configuration |= CONF_MATHCO;
 
@@ -391,7 +392,8 @@ void memory_init(void)
 #endif
 
 #ifdef USING_NET
-  pkt_init(0x60);              /* Install the new packet driver interface */
+  if (config.pktdrv)
+    pkt_init(0x60);              /* Install the new packet driver interface */
 #endif
 
   bios_mem_setup();            /* setup values in BIOS area */
@@ -410,11 +412,6 @@ void memory_init(void)
     init_vga_card();
   }
 
-  if (config.exitearly) {
-    dbug_printf("Leaving DOS before booting\n");
-    leavedos(0);
-  }
-
   if (first_call) {
     ems_init();                /* initialize ems */
     xms_init();                /* initialize xms */
@@ -439,6 +436,7 @@ void device_init(void)
   check_console();
 
   scr_state_init();
+#ifdef HAVE_KEYBOARD_V1
   if (!keyb_server_init()) {
     error("can't init keyboard server\n");
     leavedos(19);
@@ -447,6 +445,7 @@ void device_init(void)
     error("can't open keyboard client\n");
     leavedos(19);
   }
+#endif
    
   if (!config.vga)
     config.allowvideoportaccess = 0;
@@ -457,7 +456,6 @@ void device_init(void)
 		     console_speaker_on, console_speaker_off);
   }
   iodev_init();
-  dosemu_mouse_init();
   printer_init();
   disk_init();
 }
