@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -35,15 +35,12 @@
 #include "int.h"
 #include "fatfs.h"
 #include "utilities.h"
-
-#ifdef NEED_LLSEEK_PROTOTYPE
-  /* well, if we don't have llseek prototype,
-   * we most likely won't have __loff_t too, hence using long long
-   */
-  extern long long libless_llseek(int fd, long long offset, int origin);
-  #define llseek libless_llseek
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
 #endif
 
+#define llseek libless_llseek
+
 static int disks_initiated = 0;
 
 #define FDISKS config.fdisks
@@ -217,14 +214,20 @@ read_sectors(struct disk *dp, char *buffer, long head, long sector,
       error("Sector not found in read_sector, error = %s!\n", strerror(errno));
       return -DERR_NOTFOUND;
     }
+#ifdef X86_EMULATOR
+    if (config.cpuemu>1)
+	tmpread = e_dos_read(dp->fdesc, buffer, count * SECTOR_SIZE - already);
+    else
+#endif
     tmpread = RPT_SYSCALL(read(dp->fdesc, buffer, count * SECTOR_SIZE - already));
   }
 
   if(tmpread != -1) {
-    if(d.disk > 8) dump_disk_blks(buffer, count - already / SECTOR_SIZE, SECTOR_SIZE);
+    if (debug_level('d') > 8) dump_disk_blks(buffer, count - already / SECTOR_SIZE, SECTOR_SIZE);
     return tmpread + already;
   }
   else {
+/**/ dbug_printf("disks.c: read failed\n");
     return -DERR_ECCERR;
   }
 }
@@ -647,7 +650,7 @@ disk_close(void)
   if (!disks_initiated) return;  /* just to be safe */
   for (dp = disktab; dp < &disktab[FDISKS]; dp++) {
     if (dp->removeable && dp->fdesc >= 0) {
-      d_printf("DISK: %s: Closing a disk\n",dp->dev_name);
+      d_printf("DISK: Closing disk %s\n",dp->dev_name);
       (void) close(dp->fdesc);
       dp->fdesc = -1;
     }
@@ -881,6 +884,12 @@ disk_init(void)
     else dp->rdonly = dp->wantrdonly;
   }
 
+  if (!FDISKS && use_bootdisk) {
+  /* if we don't have any configured floppies, we have to use bootdisk instead */
+    memcpy(&disktab[0], &bootdisk, sizeof(bootdisk));
+    FDISKS++;	/* now we have one */
+  }
+
   /*
    * Open hard disks
    */
@@ -1001,10 +1010,9 @@ int13(u_char i)
   int checkdp_val;
 
   disk = LO(dx);
-  if (disk < FDISKS) {
-    if (!disk && use_bootdisk)
-      dp = &bootdisk;
-    else
+  if (!disk && use_bootdisk)
+    dp = &bootdisk;
+  else if (disk < FDISKS) {
       dp = &disktab[disk];
     switch (HI(ax)) {
       #define DISKETTE_MOTOR_TIMEOUT (*((unsigned char *)0x440))
@@ -1021,6 +1029,8 @@ int13(u_char i)
   else
     dp = NULL;
 
+  d_printf("INT13: disk=%#x ah=%#x dp=%p\n", LO(dx), HI(ax), dp);
+
   /* this is a bad hack to ensure that the cached blocks aren't.
    * Linux only checks disk change on open()
    */
@@ -1053,6 +1063,10 @@ int13(u_char i)
       track |= (HI(dx) & 0xc0) << 4;
     buffer = SEG_ADR((char *), es, bx);
     number = LO(ax);
+#ifdef X86_EMULATOR
+    e_printf("DISK %d read [h:%d,s:%d,t:%d](%d)->%04x:%04x\n",
+	     disk, head, sect, track, number, REG(es), LWORD(ebx));
+#endif
 
     if (checkdp_val || head >= dp->heads ||
 	sect >= dp->sectors || track >= dp->tracks) {
@@ -1334,7 +1348,7 @@ int13(u_char i)
 	NOCARRY;
       }
       else {
-	error("gettype: no disk %d\n", disk);
+	d_printf("gettype: no disk %d\n", disk);
 	HI(ax) = 0;		/* disk not there */
 	REG(eflags) |= CF;	/* error */
       }
@@ -1403,7 +1417,7 @@ int13(u_char i)
     diskaddr->blocks = 0;
 
     if (checkdp(dp) || track >= dp->tracks) {
-      error("ERROR: Sector not found, AH=0x42!\n");
+      error("Sector not found, AH=0x42!\n");
       d_printf("DISK %d ext read [h:%d,s:%d,t:%d](%d)->%p\n",
 	       disk, head, sect, track, number, (void *) buffer);
       if (dp) {
@@ -1427,7 +1441,7 @@ int13(u_char i)
       break;
     }
     else if (res & 511) {	/* must read multiple of 512 bytes */
-      error("ERROR: sector_corrupt 1, return = %d!\n", res);
+      error("sector_corrupt 1, return = %d!\n", res);
       HI(ax) = DERR_BADSEC;	/* sector corrrupt */
       CARRY;
       break;
@@ -1455,7 +1469,7 @@ int13(u_char i)
     diskaddr->blocks = 0;
 
     if (checkdp(dp) || track >= dp->tracks) {
-      error("ERROR: Sector not found, AH=0x42!\n");
+      error("Sector not found, AH=0x42!\n");
       d_printf("DISK %d ext write [h:%d,s:%d,t:%d](%d)->%p\n",
 	       disk, head, sect, track, number, (void *) buffer);
       if (dp) {
@@ -1492,7 +1506,7 @@ int13(u_char i)
       break;
     }
     else if (res & 511) {	/* must read multiple of 512 bytes */
-      error("ERROR: sector_corrupt 1, return = %d!\n", res);
+      error("sector_corrupt 1, return = %d!\n", res);
       HI(ax) = DERR_BADSEC;	/* sector corrrupt */
       CARRY;
       break;
@@ -1525,7 +1539,7 @@ int13(u_char i)
     params = SEG_ADR((struct ibm_ms_drive_params *), ds, si);
 
     if (checkdp(dp)) {
-      error("ERROR: Invalid drive, AH=0x48!\n");
+      error("Invalid drive, AH=0x48!\n");
       if (dp) {
 	  d_printf("DISK dev %s GEOM %d heads %d sects %d trk\n",
 		   dp->dev_name, dp->heads, dp->sectors, dp->tracks);
@@ -1587,10 +1601,12 @@ int13(u_char i)
 void
 floppy_tick(void)
 {
-  static int secs = 0;
+  static int ticks = 0;
 
-  if (++secs == config.fastfloppy) {
+  if (++ticks >= config.fastfloppy) {
     disk_close();
-    secs = 0;
+    if (debug_level('d') > 2)
+      d_printf("FLOPPY: flushing after %d ticks\n", ticks);
+    ticks = 0;
   }
 }
