@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -37,14 +37,6 @@
 #include "bitops.h"
 #include "mapping.h"
 
-extern void DOSEMUMouseEvents(void);
-
-extern void xms_init(void);
-extern void video_memory_setup(void);
-extern void dump_kbuffer(void);
-extern int int_count[];
-extern int in_readkeyboard, keybint;
-
 #ifndef PAGE_SIZE
 #define PAGE_SIZE	4096
 #endif
