@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -25,7 +25,7 @@
  * Debug level.
  * 0 - normal / 1 - useful / 2 - too much
  */
-#define DEBUG_FATFS	2
+#define DEBUG_FATFS	1
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -424,9 +424,9 @@ void scan_dir(fatfs_t *f, unsigned oi)
   DIR *dir;
   struct dirent* dent;
   struct stat sb;
-  char *s, *name;
+  char *s, *name, *buf, *buf_ptr;
   unsigned u, scans = 1;
-  int i, j, fd;
+  int i, j, fd, size;
   char *sf[3] = { NULL, NULL, NULL };
 
   // just checking...
@@ -493,7 +493,30 @@ void scan_dir(fatfs_t *f, unsigned oi)
       while((dent = readdir(dir))) {
         if(!strcasecmp(dent->d_name, "io.sys")) f->sys_type |= 1;
         if(!strcasecmp(dent->d_name, "msdos.sys")) f->sys_type |= 2;
-        if(!strcasecmp(dent->d_name, "ibmbio.com")) f->sys_type |= 4;
+        if(!strcasecmp(dent->d_name, "ibmbio.com")) {
+	  f->sys_type |= 4;
+          if((s = full_name(f, oi, dent->d_name))) {
+            if(!stat(s, &sb)) {
+              if(S_ISREG(sb.st_mode)) {
+                if((fd = open(s, O_RDONLY)) != -1) {
+                  buf = malloc(sb.st_size + 1);
+		  size = read(fd, buf, sb.st_size);
+		  if (size > 0) {
+		    buf[size] = 0;
+		    buf_ptr = buf;
+		    while (!strstr(buf_ptr, "PC DOS") && buf_ptr < buf + size) {
+		      buf_ptr += strlen(buf_ptr) + 1;
+		    }
+		    if (buf_ptr < buf + size)
+		      f->sys_type |= 0x40;
+		  }
+                  free(buf);
+                  close(fd);
+                }
+              }
+            }
+          }
+	}
         if(!strcasecmp(dent->d_name, "ibmdos.com")) f->sys_type |= 8;
         if(!strcasecmp(dent->d_name, "ipl.sys")) f->sys_type |= 0x10;
         if(!strcasecmp(dent->d_name, "kernel.sys")) f->sys_type |= 0x20;
@@ -507,8 +530,14 @@ void scan_dir(fatfs_t *f, unsigned oi)
       sf[1] = "msdos.sys";
       scans = 3;
     }
+    if(f->sys_type & 0x40) {
+      f->sys_type = 0x40;		/* PC-DOS */
+      sf[0] = "ibmbio.com";
+      sf[1] = "ibmdos.com";
+      scans = 3;
+    }
     if((f->sys_type & 0x0c) == 0x0c) {
-      f->sys_type = 0x0c;	/* PC-DOS */
+      f->sys_type = 0x0c;	/* DR-DOS */
       sf[0] = "ibmbio.com";
       sf[1] = "ibmdos.com";
       scans = 3;
@@ -628,7 +657,6 @@ void add_object(fatfs_t *f, unsigned parent, char *name)
   static int first = 1;
   static int exxx = 0;
   static char esys[16] = "";
-  static char ebat[16] = "";
 
   if (first) {
     first = 0;
@@ -637,11 +665,6 @@ void add_object(fatfs_t *f, unsigned parent, char *name)
       esys[12] =0;
       exxx += 2;
     }
-    if (config.emubat) {
-      snprintf(ebat, 8+1+3+1,"autoexec.%s", config.emubat);
-      ebat[12] =0;
-      exxx += 2;
-    }
   }
 
   if (!parent && exxx) {
@@ -650,21 +673,11 @@ void add_object(fatfs_t *f, unsigned parent, char *name)
       exxx -= 1;
       return;
     }
-    if (ebat[0] && !strcmp(name, "autoexec.bat")) {
-      fatfs_deb("add_object: ignored real %s in favour of %s\n", name, ebat);
-      exxx -= 1;
-      return;
-    }
     if (esys[0] && !strcmp(name, esys)) {
       tmp_o.is.faked_sys = 1;
       fatfs_deb("add_object: faked %s as DOS entry CONFIG.SYS\n", esys);
       exxx -= 1;
     }
-    else if (ebat[0] && !strcmp(name, ebat)) {
-      tmp_o.is.faked_bat = 1;
-      fatfs_deb("add_object: faked %s as DOS entry AUTOEXEC.BAT\n", ebat);
-      exxx -= 1;
-    }
   }
   if(!(strcmp(name, ".") && strcmp(name, ".."))) return;
 
@@ -739,7 +752,6 @@ unsigned make_dos_entry(fatfs_t *f, obj_t *o, unsigned char **e)
 
   s = o->name;
   if (o->is.faked_sys) s = "config.sys";
-  if (o->is.faked_bat) s = "autoexec.bat";
 
   if(o->is.this_dir) {
     s = ".";
@@ -1115,6 +1127,7 @@ void build_boot_blk(fatfs_t *f)
 
   switch(f->sys_type) {
     case 0x03:
+    case 0x40:
       i = read_data(f, 0);
       if(i || f->sec[0] != 'M' || f->sec[1] != 'Z') {
         /* for IO.SYS, MS-DOS version < 7 */
