@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,13 +27,7 @@
 
 #ifdef __linux__
 #include <signal.h>
-#if GLIBC_VERSION_CODE >= 2000
 #include <sys/vt.h>
-#else
-#include <linux/vt.h>
-#endif
-#include <linux/fd.h>
-#include <linux/hdreg.h>
 #include <sys/vm86.h>
 #include <syscall.h>
 #endif
@@ -75,11 +69,8 @@
 #endif
 
 #include "dosemu_select.h"
+#include "userhook.h"
   
-#ifdef SIG
-extern SillyG_t *SillyG;
-#endif
-
 #ifndef PAGE_SIZE
 #define PAGE_SIZE       4096
 #endif
@@ -182,8 +173,6 @@ io_select(fd_set fds)
         }
       }
       {
-	extern int uhook_fdin;
-	extern void uhook_input(void);
 	if (uhook_fdin != -1) if (FD_ISSET(uhook_fdin, &fds)) uhook_input();
       }
 
