@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -24,6 +24,7 @@
 #include "config.h"
 #include "emu.h"
 #include "bios.h"
+#include "int.h"
 #include "memory.h"
 #include "video.h"		/* video base address */
 #include "mouse.h"
@@ -39,6 +40,7 @@
 #include "keyboard.h"
 
 #include "priv.h"
+#include "iodev.h"
 
 extern int get_perm ();
 extern int release_perm ();
@@ -211,14 +213,14 @@ mouse_helper(void)
       m_printf("MOUSE Vertical speed out of range. ERROR!\n");
       LWORD(eax) = 1;
     } else 
-      mouse.speed_y = LO(cx);
+      mouse.init_speed_y = mouse.speed_y = LO(cx);
     break;
   case 5:				/* Set horizontal speed */
     if ((LO(cx) < 1) || (LO(cx) > 255)) {
       m_printf("MOUSE Horizontal speed out of range. ERROR!\n");
       LWORD(eax) = 1;
     } else
-      mouse.speed_x = LO(cx);
+      mouse.init_speed_x = mouse.speed_x = LO(cx);
     break;
   case 6:				/* Ignore horz/vert selection */
     if (LO(cx) == 1) 
@@ -255,9 +257,12 @@ mouse_helper(void)
   }
 }
 
-void
+int
 mouse_int(void)
 {
+  if (!mice->intdrv || (!mice->enabled && LWORD(eax) != 0x20)) 
+    return 0;
+  
   m_printf("MOUSEALAN: int 0x%x ebx=%x\n", LWORD(eax), LWORD(ebx));
   switch (LWORD(eax)) {
   case 0x00:			/* Mouse Reset/Get Mouse Installed Flag */
@@ -505,6 +510,8 @@ mouse_int(void)
 
   if (mouse.cursor_on == 0)
      mouse_update_cursor();
+
+  return 1;
 }
 
 void
@@ -788,14 +795,8 @@ mouse_reset_to_current_video_mode(void)
    * then in mouse_reset, as it gets called more often.
    * -- Eric Biederman 29 May 2000
    */
-  if (mice->type != MOUSE_X) {
-    mouse.speed_x = 8;
-    mouse.speed_y = 16;
-  } else {
-    /* default to 1 to 1 scaling for apps tha read mickeys */
-    mouse.speed_x = 8;
-    mouse.speed_y = 8;
-  }
+   mouse.speed_x = mouse.init_speed_x;
+   mouse.speed_y = mouse.init_speed_y;
 
  /*
   * Here we make sure text modes are resolved properly, according to the
@@ -882,10 +883,6 @@ mouse_reset_to_current_video_mode(void)
   mouse.maxx = current_video.width << mouse.xshift;
   mouse.maxy = current_video.height << mouse.yshift;
 
-  /* Find the center of the screen */
-  mouse.x = (mouse.minx + mouse.maxx) / 2;
-  mouse.y = (mouse.miny + mouse.maxy) / 2;
-
   /* force update first time after reset */
   mouse.oldrx = -1;
   
@@ -1219,7 +1216,7 @@ mouse_disable_internaldriver()
   LWORD(es) = mouse.cs;
   LWORD(ebx) = mouse.ip;
 
-  mice->intdrv = FALSE;
+  mice->enabled = FALSE;
 
   m_printf("MOUSE: Disable InternalDriver\n");
 }
@@ -1227,7 +1224,7 @@ mouse_disable_internaldriver()
 void
 mouse_enable_internaldriver()
 {
-  mice->intdrv = TRUE;
+  mice->enabled = TRUE;
   SETIVEC(0x33, Mouse_SEG, Mouse_INT);
 #ifndef USE_NEW_INT
   SETIVEC(0x74, Mouse_SEG, Mouse_INT74);
@@ -1240,7 +1237,87 @@ mouse_enable_internaldriver()
 }
 
 
-
+#ifdef HAVE_UNICODE_KEYB
+/* Keyboard mouse control :)  */
+void mouse_keyboard(Boolean make, t_keysym key)
+{
+	static struct keyboard_mouse_state {
+		int l, r, u, d;
+		int ul, ur, dl, dr;
+		int lbutton, mbutton, rbutton; 
+	} state;
+	int dx, dy;
+	switch (key) {
+	case KEY_MOUSE_DOWN:
+		state.d = !!make;
+		break;
+	case KEY_MOUSE_LEFT:
+		state.l = !!make;
+		break;
+	case KEY_MOUSE_RIGHT:
+		state.r = !!make;
+		break;
+	case KEY_MOUSE_UP:
+		state.u = !!make;
+		break;
+	case KEY_MOUSE_UP_AND_LEFT:
+		state.ul = make;
+		break;
+	case KEY_MOUSE_UP_AND_RIGHT:
+		state.ur = make;
+		break;
+	case KEY_MOUSE_DOWN_AND_LEFT:
+		state.dl = make;
+		break;
+	case KEY_MOUSE_DOWN_AND_RIGHT:
+		state.dr = make;
+		break;
+	case KEY_MOUSE_BUTTON_LEFT:
+		state.lbutton = make;
+		break;
+	case KEY_MOUSE_BUTTON_RIGHT:
+		state.rbutton = make;
+		break;
+	case KEY_MOUSE_BUTTON_MIDDLE:
+		state.mbutton = make;
+		break;
+	default:
+		m_printf("MOUSE: keyboard_mouse(), key 0x%02x unknown!\n", key);
+		break;
+	}
+	dx = dy = 0;
+	if (state.d) {
+		dy += 1;
+	}
+	if (state.dl) {
+		dy += 1;
+		dx -= 1;
+	}
+	if (state.dr) {
+		dy += 1;
+		dx += 1;
+	}
+	if (state.u) {
+		dy -= 1;
+	}
+	if (state.ul) {
+		dy -= 1;
+		dx -= 1;
+	}
+	if (state.ur) {
+		dy -= 1;
+		dx += 1;
+	}
+	if (state.r) {
+		dx += 1;
+	}
+	if (state.l) {
+		dx -= 1;
+	}
+	mouse_move_relative(dx, dy);
+	mouse_move_buttons(state.lbutton, state.mbutton, state.rbutton);
+}
+#else
 /* XXX - something's wrong here. Shouldn't the first 4 cases move the cursor?
  */
 void mouse_keyboard(int sc)
@@ -1272,6 +1349,7 @@ void mouse_keyboard(int sc)
     m_printf("MOUSE: keyboard_mouse(), sc 0x%02x unknown!\n", sc);
   }
 }
+#endif
 
 
 static void mouse_round_coords(void)
@@ -1466,65 +1544,6 @@ void mouse_move_absolute(int x, int y, int x_range, int y_range)
 	   mouse_move();
 }
 
-void 
-fake_int(void)
-{
-  unsigned char *ssp;
-  unsigned long sp;
-
-  m_printf("MOUSE: fake_int: CS:IP %04x:%04x\n",LWORD(cs),LWORD(eip));
-  ssp = (unsigned char *)(LWORD(ss)<<4);
-  sp = (unsigned long) LWORD(esp);
-
-  pushw(ssp, sp, vflags);
-  pushw(ssp, sp, LWORD(cs));
-  pushw(ssp, sp, LWORD(eip));
-  LWORD(esp) -= 6;
-#if 0
-  do_hard_int(0x74);
-#endif
-  
-}
-
-void 
-fake_call(int cs, int ip)
-{
-  unsigned char *ssp;
-  unsigned long sp;
-
-  ssp = (unsigned char *)(LWORD(ss)<<4);
-  sp = (unsigned long) LWORD(esp);
-
-  m_printf("MOUSE: fake_call() shows S: %04x:%04x, CS:IP %04x:%04x\n",
-	   LWORD(ss), LWORD(esp), cs, ip);
-  pushw(ssp, sp, cs);
-  pushw(ssp, sp, ip);
-  LWORD(esp) -= 4;
-}
-
-void 
-fake_pusha(void)
-{
-  unsigned char *ssp;
-  unsigned long sp;
-
-  ssp = (unsigned char *)(LWORD(ss)<<4);
-  sp = (unsigned long) LWORD(esp);
-
-  pushw(ssp, sp, LWORD(eax));
-  pushw(ssp, sp, LWORD(ecx));
-  pushw(ssp, sp, LWORD(edx));
-  pushw(ssp, sp, LWORD(ebx));
-  pushw(ssp, sp, LWORD(esp));
-  pushw(ssp, sp, LWORD(ebp));
-  pushw(ssp, sp, LWORD(esi));
-  pushw(ssp, sp, LWORD(edi));
-  LWORD(esp) -= 16;
-  pushw(ssp, sp, REG(ds));
-  pushw(ssp, sp, REG(es));
-  LWORD(esp) -= 4;
-}
-
 /*
  * add the event to the current event mask
  */
@@ -1538,17 +1557,18 @@ mouse_delta(int event)
  * call user event handler
  */
 void
-mouse_event()
+do_mouse_irq()
 {
-  if (mouse.mask & mouse_events && (mouse.cs || mouse.ip)) {
-    if(in_dpmi && !in_dpmi_dos_int 
-          && !((mouse.cs == DPMI_SEG) && 
-          ((Bit32u)mouse.ip == (DPMI_OFF+HLT_OFF(DPMI_mouse_callback)))))
-                  fake_pm_int();
+    if(in_dpmi && !in_dpmi_dos_int)
+      fake_pm_int();
 
-    /* push iret frame on _SS:_SP. At F000:20F7 (bios.S) we get an
+    /* push iret frame on _SS:_SP. At F000:2146 (bios.S) we get an
      * iret and return to _CS:_IP */
-    fake_int();
+    fake_int(LWORD(cs), LWORD(eip));
+
+    /* push iret frame. At F000:20F7 (bios.S) we get an
+     * iret and return to F000:2140 for EOI */
+    fake_int(Mouse_SEG, Mouse_ROUTINE_OFF);
 
     /* push all 16-bit regs plus _DS,_ES. At F000:20F4 (bios.S) we find
      * 'pop es; pop ds; popa' */
@@ -1572,22 +1592,22 @@ mouse_event()
 
     REG(ds) = *mouse.csp;	/* put DS in user routine */
 
-    if (in_dpmi && 
-	(REG(cs) == DPMI_SEG) 
-	&& (REG(eip) == (DPMI_OFF+HLT_OFF(DPMI_mouse_callback))))
-	run_pm_mouse();
-    
     m_printf("MOUSE: event %d, x %d ,y %d, mx %d, my %d, b %x\n",
 	     mouse_events, mouse.x, mouse.y, mouse.maxx, mouse.maxy, LWORD(ebx));
     m_printf("MOUSE: "
 	     "should call %04x:%04x (actually %04x:%04x)"
 	     ".........jumping to %04x:%04x\n",
 	     mouse.cs, mouse.ip, *mouse.csp, *mouse.ipp,
-	     LWORD(cs), LWORD(eip));	
-  }
-  mouse_events = 0;
+	     LWORD(cs), LWORD(eip));
 }
 
+void
+mouse_event()
+{
+  if (mouse.mask & mouse_events && (mouse.cs || mouse.ip))
+    do_irq();
+  mouse_events = 0;
+}
 
 /* unconditional mouse cursor update */
 static void
@@ -1745,6 +1765,8 @@ dosemu_mouse_init(void)
   mouse.min_max_x = 640;
   mouse.min_max_y = 200;
 
+  mice->enabled = TRUE;
+
   /* we'll admit we have three buttons if the user asked us to emulate
   	one or else we think the mouse actually has a third button. */
   if (mice->emulate3buttons || mice->has3buttons)
@@ -1769,6 +1791,7 @@ dosemu_mouse_init(void)
       mice->fd = DOS_SYSCALL(open(mice->dev, O_RDWR | O_NONBLOCK));
       leave_priv_setting();
       if (mice->fd == -1) {
+	error("Cannot open internal mouse device %s\n",mice->dev);
  	mice->intdrv = FALSE;
  	mice->type = MOUSE_NONE;
  	mice->add_to_io_select = 0;
@@ -1778,21 +1801,7 @@ dosemu_mouse_init(void)
       add_to_io_select(mice->fd, mice->add_to_io_select);
       DOSEMUSetupMouse();
       memcpy(p,mouse_ver,sizeof(mouse_ver));
-    }
-    else if ((mice->type == MOUSE_PS2) || (mice->type == MOUSE_IMPS2) ||
-	     (mice->type == MOUSE_BUSMOUSE)) {
-      enter_priv_on();
-      mice->fd = open(mice->dev, O_RDWR | O_NONBLOCK);
-      leave_priv_setting();
-      mice->add_to_io_select = 0;
-      if (mice->fd == -1) {
-	  error("Cannot open internal mouse device %s\n",mice->dev);
-	  mice->intdrv = FALSE;
-	  mice->type = MOUSE_NONE;
-	  return;
-      }
-      add_to_io_select(mice->fd, mice->add_to_io_select); 
-      memcpy(p,mouse_ver,sizeof(mouse_ver));
+      iodev_add_device(mice->dev);
     }
     else {
       int x;
@@ -1821,6 +1830,15 @@ dosemu_mouse_init(void)
     DOSEMUSetupMouse();
     memcpy(p,mouse_ver,sizeof(mouse_ver));
   }
+
+  if (mice->type != MOUSE_X) {
+    mouse.init_speed_x = 8;
+    mouse.init_speed_y = 16;
+  } else {
+    /* default to 1 to 1 scaling for apps tha read mickeys */
+    mouse.init_speed_x = 8;
+    mouse.init_speed_y = 8;
+  }
   
   /* We set the defaults at the end so that we can test the mouse type */
   mouse_reset(1);		/* Let's set defaults now ! */
