@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -164,15 +164,12 @@ void uart_fill(int num)
        * Reset the receive FIFO counter up to a value of 16 (configurable)
        * if we are not within an interrupt.
        */
-      if (!com[num].int_pend) {
-        if (com[num].rx_buf_bytes > com[num].rx_fifo_size)
-          com[num].rx_fifo_bytes = com[num].rx_fifo_size;
-        else
-          com[num].rx_fifo_bytes = com[num].rx_buf_bytes;
-      }
+      com[num].rx_fifo_bytes = com[num].rx_buf_bytes;
+      if (com[num].rx_fifo_bytes > com[num].rx_fifo_size)
+	      com[num].rx_fifo_bytes = com[num].rx_fifo_size;
       
       /* Has it gone above the receive FIFO trigger level? */
-      if (com[num].rx_buf_bytes >= com[num].rx_fifo_trigger) {
+      if (com[num].rx_fifo_bytes >= com[num].rx_fifo_trigger) {
         if(s3_printf) s_printf("SER%d: Func uart_fill requesting RX_INTR\n",num);
         com[num].LSRqueued |= UART_LSR_DR;	/* Update queued LSR */
         serial_int_engine(num, RX_INTR);	/* Update interrupt status */
@@ -471,7 +468,8 @@ static int get_rx(int num)
         com[num].int_condition &= ~RX_INTR;	/* Clear receive condition */
 
         /* DANG_FIXTHIS Is this safe to put this here? */
-        serial_int_engine(num, 0);		/* Update interrupt status */
+	/* I don't think so  - SS */
+//        serial_int_engine(num, 0);		/* Update interrupt status */
       }
       else {		
         /* No, the FIFO didn't drop below trigger, but clear timeout bit */
@@ -776,6 +774,7 @@ put_lcr(int num, int val)
      */
   }
   else {
+    s_printf("SER%d: LCR = 0x%x, DLAB low.\n", num, val);
     com[num].DLAB = 0;			/* Baudrate Divisor Latch flag */
     ser_termios(num);			/* Sets new line settings */
   }
@@ -828,7 +827,7 @@ put_mcr(int num, int val)
     if (changed & UART_MCR_LOOP) uart_clear_fifo(num,UART_FCR_CLEAR_CMD);
 
     /* Set interrupt enable flag according to OUT2 bit in MCR */
-    com[num].int_enab = (val & UART_MCR_OUT2);
+    com[num].int_enab = (val & UART_MCR_OUT2) ? 1 : 0;
 
     /* Force RTS & DTR reinitialization if the loopback state has changed */
     if (UART_MCR_LOOP) changed |= UART_MCR_RTS | UART_MCR_DTR;
