@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -17,14 +17,16 @@ void X_speaker_on(void *gp, unsigned ms, unsigned short period)
 	XKeyboardControl new_state;
 	Display *display = gp;
 
+#if 0
 	/* Turn off the previos sound, I hope */
 	new_state.bell_pitch = 0;  
 	new_state.bell_duration = 0;
 	XChangeKeyboardControl(display, KBBellDuration | KBBellPitch, &new_state);
 	XBell(display, -100);
+#endif
 
-        /* Make the sound I want */
-	new_state.bell_pitch = speaker_period_to_Hz(period);  
+	/* Make the sound I want */
+	new_state.bell_pitch = speaker_period_to_Hz(period);
 	/* Prohibit a crashing X server by not allowing to go higher than
 	 * 32767 kHz. Since the X server interpretes the pitch as a short int
 	 */
