@@ -1,5 +1,5 @@
 /*
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -20,7 +20,7 @@
 #define DOSEMU_BIOS_DATE		"02/25/93"
 #define DOSEMU_MAGIC			"$DOSEMU$"
 
-#if defined(__GNUC__)
+#if defined(__GNUC__) && !defined(__ASSEMBLER__)
 #define DOSEMU_BIOS_DATE_LOCATION	0x000FFFF5
 #define DOSEMU_MAGIC_LOCATION		0x000FFFE0
 #define FAR
