@@ -1,6 +1,6 @@
 /*
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -26,6 +26,10 @@
  *  Checking for DosC and exiting with error, if not redirector capable.
  *  Printing DOS errors in clear text.
  *
+ * Changes: 010211 Bart Oldeman
+ *   Make it possible to "reredirect": LREDIR drive filepath
+ *   can overwrite the direction for the drive.
+ *
  * NOTES:
  *  LREDIR supports the following commands:
  *  LREDIR drive filepath
@@ -536,6 +540,13 @@ main(int argc, char **argv)
     ccode = RedirectDevice(deviceStr, resourceStr, REDIR_DISK_TYPE,
                            deviceParam);
 
+    /* duplicate redirection: try to reredirect */
+    if (ccode == 0x55) {
+      DeleteDriveRedirection(deviceStr);     
+      ccode = RedirectDevice(deviceStr, resourceStr, REDIR_DISK_TYPE,
+                             deviceParam);
+    }
+
     if (ccode) {
       printf("Error %x (%s)\nwhile redirecting drive %s to %s\n",
              ccode, decode_DOS_error(ccode), deviceStr, resourceStr);
