@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -43,12 +43,12 @@ EXTERN unsigned char dpmi_mhp_intxxtab[256] INIT({0});
 
 void dpmi_get_entry_point();
 
+void indirect_dpmi_switch(struct sigcontext_struct *);
 #ifdef __linux__
 void dpmi_fault(struct sigcontext_struct *);
 #endif
 void dpmi_realmode_hlt(unsigned char *);
 void run_pm_int(int);
-void run_pm_mouse();
 void fake_pm_int(void);
 
 #ifdef __linux__
@@ -184,6 +184,7 @@ dpmi_pm_block *lookup_pm_block(unsigned long h);
 int
 DPMIMapConventionalMemory(dpmi_pm_block *block, unsigned long offset,
 			  unsigned long low_addr, unsigned long cnt);
+unsigned long GetSegmentBaseAddress(unsigned short);
 
 extern void DPMI_show_state(struct sigcontext_struct *scp);
 extern void dpmi_sigio(struct sigcontext_struct *scp);
