@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -56,7 +56,7 @@
 #include <unistd.h>
 
 #include <scsi/sg.h>
-#include <asm/param.h>  /* for HZ */
+#include <sys/param.h>  /* for HZ */
 
 #include "emu.h"
 #include "aspi.h"
