@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -15,7 +15,7 @@
  * around in specific calls. This spec also includes the allocation of UMB's,
  * so they are also included as part of this file. The amount of xms memory
  * returned to DOS programs via the XMS requests, or int15 fnc88 is set in
- * "/etc/dosemu.conf" via the XMS paramter.
+ * "/etc/dosemu.conf" via the XMS parameter.
  *
  * /REMARK
  * DANG_END_MODULE
@@ -36,6 +36,8 @@
 #include "dosio.h"
 #include "machcompat.h"
 
+#undef  DEBUG_XMS
+
 static int umb_find_unused(void);
 /* 128*1024 is the amount of memory currently reserved in dos.c above
  * the 1 MEG mark.  ugly.  fix this.
@@ -147,7 +149,7 @@ umb_memory_empty(addr, size)
       return FALSE;
     }
   }
-#if 0
+#ifdef DEBUG_XMS
   Debug0((dbg_fd, "Found free UMB region: %p\n", (void *) addr));
 #endif
   return (TRUE);
@@ -224,7 +226,7 @@ umb_setup(void)
     for (addr = UMB_BASE; addr < (vm_address_t) (UMB_BASE + UMB_SIZE);
 	 addr += UMB_PAGE) {
       if (IN_EMM_SPACE(addr) || IN_EMU_SPACE(addr) || IN_HARDWARE_PAGES(addr)) {
-#if 0
+#ifdef DEBUG_XMS
 	Debug0((dbg_fd, "skipped UMB region: %p EMM-%d EMU=%d hardw=%d\n",(void *) addr, 
 		IN_EMM_SPACE(addr),IN_EMU_SPACE(addr),IN_HARDWARE_PAGES(addr)));
 #endif
@@ -301,15 +303,13 @@ umb_allocate(int size)
 {
   int i;
 
-  if (size == 0)
-    return ((vm_address_t) 0);
-
   for (i = 0; i < UMBS; i++) {
     if (umbs[i].in_use && umbs[i].free) {
       if (umbs[i].size > size) {
 	int new_umb = umb_find_unused();
 
 	if (new_umb != UMB_NULL) {
+	 if(size) {
 	  umbs[new_umb].in_use = TRUE;
 	  umbs[new_umb].free = TRUE;
 	  umbs[new_umb].addr =
@@ -318,7 +318,8 @@ umb_allocate(int size)
 	    umbs[i].size - size;
 	  umbs[i].size = size;
 	  umbs[i].free = FALSE;
-	  return (umbs[i].addr);
+	 }
+	 return (umbs[i].addr);
 	}
       }
       else
@@ -383,7 +384,8 @@ umb_query(void)
 	largest = umbs[i].size;
     }
   }
-  Debug0((dbg_fd, "umb_query: largest UMB was %d bytes\n", largest));
+  Debug0((dbg_fd, "umb_query: largest UMB was %d(%#x) bytes\n",
+	largest, largest));
   return (largest);
 }
 
@@ -451,7 +453,6 @@ xms_control(void)
       LWORD(eax) = 0;
       LO(bx) = 0x91;
     }
-    LWORD(ebx) = 0xff00;
     break;
 
   case 2:			/* Release High Memory Area */
@@ -466,7 +467,6 @@ xms_control(void)
       LWORD(eax) = 1;
       freeHMA = 1;
     }
-    LWORD(ebx) = 0xff00;
     break;
 
   case 3:			/* Global Enable A20 */
@@ -486,6 +486,7 @@ xms_control(void)
     break;
 
   case 5:			/* Local Enable A20 */
+    x_printf("XMS LOCAL enable A20\n");
     a20++;
     if (a20 == 1)
       set_a20(1);
@@ -506,7 +507,7 @@ xms_control(void)
           */
   case 7:			/* Query A20 */
     LWORD(eax) = a20 ? 1 : 0;
-    LWORD(ebx) = 0;		/* no error */
+    LO(bx) = 0;			/* no error */
     break;
 
   case 8:			/* Query Free Extended Memory */
@@ -537,23 +538,25 @@ xms_control(void)
     xms_EMB_info(OLDXMS);
     return;
 
-  case 0xf:			/* Realocate Extended Memory Block */
+  case 0xf:			/* Reallocate Extended Memory Block */
     xms_realloc_EMB(OLDXMS);
     break;
 
 #define state (&_regs)
   case XMS_ALLOCATE_UMB:
     {
-      int size = WORD(state->edx) * 16;
-      vm_address_t addr = size ? umb_allocate(size) : (vm_address_t) 0;
+      int size = WORD(state->edx) << 4;
+      vm_address_t addr = umb_allocate(size);
 
-      Debug0((dbg_fd, "Allocate UMB memory: 0x%04x\n",
-	      (unsigned) WORD(state->edx)));
+      Debug0((dbg_fd, "Allocate UMB memory: %#x\n", size));
       if (addr == (vm_address_t) 0) {
+        int avail=umb_query();
+
 	Debug0((dbg_fd, "Allocate UMB Failure\n"));
 	SETWORD(&(state->eax), 0);
-	SETLOW(&(state->ebx), 0xb0);
-	SETWORD(&(state->edx), umb_query() >> 4);
+	if(avail) SETLOW(&(state->ebx), 0xb0);
+	else SETLOW(&(state->ebx), 0xb1);
+	SETWORD(&(state->edx), avail >> 4);
       }
       else {
 	Debug0((dbg_fd, "Allocate UMB Success\n"));
@@ -561,7 +564,7 @@ xms_control(void)
 	SETWORD(&(state->ebx), (int) addr >> 4);
 	SETWORD(&(state->edx), size >> 4);
       }
-      Debug0((dbg_fd, "umb_allocated: 0x%04x, 0x%04x\n",
+      Debug0((dbg_fd, "umb_allocated: %#x0:%#x0\n",
 	      (unsigned) WORD(state->ebx),
 	      (unsigned) WORD(state->edx)));
       /* retval = UNCHANGED; */
@@ -683,8 +686,6 @@ xms_query_freemem(int api)
     x_printf("XMS query free memory(new): %ldK %ldK\n",
 	     REG(eax), REG(edx));
   }
-
-  LWORD(ebx) = 0xff00;			/* no error */
 }
 
 static void
@@ -698,7 +699,7 @@ xms_allocate_EMB(int api)
     kbsize = LWORD(edx);
   else
     kbsize = REG(edx);
-  x_printf("XMS alloc EMB(%s) size 0x%08lx KB\n", (api==OLDXMS)?"old":"new",kbsize);
+  x_printf("XMS alloc EMB(%s) size %ld KB\n", (api==OLDXMS)?"old":"new",kbsize);
 
   totalBytes = 0;
   for (h = FIRST_HANDLE; h <= NUM_HANDLES; h++) {
@@ -712,14 +713,14 @@ xms_allocate_EMB(int api)
   {
     x_printf("XMS: out of memory (only %ldK available)\n",subtotal);
     LWORD(eax) = 0;
-    LWORD(ebx) = 0xFFa0; /* Out of memory */
+    LO(bx) = 0xa0; /* Out of memory */
     return;
   }
   
   if (!(h = FindFreeHandle(FIRST_HANDLE))) {
     x_printf("XMS: out of handles\n");
     LWORD(eax) = 0;
-    LWORD(ebx) = 0xffa1;
+    LO(bx) = 0xa1;
   }
   else {
     handles[h].num = h;
@@ -750,7 +751,6 @@ xms_allocate_EMB(int api)
       REG(edx) = h;
 
     LWORD(eax) = 1;		/* success */
-    LWORD(ebx) = 0xa900;
   }
 }
 
@@ -776,7 +776,6 @@ xms_free_EMB(void)
 
     x_printf("XMS: free'd EMB %d\n", h);
     LWORD(eax) = 1;
-    LWORD(ebx) = 0xff00;
   }
 }
 
@@ -789,7 +788,13 @@ xms_move_EMB(void)
   x_printf("XMS move extended memory block\n");
   show_emm(e);
   LWORD(eax) = 1;		/* start with success */
+#if 0
   LWORD(ebx) = 0xff00;		/* start with success */
+  LO(bx) = 0x00; /* do not touch BH, 
+		    else Win98 boot failure. */
+  /* also don't touch BL - the XMS spec only specifies  *
+   * BL = 0x00 for function 7 (query A20)		*/
+#endif
 
   if (e.SourceHandle == 0) {
     src = (char *) (((e.SourceOffset >> 16) << 4) + \
@@ -934,8 +939,8 @@ xms_realloc_EMB(int api)
   else
     handles[h].size = REG(ebx) * 1024;
 
-  x_printf((api == OLDXMS) ? "XMS realloc EMB(old) %d to size 0x%04x\n" :
-	   "XMS realloc EMB(new) %d to size 0x%08x\n",
+  x_printf((api == OLDXMS) ? "XMS realloc EMB(old) %d to size 0x%04lx\n" :
+	   "XMS realloc EMB(new) %d to size 0x%08lx\n",
 	   h, handles[h].size);
 
   handles[h].addr = malloc(handles[h].size);
@@ -1012,9 +1017,7 @@ show_emm(struct EMM e)
 {
   x_printf("XMS show_emm:\n");
 
-  x_printf("L: 0x%08lx\n"
-	   "SH: 0x%04x  SO: 0x%08lx\n"
-	   "DH: 0x%04x  DO: 0x%08lx\n",
+  x_printf("L: 0x%08lx SH: 0x%04x  SO: 0x%08lx DH: 0x%04x  DO: 0x%08lx\n",
 	   e.Length, e.SourceHandle, e.SourceOffset,
 	   e.DestHandle, e.DestOffset);
 }
