@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -24,16 +24,16 @@
 #include <sys/ioctl.h>
 #include <net/route.h>
 #if GLIBC_VERSION_CODE >= 2000
-  #include<netipx/ipx.h>
+  #include <netipx/ipx.h>
 #else
-  #include <linux/sockios.h>
-  #include <linux/ipx.h>
+  #include "Linux/ipx.h"
 #endif
 #include <netinet/in.h>
 #include <errno.h>
 
 #include "emu.h"
 #include "dosemu_select.h"
+#include "utilities.h"
 
 #define FALSE   0
 #define TRUE    1
@@ -56,14 +56,16 @@ int AddRoute( unsigned long targetNet, unsigned network,
 	struct sockaddr_ipx	*st = (struct sockaddr_ipx *)&rt.rt_dst;
 	struct sockaddr_ipx	*sr = (struct sockaddr_ipx *)&rt.rt_gateway;
 	int sock;
-	int i;
 	
+	if (!can_do_root_stuff) {
+		error("IPX: Cannot add route, root privs required!\n");
+		return -2;
+	}
+
 	rt.rt_flags = RTF_GATEWAY;
 	st->sipx_network = targetNet;
         sr->sipx_network = network;
-	for( i=0; i<6; i++ ) {
-		sr->sipx_node[i] = node[i];
-	}	
+	memcpy(sr->sipx_node, node, 6);
 	sr->sipx_family = st->sipx_family = AF_IPX;
 	
 	enter_priv_on();
@@ -86,6 +88,45 @@ int AddRoute( unsigned long targetNet, unsigned network,
         return(0);
 }
 
+int CheckRouteExist(unsigned long targetNet, unsigned network,
+        unsigned char node[])
+{
+char buf_targ[9], buf_net[9], buf_node[13], proc_net[9], proc_node[13], *proc_str;
+
+	sprintf(buf_targ, "%08lX", (unsigned long)htonl(targetNet));
+	sprintf(buf_net, "%08lX", (unsigned long)htonl(network));
+	sprintf(buf_node, "%02X%02X%02X%02X%02X%02X", node[0], node[1],
+                     node[2], node[3], node[4], node[5]);
+
+	open_proc_scan("/proc/net/ipx_route");
+	proc_str = get_proc_string_by_key(buf_targ);
+
+	if (!proc_str) {
+		close_proc_scan();
+		return 0;
+	}
+
+	sscanf(proc_str, "%s %s", proc_net, proc_node);
+	close_proc_scan();
+
+	if (strcmp(buf_net, proc_net) || strcmp(buf_node, proc_node)) {
+		open_proc_scan("/proc/net/ipx_interface");
+		proc_str = get_proc_string_by_key(buf_net);
+		if (!proc_str) {
+			close_proc_scan();
+			return 0;
+		}
+		if (!strstr(proc_str, "Internal")) {
+			close_proc_scan();
+			return 0;
+		}
+		close_proc_scan();
+		/* fall through */
+	}
+
+	return 1;
+}
+
 /* network must be passed in in network order (that is, high-low) */
 /* returns 0 on success, less than 0 on failure */
 int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
@@ -108,21 +149,16 @@ int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
 	leave_priv_setting();
 	if(sock==-1)
 	{
+		n_printf("IPX: could not open IPX socket: %s.\n", strerror(errno));
 		goto GLTExit;
 	}
 	
-	/* Socket debugging */
 	enter_priv_on();
-	if(setsockopt(sock,SOL_SOCKET,SO_DEBUG,&opt,sizeof(opt))==-1)
-	{
-		leave_priv_setting();
-		goto CloseGLTExit;
-	}
-	
 	/* Permit broadcast output */
 	if(setsockopt(sock,SOL_SOCKET,SO_BROADCAST, &opt,sizeof(opt))==-1)
 	{
 		leave_priv_setting();
+		n_printf("IPX: could not set socket option for broadcast: %s.\n", strerror(errno));
 		goto CloseGLTExit;
 	}
 	
@@ -131,9 +167,10 @@ int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
 	   
 	opt=4;		/* Remember no htons! - its a byte */
 	
-	if(setsockopt(sock,SOL_SOCKET,IPX_TYPE,&opt,sizeof(opt))==-1)
+	if (setsockopt(sock, SOL_IPX, IPX_TYPE, &opt, sizeof(opt)) == -1)
 	{
 		leave_priv_setting();
+		n_printf("IPX: could not set socket option for type: %s.\n", strerror(errno));
 		goto CloseGLTExit;
 	}
 	
@@ -144,6 +181,7 @@ int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
 	if(bind(sock,(struct sockaddr *)&ipxs,sizeof(ipxs))==-1)
 	{
 		leave_priv_setting();
+		n_printf("IPX: could not bind socket to address: %s\n", strerror(errno));
 		goto CloseGLTExit;
 	}
 	leave_priv_setting();
@@ -173,6 +211,7 @@ int IPXGetLocalTarget( unsigned long network, int *hops, int *ticks )
 	        {
 			leave_priv_setting();
                         retCode = -2;
+			n_printf("IPX: sendto() failed: %s\n", strerror(errno));
         		goto CloseGLTExit;
 	        }
 		leave_priv_setting();
@@ -223,20 +262,27 @@ RepeatSelect:
         
                 *hops = htons(RipResponse.Hops);
                 *ticks = htons(RipResponse.Ticks);
-                retCode = AddRoute( network, ipxs.sipx_network,
+		if (CheckRouteExist(network, ipxs.sipx_network, ipxs.sipx_node)) {
+		   n_printf("IPX: Route <%08lx through %08lx:%02x%02x%02x%02x%02x%02x> already exists\n",
+                     (unsigned long int)htonl(network), (unsigned long int)htonl(ipxs.sipx_network),
+                     ipxs.sipx_node[0],ipxs.sipx_node[1],ipxs.sipx_node[2],
+                     ipxs.sipx_node[3],ipxs.sipx_node[4],ipxs.sipx_node[5] );
+		} else {
+                  retCode = AddRoute( network, ipxs.sipx_network,
                         ipxs.sipx_node );
-                if( retCode < 0 ) {
-                        n_printf("IPX: Failure %d adding route <%08lx through %08lx:%02x%02x%02x%02x%02x%02x>\n",
-                                retCode,
+                  if( retCode < 0 ) {
+                        error("IPX: Failure adding route <%08lx through %08lx:%02x%02x%02x%02x%02x%02x>\n",
                                 (unsigned long int)htonl(network), (unsigned long int)htonl(ipxs.sipx_network),
                                 ipxs.sipx_node[0],ipxs.sipx_node[1],ipxs.sipx_node[2],
                                 ipxs.sipx_node[3],ipxs.sipx_node[4],ipxs.sipx_node[5] );
-                } else {
+			error("IPX: Try manually add this route with ipx_route command\nor use RIP daemon like ipxripd\n");
+                  } else {
                         n_printf("IPX: Success adding route <%08lx through %08lx:%02x%02x%02x%02x%02x%02x>\n",
                                 (unsigned long int)htonl(network), (unsigned long int)htonl(ipxs.sipx_network),
                                 ipxs.sipx_node[0],ipxs.sipx_node[1],ipxs.sipx_node[2],
                                 ipxs.sipx_node[3],ipxs.sipx_node[4],ipxs.sipx_node[5] );
-                }
+                  }
+		}
         } else {
                 printf("IPX: Error %d in GetLocalTarget main packet send/receive loop\n", retCode);
         }
