@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -42,25 +42,20 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/time.h>
+#include <netinet/in.h>
 #if GLIBC_VERSION_CODE >= 2000
-  #if GLIBC_VERSION_CODE == 2000
-    #include <asm/byteorder.h>
-  #endif
   #include <netinet/if_ether.h>
 #else
-  #include <asm/byteorder.h>
-  #include <linux/if_ether.h>
+  #include "Linux/if_ether.h"
 #endif
 #include "libpacket.h"
 #include "dosnet.h"
 #include "dosemu_select.h"
 
 /* flag to activate use of pic by packet driver */
-#if 0
-#define PICPKT 1
-#endif
 #ifdef PICPKT
 #include "pic.h"
+#include "dpmi.h"
 #endif
 
 #define min(a,b)	((a) < (b)? (a) : (b))
@@ -113,7 +108,7 @@ struct pkt_globs
     } handle[MAX_HANDLE];          
 } pg;
 
-static char devname[10] = "eth0";   /* linux device name */
+static char devname[10];		/* linux device name */
 
 /* various declarations for interfacing with the packet driver code in
    bios.S */
@@ -164,6 +159,9 @@ pkt_init(int vec)
 	strcpy(devname, DOSNET_DEVICE);
 	if (Open_sockets() < 0)
 	    goto fail;
+    } else {
+      strncpy(devname, config.netdev, sizeof(devname) - 1);
+      devname[sizeof(devname) - 1] = 0;
     }
 
     /* hook the interrupt vector by pointing it into the magic table */
@@ -205,12 +203,14 @@ pkt_int (void)
     if (!pktdrvr_installed)
 	return 0;
 
-#if 0
-    pd_printf("NPKT: AX=%04x BX=%04x CX=%04x DX=%04x FLAGS=%08x\n",
+#if 1
+  if (debug_level('P') > 8) {
+    pd_printf("NPKT: AX=%04x BX=%04x CX=%04x DX=%04x FLAGS=%08lx\n",
 		LWORD(eax),LWORD(ebx),LWORD(ecx),LWORD(edx),REG(eflags));
     pd_printf("      SI=%04x DI=%04x BP=%04x SP=%04x CS=%04x DS=%04x ES=%04x SS=%04x\n",
 		LWORD(esi),LWORD(edi),LWORD(ebp),LWORD(esp),
 		LWORD(cs),LWORD(ds),LWORD(es),LWORD(ss));
+  }
 #endif
 
     /* first clear CARRY flag that we may later set (on error) */
@@ -408,6 +408,8 @@ pkt_int (void)
 	p_stats->packets_out++;
 	p_stats->bytes_out += LWORD(ecx);
 
+	pd_printf("========Sending packet======\n");
+	printbuf("packet to send:", SEG_ADR((struct ethhdr *), ds, si)); 
 	for (handle = 0; handle < MAX_HANDLE; handle++) {
 	    hdlp = &pg.handle[handle];
 
@@ -611,6 +613,16 @@ Remove_Type(int handle)
     return 0;
 }
 
+void pkt_receiver_callback(void)
+{
+    if(in_dpmi && !in_dpmi_dos_int)
+	fake_pm_int();
+    run_int(pg.helpvec);
+   /* push iret frame. At F000:20F7 (bios.S) we get an
+    * iret and return to F000:2140 for EOI */
+    fake_int(BIOSSEG, EOI_OFF);
+}
+
 void
 pkt_check_receive_quick(void)
 {
@@ -829,7 +841,7 @@ Find_Handle(u_char *buf)
 	p = buf + 2 * ETH_ALEN;		/* Ethernet-II */
     else 
 	p = buf + 2 * ETH_ALEN + 2;     /* All the rest frame types. */
-    pd_printf("Received packet type: %.2x\n", ntohs(eth->h_proto));
+    pd_printf("Received packet type: 0x%x\n", ntohs(eth->h_proto));
 
     for(i=0; i<max_pkt_type_array; i++) {
 	nchars = pkt_type_array[i].pkt_type_len;
@@ -845,7 +857,7 @@ printbuf(char *mesg, struct ethhdr *buf)
 { int i;
   pd_printf( "%s :\n Dest.=", mesg);
   for (i=0;i<6;i++) pd_printf("%x:",buf->h_dest[i]);
-  pd_printf( " Source=") ;
+  pd_printf( " Source=");
   for (i=0;i<6;i++) pd_printf("%x:",buf->h_source[i]);
-  pd_printf( " Type= %x \n", buf->h_proto) ;
+  pd_printf( " Type= 0x%x \n", ntohs(buf->h_proto));
 }
