@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -25,6 +25,7 @@
 #include "emu.h"
 #include "timers.h"
 #include "pic.h"
+#include "speaker.h"
 
 /* --------------------------------------------------------------------- */
 /*
@@ -120,13 +121,13 @@ static hitimer_t rawP5time(void)
 static hitimer_t getC4time(void)
 {
   if (cpu_time_stop) return LastTimeRead;
-  return (rawC4time() - ZeroTimeBase.td);	/* ZeroTimeBase in us */
+  return (rawC4time() - ZeroTimeBase.td);
 }
 
 static hitimer_t getP5time(void)
 {
   if (cpu_time_stop) return LastTimeRead;
-  return CPUtoUS_Z();				/* ZeroTimeBase in cycles */
+  return (rawP5time() - ZeroTimeBase.td);
 }
 
 /*
@@ -180,7 +181,6 @@ void get_time_init (void)
      * but only flags processing (& other features) */
     RAWcpuTIME = rawP5time;		/* in usecs */
     GETcpuTIME = getP5time;		/* in usecs */
-    ZeroTimeBase.td = GETTSC();		/* in CPU cycles */
     g_printf("TIMER: using pentium timing\n");
   }
   else {
@@ -188,7 +188,6 @@ void get_time_init (void)
      * 'rdtsc off' into config file */
     RAWcpuTIME = rawC4time;		/* in usecs */
     GETcpuTIME = getC4time;		/* in usecs */
-    ZeroTimeBase.td = rawC4time();	/* in usecs */
     if (config.realcpu) {
       if (kernel_version_code < 0x2017e)
         g_printf("TIMER: using gettimeofday\n");
@@ -196,6 +195,7 @@ void get_time_init (void)
         g_printf("TIMER: using new gettimeofday with microsecond resolution\n");
     }
   }
+  ZeroTimeBase.td = RAWcpuTIME();
 }
 
 
@@ -207,7 +207,7 @@ int stop_cputime (int quiet)
   if (cpu_time_stop) return 1;
   if (!quiet) dbug_printf("STOP TIME\n");
   StopTimeBase = RAWcpuTIME();
-  LastTimeRead = StopTimeBase - TSCtoUS(ZeroTimeBase.td);
+  LastTimeRead = StopTimeBase - ZeroTimeBase.td;
   cpu_time_stop = 1;
   return 0;
 }
@@ -217,7 +217,7 @@ int restart_cputime (int quiet)
 {
   if (!cpu_time_stop) return 1;
   if (!quiet) dbug_printf("RESTART TIME\n");
-  ZeroTimeBase.td += ((RAWcpuTIME() - StopTimeBase) * config.CPUSpeedInMhz);
+  ZeroTimeBase.td += RAWcpuTIME() - StopTimeBase;
   cpu_time_stop = 0;
   return 0;
 }
@@ -231,11 +231,13 @@ void freeze_dosemu(void)
   stop_cputime(0);
   dbug_printf("*** dosemu frozen\n");
   dosemu_frozen = 1;
+  speaker_pause();
 }
 
 void unfreeze_dosemu(void)
 {
   if (!dosemu_frozen) return;
+  speaker_resume ();
   restart_cputime(0);
   dbug_printf("*** dosemu unfrozen\n");
   dosemu_frozen = 0;
