@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -129,39 +129,6 @@ void vm86_GP_fault(void)
     }
 #endif
 
-  if (ignore_segv) {
-    error("sigsegv ignored!\n");
-    return;
-  }
-
-  if (in_sigsegv)
-    error("in_sigsegv=%d!\n", in_sigsegv);
-
-  /* in_vm86 = 0; */
-  in_sigsegv++;
-
-  /* DANG_BEGIN_REMARK 
-   * In a properly functioning emulator :-), sigsegv's will never come
-   * while in a non-reentrant system call (ioctl, select, etc).  Therefore,
-   * there's really no reason to worry about them, so I say that I'm NOT
-   * in a signal handler (I might make this a little clearer later, to
-   * show that the purpose of in_sighandler is to stop non-reentrant system
-   * calls from being reentered.
-   * I reiterate: sigsegv's should only happen when I'm running the vm86
-   * system call, so I really shouldn't be in a non-reentrant system call
-   * (except maybe vm86)
-   * - Robert Sanders
-   * DANG_END_REMARK
-   */
-  in_sighandler = 0;
-
-#ifndef TRACE_DPMI
-  if (LWORD(eflags) & TF) {
-    g_printf("SIGSEGV received while TF is set\n");
-    show_regs(__FILE__, __LINE__);
-  }
-#endif
-
   csp = lina = SEG_ADR((unsigned char *), cs, ip);
 
   /* fprintf(stderr, "CSP in cpu is 0x%04x\n", *csp); */
@@ -318,10 +285,6 @@ void vm86_GP_fault(void)
 
   case 0xf4:			/* hlt...I use it for various things,
 		  like trapping direct jumps into the XMS function */
-#ifndef USE_NEW_INT
-       /* set VIF (only if necessary) */
-    if (REG(eflags) & IF_MASK) REG(eflags) |= VIF_MASK;
-#endif /* not USE_NEW_INT */
 #if defined(X86_EMULATOR) && defined(SKIP_EMU_VBIOS)
     if ((config.cpuemu>1) && (lina == (unsigned char *) CPUEMUI10_ADD)) {
       e_printf("EMU86: HLT at int10 end\n");
@@ -330,10 +293,8 @@ void vm86_GP_fault(void)
     }
     else
 #endif
-          /* return with STI if VIP was set from run_dpmi; this happens
-           * if pic_count is >0 and the VIP flag in dpmi_eflags was on
-           */
-    if ((lina == (unsigned char *) PIC_ADD) || (pic_icount && (REG(eflags) & VIP_MASK))) {
+
+    if (lina == (unsigned char *) PIC_ADD) {
       pic_iret();
     }
 
@@ -356,14 +317,17 @@ void vm86_GP_fault(void)
     }
 
     else {
-#ifndef SKIP_EMU_VBIOS
-      error("HLT requested: lina=%p!\n", lina);
-      show_regs(__FILE__, __LINE__);
+      if(pic_icount) {
+    /* looks like we have failed to catch iret... */
+         g_printf("HLT requested with pic_icount=%li: lina=%p!\n",
+	    pic_icount, lina);
+         show_regs(__FILE__, __LINE__);
+      }
+      pic_resched();
 #if 0
       haltcount++;
       if (haltcount > MAX_HALT_COUNT)
 	fatalerr = 0xf4;
-#endif
 #endif
       LWORD(eip) += 1;
     }
@@ -383,25 +347,22 @@ op0ferr:
 #ifdef USE_MHPDBG
     mhp_debug(DBG_GPF, 0, 0);
 #endif
-    d.general=1;
+    set_debug_level('g', 1);
     error("general protection at %p: %x\n", lina,*lina);
     show_regs(__FILE__, __LINE__);
     show_ints(0, 0x33);
-    error("SIGSEGV, protected insn...exiting!\n");
     fatalerr = 4;
     leavedos(fatalerr);		/* shouldn't return */
   }				/* end of switch() */
 
 #ifdef TRACE_DPMI
-  if (d.dpmit==0)
+  if (debug_level('t')==0)
 #endif
   if (LWORD(eflags) & TF) {
     g_printf("TF: trap done");
     show_regs(__FILE__, __LINE__);
   }
 
-  in_sigsegv--;
-  in_sighandler = 0;
 }
 /* @@@ MOVE_END @@@ 32768 */
 
@@ -449,12 +410,12 @@ run_vm86(void)
 
     if (
 #ifdef X86_EMULATOR
-	(d.emu>1)||
+	(debug_level('e')>1)||
 #endif
-	(d.general>3)) {
+	(debug_level('g')>3)) {
 	dbug_printf ("DO_VM86,  cs=%04x:%04x ss=%04x:%04x f=%08x\n",
 		_CS, _EIP, _SS, _SP, _EFLAGS);
-	if (d.general>8)
+	if (debug_level('g')>8)
 	    dbug_printf ("ax=%04x bx=%04x ss=%04x sp=%04x bp=%04x\n"
 	      		 "           cx=%04x dx=%04x ds=%04x cs=%04x ip=%04x\n"
 	      		 "           si=%04x di=%04x es=%04x flg=%08x\n",
@@ -464,28 +425,30 @@ run_vm86(void)
 
     retval = DO_VM86(&vm86s);
 
-#ifdef USE_NEW_INT
   /* sync the pic interrupt state with the flags && sync VIF & IF */
     if (_EFLAGS & VIF) {
       set_IF();
     } else {
       clear_IF();
     }
-#endif /* USE_NEW_INT */
 
+#if 0
     /* This will protect us from Mr.Norton's bugs */
     if (_EFLAGS & (AC|ID)) {
-      g_printf("BUG: flags changed to %08x\n",_EFLAGS);
       _EFLAGS &= ~(AC|ID);
+      if (debug_level('g')>3)
+	dbug_printf("BUG: AC,ID set; flags changed to %08x\n",_EFLAGS);
     }
+#endif
+
     if (
 #ifdef X86_EMULATOR
-	(d.emu>1)||
+	(debug_level('e')>1)||
 #endif
-	(d.general>3)) {
+	(debug_level('g')>3)) {
 	dbug_printf ("RET_VM86, cs=%04x:%04x ss=%04x:%04x f=%08x ret=0x%x\n",
 		_CS, _EIP, _SS, _SP, _EFLAGS, retval);
-	if (d.general>8)
+	if (debug_level('g')>8)
 	    dbug_printf ("ax=%04x bx=%04x ss=%04x sp=%04x bp=%04x\n"
 	      		 "           cx=%04x dx=%04x ds=%04x cs=%04x ip=%04x\n"
 	      		 "           si=%04x di=%04x es=%04x flg=%08x\n",
@@ -555,15 +518,6 @@ freeze_idle:
     if (iq.queued)
 	do_queued_ioctl();
     /* update the pic to reflect IEF */
-#ifndef USE_NEW_INT
-    if (REG(eflags) & VIF_MASK) {
-	if (pic_iflag)
-	    pic_sti();
-    } else {
-	if (!pic_iflag)
-	    pic_cli();		/* pic_iflag=0 => enabled */
-    }
-#endif /* not USE_NEW_INT */
 
   if (dosemu_frozen) {
     static int minpoll = 0;
