@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -136,12 +136,12 @@ void init_port_traceing(void)
 {
   if (portlog_map) return;
   clear_port_traceing();
-  register_port_traceing(0x100, 0x2ff);
+  register_port_traceing(0x0, 0xffff);
 }
 
 #define TT_printf(p,f,v,m) ({ \
-  if ( d.io_trace && test_bit(p & PORTLOG_MASK, portlog_map)) { \
-    log_printf(1, "%x %c %x\n", p, f, v & m); \
+  if (debug_level('T') && test_bit(p & PORTLOG_MASK, portlog_map)) { \
+    log_printf(1, "%hx %c %x\n", p, f, v & m); \
   } \
 })
 
@@ -178,13 +178,13 @@ static void log_port_write_d(ioport_t port, Bit32u w)
   TT_printf(port, '[', w, 0xffffffff);
 }
 
-#define LOG_PORT_READ(port, r) (d.io_trace ? log_port_read(port, r) : r)
-#define LOG_PORT_READ_W(port, r) (d.io_trace ? log_port_read_w(port, r) : r)
-#define LOG_PORT_READ_D(port, r) (d.io_trace ? log_port_read_d(port, r) : r)
+#define LOG_PORT_READ(port, r) (debug_level('T') ? log_port_read(port, r) : r)
+#define LOG_PORT_READ_W(port, r) (debug_level('T') ? log_port_read_w(port, r) : r)
+#define LOG_PORT_READ_D(port, r) (debug_level('T') ? log_port_read_d(port, r) : r)
 
-#define LOG_PORT_WRITE(port, w) do{ if (d.io_trace) log_port_write(port, w); }while(0)
-#define LOG_PORT_WRITE_W(port, w) do{ if (d.io_trace) log_port_write_w(port, w); }while(0)
-#define LOG_PORT_WRITE_D(port, w) do{ if (d.io_trace) log_port_write_d(port, w); }while(0)
+#define LOG_PORT_WRITE(port, w) do{ if (debug_level('T')) log_port_write(port, w); }while(0)
+#define LOG_PORT_WRITE_W(port, w) do{ if (debug_level('T')) log_port_write_w(port, w); }while(0)
+#define LOG_PORT_WRITE_D(port, w) do{ if (debug_level('T')) log_port_write_d(port, w); }while(0)
 
 /* ---------------------------------------------------------------------- */
 /* SIDOC_BEGIN_REMARK
@@ -282,7 +282,7 @@ Bit32u port_ind(ioport_t port)
 		res = EMU_HANDLER(port).read_portd(port);
 	}
 	else {
-		res = std_port_ind(port);
+		res = (Bit32u) port_inw(port) | (((Bit32u) port_inw(port + 2)) << 16);
 	}
 	return LOG_PORT_READ_D(port, res);
 }
@@ -294,7 +294,8 @@ void port_outd(ioport_t port, Bit32u dword)
 		EMU_HANDLER(port).write_portd(port, dword);
 	}
 	else {
-		std_port_outd(port, dword);
+		port_outw(port, dword & 0xffff);
+		port_outw(port+2, (dword >> 16) & 0xffff);
 	}
 }
 
@@ -367,35 +368,35 @@ static Bit8u port_not_avail_inb(ioport_t port)
 /* it is a fact of (hardware) life that unused locations return all
    (or almost all) the bits at 1; some software can try to detect a
    card basing on this fact and fail if it reads 0x00 - AV */
-	if (d.io) pna_emsg(port,'b',"read");
+	if (debug_level('i')) pna_emsg(port,'b',"read");
 	return 0xff;
 }
 
 static void port_not_avail_outb(ioport_t port, Bit8u byte)
 {
-	if (d.io) pna_emsg(port,'b',"write");
+	if (debug_level('i')) pna_emsg(port,'b',"write");
 }
 
 static Bit16u port_not_avail_inw(ioport_t port)
 {
-	if (d.io) pna_emsg(port,'w',"read");
+	if (debug_level('i')) pna_emsg(port,'w',"read");
 	return 0xffff;
 }
 
 static void port_not_avail_outw(ioport_t port, Bit16u value)
 {
-	if (d.io) pna_emsg(port,'w',"write");
+	if (debug_level('i')) pna_emsg(port,'w',"write");
 }
 
 static Bit32u port_not_avail_ind(ioport_t port)
 {
-	if (d.io) pna_emsg(port,'d',"read");
+	if (debug_level('i')) pna_emsg(port,'d',"read");
 	return 0xffffffff;
 }
 
 static void port_not_avail_outd(ioport_t port, Bit32u value)
 {
-	if (d.io) pna_emsg(port,'d',"write");
+	if (debug_level('i')) pna_emsg(port,'d',"write");
 }
 
 
@@ -433,7 +434,7 @@ int port_rep_inb(ioport_t port, Bit8u *base, int df, Bit32u count)
 	    dest += incr;
 	  }
 	}
-	if (d.io_trace) {
+	if (debug_level('T')) {
 		dest = base;
 		while (count_--) {
 			LOG_PORT_READ(port, *dest);
@@ -466,7 +467,7 @@ int port_rep_outb(ioport_t port, Bit8u *base, int df, Bit32u count)
 	    dest += incr;
 	  }
 	}
-	if (d.io_trace) {
+	if (debug_level('T')) {
 		dest = base;
 		while (count_--) {
 			LOG_PORT_WRITE(port, *dest);
@@ -507,7 +508,7 @@ int port_rep_inw(ioport_t port, Bit16u *base, int df, Bit32u count)
 	    dest += incr;
 	  }
 	}
-	if (d.io_trace) {
+	if (debug_level('T')) {
 		dest = base;
 		while (count_--) {
 			LOG_PORT_READ_W(port, *dest);
@@ -548,7 +549,7 @@ int port_rep_outw(ioport_t port, Bit16u *base, int df, Bit32u count)
 	    dest += incr;
 	  }
 	}
-	if (d.io_trace) {
+	if (debug_level('T')) {
 		dest = base;
 		while (count_--) {
 			LOG_PORT_WRITE_W(port, *dest);
@@ -775,6 +776,8 @@ int port_init(void)
 	port_handler[NO_HANDLE].write_portb = port_not_avail_outb;
 	port_handler[NO_HANDLE].read_portw = port_not_avail_inw;
 	port_handler[NO_HANDLE].write_portw = port_not_avail_outw;
+	port_handler[NO_HANDLE].read_portd = port_not_avail_ind;
+	port_handler[NO_HANDLE].write_portd = port_not_avail_outd;
 	port_handler[NO_HANDLE].handler_name = "unknown port";
 
   /* the STD handles will be in use by many devices, and their fd
@@ -784,30 +787,40 @@ int port_init(void)
 	port_handler[HANDLE_STD_IO].write_portb = std_port_outb;
 	port_handler[HANDLE_STD_IO].read_portw = std_port_inw;
 	port_handler[HANDLE_STD_IO].write_portw = std_port_outw;
+	port_handler[HANDLE_STD_IO].read_portd = std_port_ind;
+	port_handler[HANDLE_STD_IO].write_portd = std_port_outd;
 	port_handler[HANDLE_STD_IO].handler_name = "std port io";
 
 	port_handler[HANDLE_STD_RD].read_portb = std_port_inb;
 	port_handler[HANDLE_STD_RD].write_portb = port_not_avail_outb;
 	port_handler[HANDLE_STD_RD].read_portw = std_port_inw;
 	port_handler[HANDLE_STD_RD].write_portw = port_not_avail_outw;
+	port_handler[HANDLE_STD_RD].read_portd = std_port_ind;
+	port_handler[HANDLE_STD_RD].write_portd = port_not_avail_outd;
 	port_handler[HANDLE_STD_RD].handler_name = "std port read";
 
 	port_handler[HANDLE_STD_WR].read_portb = port_not_avail_inb;
 	port_handler[HANDLE_STD_WR].write_portb = std_port_outb;
 	port_handler[HANDLE_STD_WR].read_portw = port_not_avail_inw;
 	port_handler[HANDLE_STD_WR].write_portw = std_port_outw;
+	port_handler[HANDLE_STD_WR].read_portd = port_not_avail_ind;
+	port_handler[HANDLE_STD_WR].write_portd = std_port_outd;
 	port_handler[HANDLE_STD_WR].handler_name = "std port write";
 
 	port_handler[HANDLE_VID_IO].read_portb = video_port_in;
 	port_handler[HANDLE_VID_IO].write_portb = video_port_out;
 	port_handler[HANDLE_VID_IO].read_portw = port_not_avail_inw;
 	port_handler[HANDLE_VID_IO].write_portw = port_not_avail_outw;
+	port_handler[HANDLE_VID_IO].read_portd = port_not_avail_ind;
+	port_handler[HANDLE_VID_IO].write_portd = port_not_avail_outd;
 	port_handler[HANDLE_VID_IO].handler_name = "video port io";
 
 	port_handler[HANDLE_SPECIAL].read_portb = special_port_inb;
 	port_handler[HANDLE_SPECIAL].write_portb = special_port_outb;
 	port_handler[HANDLE_SPECIAL].read_portw = port_not_avail_inw;
 	port_handler[HANDLE_SPECIAL].write_portw = port_not_avail_outw;
+	port_handler[HANDLE_SPECIAL].read_portd = port_not_avail_ind;
+	port_handler[HANDLE_SPECIAL].write_portd = port_not_avail_outd;
 	port_handler[HANDLE_SPECIAL].handler_name = "extra stuff";
 
 	port_handles = STD_HANDLES;
