@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -75,6 +75,7 @@ __asm__("___START___: jmp _emulate\n");
 #include <limits.h>
 #include <getopt.h>
 #include <assert.h>
+#include <locale.h>
 
 #ifdef __linux__
 #if GLIBC_VERSION_CODE >= 2000
@@ -121,6 +122,8 @@ __asm__("___START___: jmp _emulate\n");
 #include "speaker.h"
 #include "utilities.h"
 #include "dos2linux.h"
+#include "iodev.h"
+#include "dosemu_config.h"
 
 #include "keyb_clients.h"
 
@@ -175,8 +178,6 @@ boot(void)
 	leavedos(15);
     }
 
-    ignore_segv++;
-
     disk_close();
     disk_open(dp);
 
@@ -213,7 +214,6 @@ boot(void)
 	}
     }
     disk_close();
-    ignore_segv--;
 }
 
 static inline void 
@@ -250,18 +250,16 @@ static void do_liability_disclaimer_prompt(void)
   FILE *f;
   char buf[32];
   char disclaimer_file_name[256];
-  static char text[] ="
-  The Linux DOSEMU, Copyright (C) 2001 the 'DOSEMU-Development-Team'.
-  This program is  distributed  in  the  hope that it will be useful,
-  but  WITHOUT  ANY  WARRANTY;   without even the implied warranty of
-  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the file
-  COPYING for more details.  Use  this  programm  at  your  own risk!
-
-  By continuing execution of this programm,  you are stating that you
-  have read the file  COPYING  and the above liability disclaimer and
-  that you accept these conditions.
-
-  Enter 'yes' to confirm/continue: ";
+  static char text[] =
+  "The Linux DOSEMU, Copyright (C) 2002 the 'DOSEMU-Development-Team'.\n"
+  "This program is  distributed  in  the  hope that it will be useful,\n"
+  "but  WITHOUT  ANY  WARRANTY;   without even the implied warranty of\n"
+  "MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the file\n"
+  "COPYING for more details.  Use  this  program  at  your  own  risk!\n\n"
+  "By continuing execution of this program,  you  are stating that you\n"
+  "have read the file  COPYING  and the above liability disclaimer and\n"
+  "that you accept these conditions.\n\n"
+  "Enter 'yes' to confirm/continue: ";
 
 
   snprintf(disclaimer_file_name, 256, "%s/disclaimer", LOCALDIR);
@@ -312,8 +310,6 @@ int
 emulate(int argc, char **argv)
 #endif
 {
-    extern void parse_dosemu_users(void);
-    extern void secure_option_preparse(int *argc, char **argv);
     int e;
 
     srand(time(NULL));
@@ -328,6 +324,8 @@ emulate(int argc, char **argv)
 	_exit(1);		/* just in case */
     }
 
+		setlocale(LC_ALL,"");
+
     /* NOW! it is safe to touch the priv code.  */
     priv_init();  /* This must come first! */
 
@@ -361,8 +359,11 @@ emulate(int argc, char **argv)
     config_init(argc, argv);	/* parse the commands & config file(s) */
 #ifdef X86_EMULATOR
 #ifdef DONT_DEBUG_BOOT		/* cpuemu only */
-    memcpy(&d_save,&d,sizeof(struct debug_flags));
-    if (d.emu) memset(&d,0,sizeof(struct debug_flags));
+    memcpy(&debug_save, &debug, sizeof(debug));
+    set_debug_level('e', 0);
+#ifdef TRACE_DPMI
+    set_debug_level('t', 0);
+#endif
 #endif
 #endif
     get_time_init();
@@ -384,6 +385,11 @@ emulate(int argc, char **argv)
     /* here we include the hooks to possible plug-ins */
     #include "plugin_init.h"
 
+    if (config.exitearly) {
+      dbug_printf("Leaving DOS before booting\n");
+      leavedos(0);
+    }
+
     boot();			/* read the boot sector & get moving */
 
     if (not_use_sigio)
@@ -403,9 +409,6 @@ emulate(int argc, char **argv)
 	loopstep_run_vm86();
     }
 
-    error("error exit: (%d,0x%04x) in_sigsegv: %d ignore_segv: %d\n",
-	  fatalerr, fatalerr, in_sigsegv, ignore_segv);
-
     sync();
     fprintf(stderr, "Not a good day to die!!!!!\n");
     leavedos(99);
@@ -422,7 +425,10 @@ dos_ctrl_alt_del(void)
     dbug_printf("DOS ctrl-alt-del requested.  Rebooting!\n");
     HMA_MAP(1);
     time_setting_init();
+#ifdef HAVE_KEYBOARD_V1
     keyb_server_reset();
+#endif
+    iodev_reset();
     video_config_init();
     serial_init();
     dosemu_mouse_init();
@@ -474,7 +480,6 @@ leavedos(int sig)
 {
     struct sigaction sa;
     struct itimerval itv;
-    extern int errno;
     extern void SIG_close(); /* obsolete now; integrated into the kernel */
     extern void do_r3da_pending (void);	/* emuretrace stuff */
    
@@ -540,14 +545,15 @@ leavedos(int sig)
 
     /* try to regain control of keyboard and video first */
     g_printf("calling keyboard_close\n");
+#ifdef HAVE_KEYBOARD_V1
     keyb_server_close();
     keyb_client_close();
+#endif
+    iodev_term();
 
 #if defined(X86_EMULATOR)
     /* if we are here with config.cpuemu>1 something went wrong... */
     if (config.cpuemu>1) {
-        extern void leave_cpu_emu(void);
-        
     	leave_cpu_emu();
     }
 #endif
@@ -598,19 +604,19 @@ leavedos(int sig)
     g_printf("calling close_all_printers\n");
     close_all_printers();
 
-    g_printf("calling serial_close\n");
-    serial_close();
     g_printf("calling mouse_close\n");
     dosemu_mouse_close();
 
     flush_log();
 
     /* remove per process tmpdir and its contents */
-    {
-       char *command = strcatdup("rm -rf >/dev/null 2>&1 ", TMPDIR_PROCESS);
-       if (command) {
-          priv_drop(); /* drop any priviledges before running system() !! */
-          system(command);
+    priv_drop(); /* drop any priviledges before running system() !! */
+    /* first try rmdir; if the directory is not empty, try system */
+    if (rmdir(TMPDIR_PROCESS) != 0) {
+       char *command = strcatdup("/bin/rm -rf >/dev/null 2>&1 ", TMPDIR_PROCESS);
+       if (command == NULL || system(command) != 0) {
+          g_printf("Failed to remove %s, you'll have to clean it up yourself.\n", 
+         TMPDIR_PROCESS);
        }
     }
 
