@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -225,13 +225,19 @@
 #include <sys/mman.h>           /* root@sjoerd:for mprotect*/
 
 #include "emu.h"
+#include "timers.h"
 #include "bios.h"
 #include "video.h"
 #include "memory.h"
 #include "remap.h"
 #include "vgaemu.h"
+#include "keyb_server.h"
 #include "X.h"
 
+#ifdef HAVE_UNICODE_TRANSLATION
+#include "translate.h"
+#endif
+
 #ifdef HAVE_MITSHM
 #include <sys/ipc.h>
 #include <sys/shm.h>
@@ -242,6 +248,10 @@
 #include <X11/extensions/xf86dga.h>
 #endif
 
+#ifdef HAVE_XKB
+#include <X11/XKBlib.h>
+#endif
+
 #if CONFIG_X_MOUSE
 #include "mouse.h"
 #include <time.h>
@@ -283,7 +293,12 @@ extern char **dosemu_argv;
 
 /* base/keyboard/keyb_X.c */
 extern void X_process_key(XKeyEvent *);
+extern void X_process_keys(XKeymapEvent *);
+#ifndef HAVE_UNICODE_KEYB
+inline void X_process_keys(XKeymapEvent *event) { return; }
+#endif
 
+#ifndef HAVE_UNICODE_TRANSLATION
 static const u_char dos_to_latin[] = {
    0xc7, 0xfc, 0xe9, 0xe2, 0xe4, 0xe0, 0xe5, 0xe7,  /* 80-87 */ 
    0xea, 0xeb, 0xe8, 0xef, 0xee, 0xec, 0xc4, 0xc5,  /* 88-8F */
@@ -341,8 +356,35 @@ static const u_char dos_to_latin2[] = {
    0xb0, 0xa8, 0xff, 0xfb, 0xd8, 0xf8, 0x00, 0xa0   /* F8-FF */
 };
 
+static const u_char dos_to_koi8[] = {
+  0xe1, 0xe2, 0xf7, 0xe7, 0xe4, 0xe5, 0xf6, 0xfa,
+  0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef, 0xf0,
+  0xf2, 0xf3, 0xf4, 0xf5, 0xe6, 0xe8, 0xe3, 0xfe,
+  0xfb, 0xfd, 0xff, 0xf9, 0xf8, 0xfc, 0xe0, 0xf1,
+  0xc1, 0xc2, 0xd7, 0xc7, 0xc4, 0xc5, 0xd6, 0xda,
+  0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf, 0xd0,
+  0x90, 0x91, 0x92, 0x81, 0x87, 0xb2, 0xb4, 0xa7,
+  0xa6, 0xb5, 0xa1, 0xa8, 0xae, 0xad, 0xac, 0x83,
+  0x84, 0x89, 0x88, 0x86, 0x80, 0x8a, 0xaf, 0xb0,
+  0xab, 0xa5, 0xbb, 0xb8, 0xb1, 0xa0, 0xbe, 0xb9,
+  0xba, 0xb6, 0xb7, 0xaa, 0xa9, 0xa2, 0xa4, 0xbd,
+  0xbc, 0x85, 0x82, 0x8d, 0x8c, 0x8e, 0x8f, 0x8b,
+  0xd2, 0xd3, 0xd4, 0xd5, 0xc6, 0xc8, 0xc3, 0xde,
+  0xdb, 0xdd, 0xdf, 0xd9, 0xd8, 0xdc, 0xc0, 0xd1,
+  0xb3, 0xa3, 0x99, 0x98, 0x93, 0x9b, 0x9f, 0x97,
+  0x9c, 0x95, 0x9e, 0x96, 0xbf, 0x9d, 0x94, 0x9a
+};
+
+#endif /* HAVE_UNICODE_TRANSLATION */
+
 static int (*OldXErrorHandler)(Display *, XErrorEvent *) = NULL;
 
+#ifdef HAVE_XKB
+int using_xkb = FALSE;
+int xkb_event_base = 0;
+int xkb_error_base = 0;
+#endif
+
 #ifdef HAVE_MITSHM
 static int shm_ok = 0;
 static int shm_error_base = 0;
@@ -420,9 +462,11 @@ static int remap_src_modes = 0;
 static vgaemu_display_type X_screen;
 
 int grab_active = 0;
+int freeze_active = 0;
 #if CONFIG_X_MOUSE
 static char *grab_keystring = "Home";
-static int grab_keysym = NoSymbol;
+static KeySym grab_keysym = NoSymbol;
+static KeySym freeze_keysym = NoSymbol;
 static int mouse_x = 0, mouse_y = 0;
 #endif
 
@@ -457,6 +501,10 @@ static int NewXErrorHandler(Display *, XErrorEvent *);
 void X_handle_events(void);		/* used in arch/linux/async/signal.c */
 
 /* interface to xmode.exe */
+static char X_title_emuname [X_TITLE_EMUNAME_MAXLEN] = {0};
+char X_title_appname [X_TITLE_APPNAME_MAXLEN] = {0};		/* used in plugin/commands/comcom.c */
+static int X_title_show_appname = 1;
+
 int X_change_config(unsigned, void *);	/* used in base/async/int.c */
 
 /* colormap related stuff */
@@ -552,11 +600,60 @@ struct video_system Video_X =
    X_update_cursor
 };
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+/* utility function for opening a connection and making certain
+ * I am either using or not using the X keyboard Extension.
+ */
+Display *XKBOpenDisplay(char *display_name)
+{
+	Display *dpy;
+#ifndef HAVE_XKB
+	{
+		PRIV_SAVE_AREA
+		enter_priv_on();	
+		dpy = XOpenDisplay(display_name);
+		leave_priv_setting();
+	}
+#else /* HAVE_XKB */
+	int use_xkb;
+	int major_version, minor_version;
+	
+	using_xkb = FALSE;
+
+	major_version = XkbMajorVersion;
+	minor_version = XkbMinorVersion;
+	use_xkb = XkbLibraryVersion(&major_version, &minor_version);
+	/* If I can't use the keyboard extension make
+	 * sure the library doesn't either.
+	 */
+	XkbIgnoreExtension(!use_xkb);
+	{
+		PRIV_SAVE_AREA
+		enter_priv_on();
+		dpy = XOpenDisplay(display_name);
+		leave_priv_setting();
+	}
+	if (dpy == NULL) {
+		return NULL;
+	}
+	if (!use_xkb) {
+		return dpy;
+	}
+	if (!XkbQueryExtension(dpy, NULL, 
+			       &xkb_event_base, &xkb_error_base,
+			       &major_version, &minor_version)) {
+		return dpy;
+	}
+	using_xkb = TRUE;
+#endif
+	return dpy;
+}
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
- * DANG_BEGIN_FUNCTION X_close
+ * DANG_BEGIN_FUNCTION X_init
  *
  * description:
  * Initialize everything X-related.
@@ -579,16 +676,16 @@ int X_init()
 
   /* Open X connection. */
   display_name = config.X_display ? config.X_display : getenv("DISPLAY");
-  {
-    PRIV_SAVE_AREA
-
-    enter_priv_on();
-    display = XOpenDisplay(display_name);
-    leave_priv_setting();
-  }
-
+  display = XKBOpenDisplay(display_name);
   if(display == NULL) {
-    error("X: Can't open display \"%s\"\n", display_name ? display_name : "");
+    error("X: Can't open display \"%s\".\n"
+    "Either the connection was refused and you do not have enough\n"
+    "access rights to connect to your X server or there is\n"
+    "something wrong with the contents of your DISPLAY variable.\n"
+    "If the connection was refused then please consult your system\n"
+    "administator or read the X documentation for a solution\n"
+    "(use xauth, xhost, or ssh -X).\n",
+        display_name ? display_name : "");
     leavedos(99);
   }
 
@@ -690,7 +787,8 @@ int X_init()
   if (our_window) {
     XWMHints wmhint;
     wmhint.window_group = mainwindow;
-    wmhint.flags = WindowGroupHint;
+    wmhint.input = True;
+    wmhint.flags = WindowGroupHint | InputHint;
     XSetWMProperties(display, mainwindow, NULL, NULL, 
       dosemu_argv, dosemu_argc, NULL, &wmhint, &xch);
   }
@@ -734,6 +832,7 @@ int X_init()
 
   if(config.X_mgrab_key) grab_keystring = config.X_mgrab_key;
   if(*grab_keystring) grab_keysym = XStringToKeysym(grab_keystring);
+  freeze_keysym = XStringToKeysym("Pause");
   if(grab_keysym != NoSymbol) {
     X_printf("X: X_init: mouse grabbing enabled, use Ctrl+Mod1+%s to activate\n", grab_keystring);
   }
@@ -1027,6 +1126,10 @@ static void X_keymap_init()
 
   if(s) X_printf("X: X_keymap_init: X server vendor is \"%s\"\n", s);
   if(config.X_keycode == 2 && s) {	/* auto */
+#if defined(HAVE_UNICODE_KEYB) && defined(HAVE_XKB)
+    /* All I need to know is that I'm using the X keyboard extension */
+    config.X_keycode = using_xkb;
+#else
     config.X_keycode = 0;
 
     /*
@@ -1038,14 +1141,17 @@ static void X_keymap_init()
       strstr(s, "The XFree86 Project") ||
       strstr(s, "Xi Graphics")
     ) config.X_keycode = 1;
+#endif /* HAVE_UNICODE_KEYB && HAVE_XKB */
   }
   X_printf(
     "X: X_keymap_init: %susing DOSEMU's internal keycode translation\n",
     config.X_keycode ? "" : "we are not "
   );
+#ifndef HAVE_UNICODE_KEYB
   if(config.X_keycode == 0) {
     X_printf("X: X_keymap_init: '$_layout'-entry will have no effect\n");
   }
+#endif /* ! HAVE_UNICODE_KEYB */
 }
 
 
@@ -1065,8 +1171,51 @@ int X_change_config(unsigned item, void *buf)
   switch(item) {
 
     case X_CHG_TITLE:
-      X_printf("X: X_change_config: win_name = %s\n", (char *) buf);
-      XStoreName(display, mainwindow, buf);
+       /* low-level write */
+       if (buf)
+       {
+         X_printf("X: X_change_config: win_name = %s\n", (char *) buf);
+         XStoreName(display, mainwindow, buf);
+       }
+       /* high-level write (shows name of emulator + running app) */
+       else
+       {
+         char title [X_TITLE_EMUNAME_MAXLEN + X_TITLE_APPNAME_MAXLEN + 5];
+           
+         /* write name of emulator */
+         if (strlen (X_title_emuname))
+           snprintf (title, X_TITLE_EMUNAME_MAXLEN, "%s", X_title_emuname);
+         else
+           snprintf (title, X_TITLE_EMUNAME_MAXLEN, "%s", config.X_title);
+           
+         /* append name of running application (if any) */
+         if (X_title_show_appname && strlen (X_title_appname))
+         {
+           if (strlen (title)) strcat (title, " - ");
+           strcat (title, X_title_appname);
+         }
+           
+         /* now actually change the title of the Window */
+         X_change_config (X_CHG_TITLE, title);
+       }
+       break;
+       
+    case X_CHG_TITLE_EMUNAME:
+      X_printf ("X: X_change_config: emu_name = %s\n", (char *) buf);
+      snprintf (X_title_emuname, X_TITLE_EMUNAME_MAXLEN, "%s", ( char *) buf);
+      X_change_config (X_CHG_TITLE, NULL);
+      break;
+      
+    case X_CHG_TITLE_APPNAME:
+      X_printf ("X: X_change_config: app_name = %s\n", (char *) buf);
+      snprintf (X_title_appname, X_TITLE_APPNAME_MAXLEN, "%s", (char *) buf);
+      X_change_config (X_CHG_TITLE, NULL);
+      break;
+
+    case X_CHG_TITLE_SHOW_APPNAME:
+      X_printf("X: X_change_config: show_appname %i\n", *((int *) buf));
+      X_title_show_appname = *((int *) buf);
+      X_change_config (X_CHG_TITLE, NULL);
       break;
 
     case X_CHG_FONT:
@@ -1343,6 +1492,7 @@ void X_handle_events()
 	  blink_state = TRUE;
 	  blink_count = config.X_blinkrate;
 	  redraw_cursor();
+	  output_byte_8042(port60_buffer | 0x80);
 	  break;
 
 	case DestroyNotify:
@@ -1391,8 +1541,9 @@ void X_handle_events()
     /* Keyboard events */
 
 	case KeyPress:
-          if((e.xkey.state & ControlMask) && (e.xkey.state & Mod1Mask) && grab_keysym != NoSymbol)
-            if(XKeycodeToKeysym(display, e.xkey.keycode, 0) == grab_keysym) {
+          if((e.xkey.state & ControlMask) && (e.xkey.state & Mod1Mask) && grab_keysym != NoSymbol) {
+            KeySym keysym = XKeycodeToKeysym(display, e.xkey.keycode, 0);
+            if(keysym == grab_keysym) {
               if(grab_active ^= 1) {
                 X_printf("X: mouse grab activated\n");
 #ifdef ENABLE_KEYBOARD_GRAB
@@ -1411,8 +1562,15 @@ void X_handle_events()
                 X_set_mouse_cursor(mouse_cursor_visible, mouse_x, mouse_y, w_x_res, w_y_res);
               }
               break;
+            } else if (keysym == freeze_keysym) {
+              if (freeze_active ^= 1) {
+                freeze_dosemu();
+              } else {
+                unfreeze_dosemu();
+              }
+              break;
             }
-
+          }
 	case KeyRelease:
 /* 
       Clears the visible selection if the cursor is inside the selection
@@ -1423,6 +1581,10 @@ void X_handle_events()
 #endif
 	  X_process_key(&e.xkey);
 	  break;
+	case KeymapNotify:
+	  X_process_keys(&e.xkeymap);
+	  break;
+
     /* A keyboard mapping has been changed (e.g., with xmodmap). */
 	case MappingNotify:  
 	  X_printf("X: MappingNotify event\n");
@@ -1531,7 +1693,7 @@ void X_handle_events()
 
   busy = 0;
 #if CONFIG_X_MOUSE  
-  mouse_event();
+  pic_request(PIC_IMOUSE);
 #endif  
 
 #if 0
@@ -2246,9 +2408,11 @@ static void X_modify_mode()
   RemapObject tmp_ro;
 
   if(vga.reconfig.mem) {
-    if(remap_obj.src_mode == MODE_PSEUDO_8 || remap_obj.src_mode == MODE_VGA_X) {
-
-      tmp_ro = remap_init(vga.mem.planes == 1 ? MODE_PSEUDO_8 : MODE_VGA_X, ximage_mode, remap_features);
+    if(remap_obj.src_mode == MODE_PSEUDO_8 || remap_obj.src_mode == MODE_VGA_X || remap_obj.src_mode == MODE_VGA_4) {
+	   if (remap_obj.src_mode == MODE_VGA_4)
+		 tmp_ro = remap_init(MODE_VGA_4, ximage_mode, remap_features);
+	   else
+		 tmp_ro = remap_init(vga.mem.planes == 1 ? MODE_PSEUDO_8 : MODE_VGA_X, ximage_mode, remap_features); 
       *tmp_ro.dst_color_space = X_csd;
       tmp_ro.dst_image = ximage->data;
       tmp_ro.src_resize(&tmp_ro, vga.width, vga.height, vga.scan_len);
@@ -3184,6 +3348,7 @@ void end_selection()
 }
 
 
+#ifndef HAVE_UNICODE_TRANSLATION
 /*
  * Send selection data to other window.
  */
@@ -3220,6 +3385,9 @@ void send_selection(Time time, Window requestor, Atom target, Atom property)
 	  break;
 	case 0x80 ... 0xff: 
           switch (config.term_charset) {
+             case CHARSET_KOI8:
+                sel_text_latin[i]=dos_to_koi8[sel_text_latin[i] - 0x80];
+                break;
              case CHARSET_LATIN1:
                 sel_text_latin[i]=dos_to_latin1[sel_text_latin[i] - 0x80];
                 break;
@@ -3248,6 +3416,76 @@ void send_selection(Time time, Window requestor, Atom target, Atom property)
   }
   XSendEvent(display, requestor, False, 0, &e);
 }
+#else /* HAVE_UNICODE_TRANSLATION */
+void send_selection(Time time, Window requestor, Atom target, Atom property)
+{
+	size_t sel_text_bytes, sel_text_latin_bytes, sel_text_latin_space;
+	u_char *sel_text_ptr, *sel_text_latin, *sel_text_latin_ptr;
+	struct char_set_state paste_state;
+	struct char_set_state video_state; /* must not have any... */
+	
+	struct char_set *paste_charset = trconfig.paste_charset;
+	struct char_set *video_charset = trconfig.video_mem_charset;
+	XEvent e;
+
+	init_charset_state(&video_state, video_charset);
+	init_charset_state(&paste_state, paste_charset);
+	
+	e.xselection.type = SelectionNotify;
+	e.xselection.selection = XA_PRIMARY;
+	e.xselection.requestor = requestor;
+	e.xselection.time = time;
+	if (sel_text == NULL) {
+		X_printf("X: Window 0x%lx requested selection, but it's empty!\n",   
+			(unsigned long) requestor);
+		e.xselection.property = None;
+	}
+	else if ((target == XA_STRING) || (target == compound_text_atom)) {
+		X_printf("X: selection (dos): %s\n",sel_text);   
+		e.xselection.target = target;
+		sel_text_bytes = strlen(sel_text);
+		sel_text_latin_bytes = sel_text_bytes + 100;
+		sel_text_latin_space = sel_text_latin_bytes;
+		sel_text_latin = malloc(sel_text_latin_bytes);
+		sel_text_ptr = sel_text;
+		sel_text_latin_ptr = sel_text_latin;
+		while(sel_text_bytes) {
+			t_unicode symbol;
+			size_t result;
+			/* If we hit any run with what we have */
+			result = charset_to_unicode(&video_state, &symbol,
+				sel_text_ptr, sel_text_bytes);
+			if (result == -1) break;
+			sel_text_bytes -= result;
+			sel_text_ptr += result;
+			result = unicode_to_charset(&paste_state, 
+				symbol,
+				sel_text_latin_ptr, sel_text_latin_space);
+			if (result == -1) break;
+			sel_text_latin_ptr += result;
+			sel_text_latin_space -= result;
+		}
+		*sel_text_latin_ptr = '\0';
+		X_printf("X: selection (latin): %s\n",sel_text_latin);  
+		XChangeProperty(display, requestor, property, target, 8, PropModeReplace, 
+			sel_text_latin, strlen(sel_text_latin));
+		e.xselection.property = property;
+		X_printf("X: Selection sent to window 0x%lx as %s\n", 
+			(unsigned long) requestor, (target==XA_STRING)?"string":"compound_text");
+		free(sel_text_latin);
+	}
+	else
+	{
+		e.xselection.property = None;
+		X_printf("X: Window 0x%lx requested unknown selection format %ld\n",
+			(unsigned long) requestor, (unsigned long) target);
+	}
+	cleanup_charset_state(&video_state);
+	cleanup_charset_state(&paste_state);
+	XSendEvent(display, requestor, False, 0, &e);
+}
+#endif /* HAVE_UNICODE_TRANSLATION */
+
 #endif /* CONFIG_X_SELECTION */
 
 
