@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -43,11 +43,10 @@
 #ifdef __linux__
 #if GLIBC_VERSION_CODE >= 2000
 #include <sys/vt.h>
-#include <sys/kd.h>
 #else
-#include <linux/vt.h>
-#include <linux/kd.h>
+#include "Linux/vt.h"
 #endif
+#include <sys/kd.h>
 #endif
 
 #define INIT_C2TRAP
@@ -134,8 +133,10 @@ void check_console(void) {
     major = chkbuf.st_rdev >> 8;
     minor = chkbuf.st_rdev & 0xff;
 
+    c_printf("major = %d minor = %d\n",
+	    major, minor);
     /* console major num is 4, minor 64 is the first serial line */
-    if (can_do_root_stuff && (major == 4) && (minor < 64)) {
+    if ((major == 4) && (minor < 64)) {
        scr_state.console_no = minor;
        config.console=1;
     }
