@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -334,14 +334,18 @@ volatile static int my_fault = 0;
 
 static void chk_ro()
 {
+#ifdef X86_EMULATOR
   if (config.cpuemu<1) {
+#endif
     volatile char *p = (char *) 0xa0077;
     my_fault = 1;
     *p = 0x99;
     if(my_fault) {
       vga_deb_map("vga_emu_fault: IS WRITABLE\n");
     }
+#ifdef X86_EMULATOR
   }
+#endif
   my_fault = 0;
 }
 #endif
@@ -791,7 +795,7 @@ void Logical_VGA_write(unsigned offset, unsigned char value)
  *
  */     
 
-int vga_emu_fault(struct sigcontext_struct *scp)
+int vga_emu_fault(struct sigcontext_struct *scp, int pmode)
 {
   int i, j;
   unsigned page_fault, vga_page = 0, u, access_type, lin_addr;
@@ -821,7 +825,7 @@ int vga_emu_fault(struct sigcontext_struct *scp)
   }
 
   vga_deb_map("vga_emu_fault: %s%s access to %s region, address 0x%05x, page 0x%x, vga page 0x%x\n",
-    in_dpmi ? "dpmi " : "", access_type? "write" : "read",
+    pmode ? "dpmi " : "", access_type? "write" : "read",
     txt1[i], lin_addr, page_fault, vga_page
   );
 
@@ -831,7 +835,7 @@ int vga_emu_fault(struct sigcontext_struct *scp)
     (unsigned) REG(cs), (unsigned) REG(eip)
   );
 
-  if(in_dpmi) {
+  if(pmode) {
 #if DEBUG_MAP >= 1
     cs_ip = (unsigned char *) (dpmi_GetSegmentBaseAddress(_cs) + _eip);
 #endif
@@ -853,9 +857,15 @@ int vga_emu_fault(struct sigcontext_struct *scp)
 
   if(i == VGAEMU_MAX_MAPPINGS) {
     if(page_fault >= 0xa0 && page_fault < 0xc0) {	/* unmapped VGA area */
-      u = instr_len(SEG_ADR((unsigned char *), cs, ip));
-      LWORD(eip) += u;
-      if(!in_dpmi && u) {
+      if (pmode) {
+        u = instr_len((unsigned char *)SEL_ADR(_cs, _eip));
+        _eip += u;
+      }
+      else {
+        u = instr_len(SEG_ADR((unsigned char *), cs, ip));
+        LWORD(eip) += u;
+      }
+      if(u) {
         vga_msg("vga_emu_fault: attempted write to unmapped area (cs:ip += %u)\n", u);
       }
       else {
@@ -865,9 +875,15 @@ int vga_emu_fault(struct sigcontext_struct *scp)
       return True;
     }
     else if(page_fault >= 0xc0 && page_fault < (0xc0 + vgaemu_bios.pages)) {	/* ROM area */
-      u = instr_len(SEG_ADR((unsigned char *), cs, ip));
-      LWORD(eip) += u;
-      if(!in_dpmi && u) {
+      if (pmode) {
+        u = instr_len((unsigned char *)SEL_ADR(_cs, _eip));
+        _eip += u;
+      }
+      else {
+        u = instr_len(SEG_ADR((unsigned char *), cs, ip));
+        LWORD(eip) += u;
+      }
+      if(u) {
         vga_msg("vga_emu_fault: attempted write to ROM area (cs:ip += %u)\n", u);
       }
       else {
@@ -902,7 +918,7 @@ int vga_emu_fault(struct sigcontext_struct *scp)
        * while we are using X.  Leave the display page read/write-protected
        * so that each instruction that accesses it can be trapped and
        * simulated. */
-        instr_emu(scp);
+        instr_emu(scp, pmode);
     }
   }
   return True;
@@ -2472,23 +2488,33 @@ void vgaemu_adj_cfg(unsigned what, unsigned msg)
       u = vga.seq.data[4] & 4 ? 0 : 1;
       u = vga.seq.data[4] & 8 ? 2 : u;
       vga.seq.addr_mode = u;
-      if(u != u0 && vga.mode_type == P8) {	/* ++HACK++ */
-        u1 = vga.seq.addr_mode == 0 ? 4 : 1;
-        vga.scan_len = (vga.scan_len * vga.mem.planes) / u1;
-        if(u1 != vga.mem.planes) {
-          vga.mem.planes = u1; vga.reconfig.mem = 1;   
-          vga_msg("vgaemu_adj_cfg: mem reconfig (%u planes)\n", u1);
-        }
+      u1 = vga.seq.addr_mode == 0 ? 4 : 1;
+      if(u1 != vga.mem.planes) {
+        vga.mem.planes = u1; vga.reconfig.mem = 1;   
+        vga_msg("vgaemu_adj_cfg: mem reconfig (%u planes)\n", u1);
       }
-      if(msg || u != u0) vga_msg("vgaemu_adj_cfg: seq.addr_mode = %s\n", txt1[u]);
+      if(msg || u != u0) vga_msg("vgaemu_adj_cfg: seq.addr_mode = %s\n", txt1[u]); 
     break;
 
     case CFG_CRTC_ADDR_MODE:
       u0 = vga.crtc.addr_mode;
+      u1 = vga.scan_len;
       u = vga.crtc.data[0x17] & 0x40 ? 0 : 1;
       u = vga.crtc.data[0x14] & 0x40 ? 2 : u;
       vga.crtc.addr_mode = u;
-      if(msg || u != u0) vga_msg("vgaemu_adj_cfg: crtc.addr_mode = %s\n", txt2[u]);
+      vga.display_start = (vga.crtc.data[0x0d] + (vga.crtc.data[0x0c] << 8)) << 
+	      vga.crtc.addr_mode;
+      /* this shift should really be a rotation, depending on mode control bit 5 */
+      screen_adr = SCREEN_ADR(0) + vga.display_start;
+      vga.crtc.cursor_location =  (vga.crtc.data[0x0f] + (vga.crtc.data[0x0e] << 8)) <<
+              vga.crtc.addr_mode;
+      vga.scan_len = vga.crtc.data[0x13] << (vga.crtc.addr_mode + 1);
+      if (vga.scan_len == 0) vga.scan_len = 160; 
+      cursor_row = (vga.crtc.cursor_location - vga.display_start) / vga.scan_len;
+      cursor_col = ((vga.crtc.cursor_location - vga.display_start) % vga.scan_len) / 2;
+      if (u1 != vga.scan_len) vga.reconfig.mem = 1;
+      if(msg || u != u0) vga_msg("vgaemu_adj_cfg: crtc.addr_mode = %s, scan_len = %d\n",
+	txt2[u], vga.scan_len);
     break;
 
     case CFG_CRTC_HEIGHT:
@@ -2531,8 +2557,10 @@ void vgaemu_adj_cfg(unsigned what, unsigned msg)
       vga_msg("vgaemu_adj_cfg: vertical_display_end = %d\n", vertical_display_end);
       vga_msg("vgaemu_adj_cfg: vertical_multiplier = %d\n", vertical_multiplier);
       vga_msg("vgaemu_adj_cfg: height = %d\n", height);
-      vga.height = height;
-      vga.reconfig.display = 1;
+      if (vga.height != height) {
+        vga.height = height;
+        vga.reconfig.display = 1;
+      }
       break;
     }
     case CFG_CRTC_WIDTH:
@@ -2552,8 +2580,7 @@ void vgaemu_adj_cfg(unsigned what, unsigned msg)
       horizontal_blanking_end = vga.crtc.data[0x3] & 0xF;
       horizontal_retrace_start = vga.crtc.data[0x4];
       horizontal_retrace_end = vga.crtc.data[0x5] & 0xF;
-      multiplier = 8; /* Can this be 9? */
-      /* Do I need to do something special for text modes? */
+      multiplier = (vga.mode_class == TEXT) ? 9 : (vga.mode_type == P8) ? 4 : 8;
       width = horizontal_display_end * multiplier;
       vga_msg("vgaemu_adj_cfg: horizontal_total = %d\n", horizontal_total);
       vga_msg("vgaemu_adj_cfg: horizontal_retrace_start = %d\n", horizontal_retrace_start);
@@ -2563,8 +2590,10 @@ void vgaemu_adj_cfg(unsigned what, unsigned msg)
       vga_msg("vgaemu_adj_cfg: horizontal_display_end = %d\n", horizontal_display_end);
       vga_msg("vgaemu_adj_cfg: multiplier = %d\n", multiplier);
       vga_msg("vgaemu_adj_cfg: width = %d\n", width);
-      vga.width = width;
-      vga.reconfig.display = 1;
+      if (vga.width != width) {
+	 vga.width = width;
+	 vga.reconfig.display = 1;
+      }
       break;
     }
     default:
