@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,21 +27,13 @@
 /* function definitions */
 
 
-#ifndef  C_RUN_IRQS
-#define _INLINE_ extern __inline__
-#define _STATIC_
-#else
-#define _INLINE_ static __inline__
-#define _STATIC_ static
-#endif
-
-_STATIC_ int find_bit(void * addr);
-_STATIC_ long atomic_inc(long * addr);
-_STATIC_ long atomic_dec(long * addr);
-_STATIC_ int set_bit(int nr, void * addr);
-_STATIC_ int clear_bit(int nr, void * addr);
-_STATIC_ int test_bit(int nr, void * addr);
-_STATIC_ int pic0_to_emu(char flags);
+static int find_bit(unsigned long int word);
+static long atomic_inc(long * addr);
+static long atomic_dec(long * addr);
+static int set_bit(int nr, void * addr);
+static int clear_bit(int nr, void * addr);
+static int test_bit(int nr, void * addr);
+static int pic0_to_emu(char flags);
 
 /*
  * These have to be done with inline assembly: that way the bit-setting is
@@ -60,27 +52,29 @@ struct __dummy {
 #define ADDR (*(struct __dummy *) addr)
 
 /* JLS's stuff */
-/* find_bit returns the bit number of the lowest bit that's set */
-/* it doesn't need to be atomic, but it's much faster than using c */
-
-_INLINE_ int
-find_bit(void *addr)
+/*
+ * find_bit returns the bit number of the lowest bit that's set
+ * Returns -1 if no one exists.
+ */
+static __inline__ int
+find_bit(unsigned long int word)
 {
-    int             bitno;
-
-    __asm__         __volatile__("bsfl %1,%0"
-				 :"=r"           (bitno):"m"(ADDR));
-    return bitno;
+       int result = -1; /* value to return on error */
+       __asm__("bsfl %2,%0"
+               :"=r" (result) /* output */
+               :"0" (result), "r" (word)); /* input */
+       return result;
 }
+
 /* atomic increment flag and decrement flag operations */
-_INLINE_ long int
+static __inline__ long int
 atomic_inc(long *addr)
 {
     __asm__         __volatile__("incl %1"
 				 :"=m"           (ADDR):"0"(ADDR));
     return *addr;
 }
-_INLINE_ long int
+static __inline__ long int
 atomic_dec(long *addr)
 {
     __asm__         __volatile__("decl %1"
@@ -88,7 +82,7 @@ atomic_dec(long *addr)
     return *addr;
 }
 
-_INLINE_ int
+static __inline__ int
 pic0_to_emu(char flags)
 {
     /* This function maps pic0 bits to their positions in priority order */
@@ -102,14 +96,14 @@ pic0_to_emu(char flags)
     /* bit 2 (cascade int) is used to mask/unmask pic1 (Larry)          */
 
     long            result;
-    __asm__         __volatile__("movzbl %1,%0\n\t
-				 shll $13, %0\n\t
-				 sarw $7, %w0\n\t
-				 shrl $5, %0 " 
-				 :"=r"           (result):"r"(flags));
+    __asm__         __volatile__("movzbl %1,%0\n\t"
+				 "shll $13, %0\n\t"
+				 "sarw $7, %w0\n\t"
+				 "shrl $5, %0 " \
+				 :"=r"(result):"r"(flags));
     return result;
 }
-_INLINE_ long
+static __inline__ long
 emu_to_pic0(long flags)
 {
     /*
@@ -121,10 +115,10 @@ emu_to_pic0(long flags)
     /* move bits 7654 3xxx xxxx 210x to xxxx xxxx 7654 3210          */
     /* where 76543210 are final 8 bits and x = don't care            */
 
-    __asm__         __volatile__("shll $6,%0\n\t
-				 shlw $7, %w0\n\t
-				 shrl $14, %0 " 
-				 :"=r"           (flags):"0"(flags));
+    __asm__         __volatile__("shll $6,%0\n\t"
+				 "shlw $7, %w0\n\t"
+				 "shrl $14, %0 " 
+				 :"=r"(flags):"0"(flags));
     return flags;
 }
 
@@ -132,25 +126,25 @@ emu_to_pic0(long flags)
  * Linus' stuff follows - except each __inline__ had an extern in front of
  * it
  */
-_INLINE_ int
+static __inline__ int
 set_bit(int nr, void *addr)
 {
     int             oldbit;
 
     __asm__         __volatile__("btsl %2,%1\n\tsbbl %0,%0"
-				 :"=r"           (oldbit), "=m"(ADDR)
-				 :"r"            (nr));
+				 :"=r"(oldbit), "=m"(ADDR)
+				 :"r"(nr));
     return oldbit;
 }
 
-_INLINE_ int
+static __inline__ int
 clear_bit(int nr, void *addr)
 {
     int             oldbit;
 
     __asm__         __volatile__("btrl %2,%1\n\tsbbl %0,%0"
-				 :"=r"           (oldbit), "=m"(ADDR)
-				 :"r"            (nr));
+				 :"=r"(oldbit), "=m"(ADDR)
+				 :"r"(nr));
     return oldbit;
 }
 
@@ -158,14 +152,14 @@ clear_bit(int nr, void *addr)
  * This routine doesn't need to be atomic, but it's faster to code it this
  * way.
  */
-_INLINE_ int
+static __inline__ int
 test_bit(int nr, void *addr)
 {
     int             oldbit;
 
     __asm__         __volatile__("btl %2,%1\n\tsbbl %0,%0"
-				 :"=r"           (oldbit)
-				 :"m"            (ADDR), "r"(nr));
+				 :"=r"(oldbit)
+				 :"m"(ADDR), "r"(nr));
     return oldbit;
 }
 
