@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -26,7 +26,7 @@
   #error "Sorry, wrong keyboard code version for this DOSEMU version"
 #endif
 
-#if defined(HAVE_KBD_UNICODE) && (HAVE_KBD_UNICODE > 1)
+#if defined(HAVE_UNICODE_KEYB) && (HAVE_UNICODE_KEYB != 2)
   #error "Sorry, wrong unicode keyboard code version for this DOSEMU version"
 #endif
 
@@ -42,6 +42,7 @@
 #if 1 /* Set to 1 to use Silly Interrupt generator */
 #define SIG 1
 typedef struct { int fd; int irq; } SillyG_t;
+extern SillyG_t *SillyG;
 #endif
 
 #define inline __inline__
@@ -153,14 +154,6 @@ void getKeys(void);
 
      int set_ioperm(int, int, int);
 
-#ifdef X86_EMULATOR
-EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
-#else
-EXTERN struct debug_flags d INIT({0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});
-#endif
-#ifdef DONT_DEBUG_BOOT
-EXTERN struct debug_flags d_save;
-#endif
  
 EXTERN u_char in_sighandler, in_ioctl;
 /* one-entry queue ;-( for ioctl's */
@@ -300,10 +293,11 @@ typedef struct vesamode_type_struct {
        boolean X_keycode;	/* use keycode field of event structure */
        boolean exitearly;
        int     realcpu;
-       boolean mathco, smp;
+       boolean mathco, smp, cpummx;
        boolean ipxsup;
        boolean vnet;
-       boolean keybint;
+       char   *netdev;
+       boolean pktdrv;
        boolean dosbanner;
        boolean allowvideoportaccess;
        boolean emuretrace;
@@ -320,7 +314,6 @@ typedef struct vesamode_type_struct {
        boolean bootdisk;	/* Special bootdisk defined */
        int  fastfloppy;
        char *emusys;		/* map CONFIG.SYS to CONFIG.EMU */
-       char *emubat;		/* map AUTOEXEC.BAT to AUTOEXEC.EMU */
        char *emuini;           /* map system.ini to  system.EMU */
 
        u_short speaker;		/* 0 off, 1 native, 2 emulated */
@@ -349,6 +342,8 @@ typedef struct vesamode_type_struct {
        				   (bitmask, bit3..15 ==> IRQ3 .. IRQ15) */
 
        struct keytable_entry *keytable;
+       struct keytable_entry *altkeytable;
+       int toggle_mask;
 
        unsigned short detach;
        unsigned char *debugout;
@@ -381,12 +376,23 @@ typedef struct vesamode_type_struct {
        int features[1];
 
        /* Sound emulation */
+       int sound; 
        __u16 sb_base;
        __u8 sb_dma;
        __u8 sb_irq;
        char *sb_dsp;
        char *sb_mixer;
        __u16 mpu401_base;
+
+       /* joystick */
+       char *joy_device[2];
+       
+       /* range for joystick axis values */
+       int joy_dos_min;		/* must be > 0 */
+       int joy_dos_max;		/* avoid setting this to > 250 */
+       
+       int joy_granularity;	/* the higher, the less sensitive - for wobbly joysticks */
+       int joy_latency;		/* delay between nonblocking linux joystick reads */
      }
 
 config_t;
@@ -417,15 +423,12 @@ EXTERN __inline__ void parent_nextscan(void);
 EXTERN __inline__ void disk_close(void);
 EXTERN void cpu_setup(void);
 EXTERN void real_run_int(int);
-#ifdef USE_NEW_INT
-  #define run_int do_int
-#else
-  #define run_int real_run_int
-#endif
+#define run_int do_int
 EXTERN int mfs_redirector(void);
 EXTERN void int10(void);
 EXTERN void int13(u_char);
 EXTERN void int14(u_char);
+EXTERN void int16(u_char);
 EXTERN void int17(u_char);
 EXTERN void io_select(fd_set);
 EXTERN int pd_receive_packet(void);
@@ -564,6 +567,7 @@ EXTERN int fatalerr INIT(0);
 EXTERN int running_DosC INIT(0);
 EXTERN int dosc_interface(void);
 
+EXTERN void dump_config_status(void *);
 EXTERN void signal_init(void);
 EXTERN void device_init(void);
 EXTERN void hardware_setup(void);
