@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -8,17 +8,18 @@
 #define INT_H
 
 #include "extern.h"
+#include <sys/types.h> /* for u_char */
+#include <time.h> /* for time_t */
 
 
 EXTERN void *interrupt_function[0x100];
 EXTERN unsigned int  check_date INIT(0);
 EXTERN time_t        start_time;
-EXTERN u_char in_sigsegv INIT(0);
-EXTERN u_char in_sighandler INIT(0);	/* so I know to not use non-reentrant
-					 * syscalls like ioctl() :-( */
-EXTERN u_char ignore_segv INIT(0);      /* ignore sigsegv's */
 
 void do_int(int);
+void fake_int(int, int);
+void fake_call(int, int);
+void fake_pusha(void);
 void setup_interrupts(void);
 void version_init(void);
 
