@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -26,7 +26,10 @@
 extern void iodev_init(void);
 extern void iodev_reset(void);
 extern void iodev_term(void);
-
+extern void iodev_register(char *name, void (*init_func)(void), 
+	void (*reset_func)(void), void (*term_func)(void));
+void iodev_unregiseter(char *name);
+void iodev_add_device(char *dev_name);
 /*******************************************************************
  * Programmable Interrupt Timer (PIT) chip                         *
  *******************************************************************/
