@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,7 +27,7 @@
 #define CISH(x) x
 #endif
 #else
-# error "Sorry, a.out format no longer is supported"
+# error "Sorry, a.out format is no longer supported"
 #endif
 
 /* split segment 0xf000 into two region, 0xf0000 to 0xf7fff is read-write */
@@ -53,6 +53,8 @@
 #define OUTB_OFF	0x1000
 #define OUTB_ADD	(u_short *)((OUTB_SEG << 4) + OUTB_OFF)
 
+#define Pause_SEG	(BIOSSEG)
+#define Pause_OFF	0x3000
 
 /* The packet driver has some code in this segment which needs to be */
 /* at BIOSSEG.  therefore use BIOSSEG and compensate for the offset. */
@@ -67,6 +69,8 @@
 #define Mouse_ROUTINE_OFF  0x2140
 #define Mouse_ADD      ((Mouse_SEG << 4)+Mouse_OFF)
 #define Mouse_ROUTINE  ((Mouse_SEG << 4)+Mouse_ROUTINE_OFF)
+#define EOI_OFF		0x2140
+#define POPA_IRET_OFF	0x20f4
 
 /* intercept-stub for dosdebugger (catches INT21/AX=4B00 */
 #define DBGload_SEG BIOSSEG
