@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -18,6 +18,7 @@
 #ifndef MHPDBG_H
 #define MHPDBG_H
 
+#include <stdarg.h>
 #include "extern.h"
 
 #if 0  /* now defined in include/vm86plus */
@@ -46,6 +47,8 @@ void mhp_input();
 void mhp_close();
 void mhp_printf(const char *,...);
 int mhp_getaxlist_value(int v, int mask);
+
+int vmhp_log_intercept(int flg, const char *fmt, va_list args);
  
 #define MHP_BUFFERSIZE 8192
 struct mhpdbg
@@ -103,6 +106,7 @@ struct mhpdbg_4bpar
 
 #define PAR4b_addr(x) ((char *)( ((long)mhpdbgc.bpload_par->x.seg << 4) \
                         + mhpdbgc.bpload_par->x.off ))
+
 struct mhpdbgc
 {
    int stopped;
