@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -9,6 +9,9 @@
 
 /* will include bitypes.h, linux/posix_types.h and asm/types.h */
 #include <sys/types.h>
+#if GLIBC_VERSION_CODE >= 2000
+#  include "Asm/types.h"
+#endif
 
 #ifndef FALSE
 #define FALSE 0
@@ -21,9 +24,16 @@
 typedef u_int64_t hitimer_t;
 
 typedef union {
-  hitimer_t td;
+  u_int64_t td;
   struct { u_int32_t tl,th; } t;
-} hitimer_u;
+} u_int64_u;
+
+typedef union {
+  int64_t td;
+  struct { int32_t tl,th; } t;
+} int64_u;
+
+typedef u_int64_u hitimer_u;
 
 typedef unsigned char      Boolean;
 typedef u_int8_t           Bit8u;   /* type of 8 bit unsigned quantity */
