@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -36,14 +36,15 @@ char *assemble_path(char *dir, char *file, int append_pid);
 char *mkdir_under(char *basedir, char *dir, int append_pid);
 char *get_path_in_HOME(char *path);
 char *get_dosemu_local_home(void);
+long long libless_llseek(int fd, long long offset, int origin);
 
 /* returns y = sqrt(x), for y*y beeing a power of 2 below x
  */
 static __inline__ int power_of_2_sqrt(int val)
 {
 	register int res;
-	__asm__ __volatile__("
-		bsrl	%2,%0
+	__asm__ __volatile__(" \
+		bsrl	%2,%0\n \
 	" : "=r" (res) : "0" ((int)-1), "r" (val) );
 	if (res <0) return 0;
 	return 1 << (res >> 1);
