@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -43,8 +43,13 @@ extern void scr_state_init(void);
 #define CURSOR_END(c)   ((int)((cshape*)&c)->end)
 #define NO_CURSOR 0x0100
 
-#define MAX_COLUMNS 132
-#define MAX_LINES 60
+/* 
+ * xterm allows to set virtually any screen size, but DOS applications
+ * and the BIOS really can't handle more than these numbers of rows and
+ * columns (128*255*2 < 65535)
+ */
+#define MAX_COLUMNS 255
+#define MAX_LINES 128
 
 #if 0
 #define SCREEN_ADR(s)	((ushort *)(virt_text_base + (s*TEXT_SIZE)))
@@ -200,6 +205,7 @@ EXTERN int v_8514_base INIT(0);
 #define CHARSET_FULLIBM	3
 #define CHARSET_LATIN1  4
 #define CHARSET_LATIN2  5
+#define CHARSET_KOI8    6
 
 
 /* Color set defines */
