@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -223,11 +223,11 @@ VERB	 *
 	 * ( look at do_page_fault in arch/i386/mm/fault.c how it
 	 *   treats growing down of stack )
  * SIDOC_END_FUNCTION */
-	__asm__ volatile ("
-		xchgl	%%esp,%%eax
-		movb	$1,(%%esp)
-		xchgl	%%esp,%%eax
-		": : "a" (address)
+	__asm__ volatile (" \
+		xchgl	%%esp,%%eax\n \
+		movb	$1,(%%esp)\n \
+		xchgl	%%esp,%%eax" \
+		: : "a" (address)
 	);
 	return (char *)address;
 }
