@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2002 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -30,18 +30,35 @@
 #include <sys/types.h>
 #include <sys/fcntl.h>
 #include <stdio.h>
+#include <errno.h>
 #ifdef __linux__
-#include <asm/types.h>
-#include <linux/genhd.h>
+  #include "Linux/genhd.h"
 #endif
 
 #include "config.h"
-#ifdef NEED_LLSEEK_PROTOTYPE
   /* well, if we don't have llseek prototype,
    * we most likely won't have __loff_t too, hence using long long
    */
-  extern long long llseek (int fd, long long offset, int origin);
-#endif
+long long libless_llseek(int fd, long long offset, int origin)
+{
+  long long result;
+  int offlo = offset;
+  int offhi = offset >> 32;
+  int __res;
+  __asm__ __volatile__(
+    "int $0x80\n"
+    :"=a" (__res)
+    :"a" ((int)140), "b" (fd), "c" (offhi), "d" (offlo),
+                    "S" ((int)&result), "D" (origin)
+  );
+  errno = 0;
+  if (__res < 0) {
+    errno = -__res;
+    result = -1;
+  }
+  return result;
+}
+#define llseek libless_llseek
 
 #define SECTOR_SIZE	512
 #define EXT_MAGIC	5	/* sys_ind for an extended partition */
